if (typeof __instrumentor_map__ === 'undefined') {
__instrumentor_map__ = {};
}
if (typeof __instrumentor_map__.ic7db2deba541272893290ef6a69a34ee === 'undefined') {
__instrumentor_map__.ic7db2deba541272893290ef6a69a34ee = {"filetag":"ic7db2deba541272893290ef6a69a34ee","filename":"lib/instrumentor.js","source":"/*global require:true, module:true */\n\nvar crypto\t\t= require(\"crypto\"),\n\tesprima\t\t= require(\"esprima\"),\n\tescodegen\t= require(\"escodegen\"),\n\testraverse\t= require(\"estraverse\"),\n\n\t// Maps for instrumenting\n\tnoReplaceMap\t\t= require(\"./instrumentor-config\").noReplaceMap,\n\tallowedReplacements\t= require(\"./instrumentor-config\").allowedReplacements,\n\tnoPrependMap\t\t= require(\"./instrumentor-config\").noPrependMap,\n\tallowedPrepend\t\t= require(\"./instrumentor-config\").allowedPrepend;\n\n// Local definitions for functions in this file\nvar callExpression,\n\texpressionStatement,\n\tfileID,\n\tprependInstrumentorMap,\n\tpreprocessNode;\n\n/*\n\tPublic: Instrument a string of JS code using the SteamShovel instrumentor.\n\tThis function executes synchronously.\n\n\tdata\t\t\t-\tThe raw JavaScript code to instrument\n\tfilename\t\t-\tThe filename of the code in question, for inclusion in\n\t\t\t\t\t\tthe instrument map\n\tincorporateMap\t-\tWhether to include the map (defaults to true, but useful\n\t\t\t\t\t\tfor testing output without the additional noise of the\n\t\t\t\t\t\tmap.)\n\treturnAsObject\t-\tReturn the data as an object containing the instrumented\n\t\t\t\t\t\tcode and the source map.\n\n\tReturns a string containing the instrumented code.\n\n\tExamples\n\n\t\tcode = instrumentCode(\"function(a) { return a; }\", \"myfile.js\");\n\n*/\n\nmodule.exports =\n\tfunction instrumentCode(data, filename, incorporateMap, returnAsObject) {\n\n\tfilename = filename || \"implicit-filename\";\n\tincorporateMap = incorporateMap === false ? false : true;\n\n\tvar esprimaOptions = {loc: true, range: true, raw: true, comment: true},\n\t\tast = esprima.parse(data, esprimaOptions),\n\t\tfiletag = fileID(filename),\n\t\tcomments = ast.comments,\n\n\t\t// State and storage\n\t\tid = 0,\n\t\tcode = null,\n\t\tsourceMap = {};\n\n\t// Add metadata\n\tsourceMap.filetag = filetag;\n\tsourceMap.filename = filename;\n\n\t// Add raw sourcecode\n\tsourceMap.source = String(data);\n\n\t// Bucket for instruments\n\tsourceMap.instruments = {};\n\n\t// Add comment ranges to sourceMap\n\tsourceMap.comments = comments.map(function(comment) {\n\t\treturn comment.range;\n\t});\n\n\tfunction sourceMapAdd(id, node, line) {\n\t\tif (sourceMap.instruments[id])\n\t\t\tthrow new Error(\"Instrument error: Instrument already exists!\");\n\n\t\tsourceMap.instruments[id] = {\n\t\t\t\"loc\": node.loc,\n\t\t\t\"range\": node.range,\n\t\t\t\"results\": [],\n\t\t\t\"stack\": node.stackPath,\n\t\t\t\"type\": node.type,\n\t\t\t\"line\": !!line\n\t\t};\n\t}\n\n\t// Process AST\n\testraverse.replace(ast, {\n\n\t\t// Enter is where we mark nodes as noReplace\n\t\t// which prevents bugs around weird edge cases, which I'll probably\n\t\t// discover as time goes on.\n\t\t//\n\t\t// We also record a stack path to be provided to the instrumentor\n\t\t// that lets us know our place in the AST when the instrument recorder\n\t\t// is called.\n\n\t\tenter: preprocessNode,\n\n\t\t// Leave is where we replace the actual nodes.\n\n\t\tleave: function (node) {\n\n\t\t\t// Does this node have a body? If so we can replace its contents.\n\t\t\tif (node.body && node.body.length) {\n\t\t\t\tnode.body =\n\t\t\t\t\t[].slice.call(node.body, 0)\n\t\t\t\t\t.reduce(function(body, node) {\n\n\t\t\t\t\t\tif (!~allowedPrepend.indexOf(node.type))\n\t\t\t\t\t\t\treturn body.concat(node);\n\n\t\t\t\t\t\tid++;\n\t\t\t\t\t\tsourceMapAdd(id, node, true);\n\n\t\t\t\t\t\treturn body.concat(\n\t\t\t\t\t\t\texpressionStatement(\n\t\t\t\t\t\t\t\tcallExpression(filetag, id)\n\t\t\t\t\t\t\t),\n\t\t\t\t\t\t\tnode\n\t\t\t\t\t\t);\n\n\t\t\t\t\t}, []);\n\t\t\t}\n\n\t\t\tif (node.noReplace)\n\t\t\t\treturn;\n\n\t\t\t// If we're allowed to replace the node,\n\t\t\t// replace it with a Call Expression.\n\n\t\t\tif (~allowedReplacements.indexOf(node.type))\n\t\t\t\treturn (\n\t\t\t\t\tid ++,\n\t\t\t\t\tsourceMapAdd(id, node, false),\n\t\t\t\t\tcallExpression(filetag, id, node)\n\t\t\t\t);\n\t\t}\n\t});\n\n\tcode = escodegen.generate(ast);\n\n\tif (incorporateMap)\n\t\tcode = prependInstrumentorMap(code, sourceMap);\n\n\tif (returnAsObject)\n\t\treturn { \"code\": code, \"map\": sourceMap };\n\n\treturn code;\n};\n\nmodule.exports.withmap = function(data, filename, incorporateMap) {\n\treturn module.exports(data, filename, incorporateMap, true);\n};\n\n/*\n\tPublic: Preprocess a node to save the AST stack/path into the node, and\n\tto mark whether its children should be replaced or not.\n\n\tdata\t\t\t-\tThe AST node as represented by Esprima.\n\n\tReturns null.\n\n\tExamples\n\n\t\tpreprocessNode(astNode);\n\n*/\n\npreprocessNode = module.exports.preprocessNode =\n\tfunction preprocessNode(node) {\n\n\tif (!node.stackPath)\n\t\tnode.stackPath = [node.type];\n\n\t// Now mark a path to the node.\n\tObject.keys(node).forEach(function(nodekey) {\n\t\tvar prop = node[nodekey];\n\n\t\tfunction saveStack(prop) {\n\t\t\t// This property most likely isn't a node.\n\t\t\tif (!prop || typeof prop !== \"object\" || !prop.type) return;\n\n\t\t\tprop.stackPath = node.stackPath.concat(prop.type);\n\t\t}\n\n\t\tif (Array.isArray(prop))\n\t\t\tprop.forEach(saveStack);\n\n\t\tsaveStack(prop);\n\t});\n\n\tvar nodeRule = noReplaceMap[node.type];\n\n\tif (!nodeRule) return;\n\n\t// Convert the rule to an array so we can handle it using\n\t// the same logic.\n\t//\n\t// Strings and arrays just wholesale exclude the child nodes\n\t// of the current node where they match.\n\n\tif (nodeRule instanceof String)\n\t\tnodeRule = [nodeRule];\n\n\tif (nodeRule instanceof Array) {\n\t\tnodeRule.forEach(function(property) {\n\t\t\tif (!node[property]) return;\n\n\t\t\tif (node[property] instanceof Array)\n\t\t\t\treturn node[property].forEach(function(item) {\n\t\t\t\t\titem.noReplace = true;\n\t\t\t\t});\n\n\t\t\tnode[property].noReplace = true;\n\t\t});\n\t}\n\n\t// Whereas this more verbose object style allows\n\t// exclusion based on subproperty matches.\n\n\tif (nodeRule instanceof Object) {\n\t\tObject.keys(nodeRule).forEach(function(property) {\n\t\t\tif (!node[property]) return;\n\n\t\t\tvar exclude =\n\t\t\t\tObject\n\t\t\t\t\t.keys(nodeRule[property])\n\t\t\t\t\t.reduce(function(prev, cur) {\n\t\t\t\t\t\tif (!prev) return prev;\n\t\t\t\t\t\treturn (\n\t\t\t\t\t\t\tnode[property][cur] ===\n\t\t\t\t\t\t\t\tnodeRule[property][cur]);\n\t\t\t\t\t}, true);\n\n\t\t\tif (exclude) node[property].noReplace = true;\n\t\t});\n\t}\n};\n\n/*\n\tPublic: Generates a replacement CallExpression for a given node,\n\twhich invokes the instrument function, passing an id and the resolved\n\tvalue of the original expression as the first and second arguments.\n\n\tid\t\t\t\t-\tA unique ID string to mark the call\n\tnode\t\t\t-\tThe AST node as represented by Esprima.\n\n\tReturns an object representing a replacement node.\n\n\tExamples\n\n\t\tcallExpression(id, astNode);\n\n*/\n\ncallExpression = module.exports.callExpression =\n\tfunction callExpression(filetag, id, node) {\n\n\tvar callArgs = [\n\t\t{\n\t\t\t\"type\": \"Literal\",\n\t\t\t\"value\": filetag\n\t\t},\n\t\t{\n\t\t\t\"type\": \"Literal\",\n\t\t\t\"value\": id\n\t\t}\n\t];\n\n\tif (!!node) callArgs.push(node);\n\n\treturn {\n\t\t\"type\": \"CallExpression\",\n\t\t\"callee\": {\n\t\t\t\"type\": \"Identifier\",\n\t\t\t\"name\": \"instrumentor_record\"\n\t\t},\n\t\t\"arguments\": callArgs\n\t};\n};\n\n/*\n\tPublic: Generates an ExpressionStatement AST node, containing an input\n\texpression.\n\n\tnode\t-\tThe expression's AST node as represented by Esprima.\n\n\tReturns an object representing a replacement node.\n\n\tExamples\n\n\t\texpressionStatement(astNode);\n\n*/\n\nexpressionStatement = module.exports.expressionStatement =\n\tfunction expressionStatement(node) {\n\treturn {\n\t\t\"type\": \"ExpressionStatement\",\n\t\t\"expression\": node\n\t};\n};\n\n/*\n\tPublic: Given a string representation of JavaScript sourcecode and a map\n\tcontaining metainformation about the instrument probes themselves, this\n\tfunction prepends a string/source representation of the instrument probes,\n\tas well as the sourcecode of the function that performs the instrumentation\n\titself.\n\n\tcode\t\t\t-\tThe source to which the instrument map should be\n\t\t\t\t\t\tappended\n\tsourceMap\t\t-\tA map of information about each instrument probe and\n\t\t\t\t\t\tcontaining the sourcecode of the file.\n\n\tReturns the instrumented JavaScript sourcecode to be executed or written\n\tto disk.\n\n\tExamples\n\n\t\tprependInstrumentorMap(code, sourceMap);\n\n*/\nprependInstrumentorMap = module.exports.prependInstrumentorMap =\n\tfunction prependInstrumentorMap(code, sourceMap) {\n\n\tvar filetag = sourceMap.filetag,\n\t\tmap = JSON.stringify(sourceMap);\n\n\treturn [\n\n\t\t// Map initialisation\n\t\t\"if (typeof __instrumentor_map__ === 'undefined') {\",\n\t\t\t\"__instrumentor_map__ = {};\",\n\t\t\"}\",\n\n\t\t// Map for key\n\t\t\"if (typeof __instrumentor_map__.\" + filetag + \" === 'undefined') {\",\n\t\t\t\"__instrumentor_map__.\" + filetag + \" = \" + map + \";\",\n\t\t\"}\",\n\n\t\t// Instrumentor\n\t\trequire(\"./instrumentor-record\").toString(),\n\n\t\t// Include the code\n\t\tcode\n\n\t].join(\"\\n\");\n};\n\n/*\n\tPublic: Given a string, this function will generate a string md5 digest\n\twhich can be safely used barely in JS literal context.\n\n\tfilename\t\t-\tThe filename to digest\n\n\tReturns the digest string.\n\n\tExamples\n\n\t\tfileID(\"myfile.js\");\n\n*/\n\nfileID = module.exports.fileID =\n\tfunction fileID(id) {\n\n\treturn \"i\" + crypto.createHash(\"md5\").update(id).digest(\"hex\");\n};","instruments":{"1":{"loc":{"start":{"line":3,"column":14},"end":{"line":3,"column":21}},"range":[53,60],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"2":{"loc":{"start":{"line":3,"column":22},"end":{"line":3,"column":30}},"range":[61,69],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"3":{"loc":{"start":{"line":3,"column":14},"end":{"line":3,"column":31}},"range":[53,70],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"4":{"loc":{"start":{"line":4,"column":12},"end":{"line":4,"column":19}},"range":[84,91],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"5":{"loc":{"start":{"line":4,"column":20},"end":{"line":4,"column":29}},"range":[92,101],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"6":{"loc":{"start":{"line":4,"column":12},"end":{"line":4,"column":30}},"range":[84,102],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"7":{"loc":{"start":{"line":5,"column":13},"end":{"line":5,"column":20}},"range":[117,124],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"8":{"loc":{"start":{"line":5,"column":21},"end":{"line":5,"column":32}},"range":[125,136],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"9":{"loc":{"start":{"line":5,"column":13},"end":{"line":5,"column":33}},"range":[117,137],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"10":{"loc":{"start":{"line":6,"column":14},"end":{"line":6,"column":21}},"range":[153,160],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"11":{"loc":{"start":{"line":6,"column":22},"end":{"line":6,"column":34}},"range":[161,173],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"12":{"loc":{"start":{"line":6,"column":14},"end":{"line":6,"column":35}},"range":[153,174],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"13":{"loc":{"start":{"line":9,"column":17},"end":{"line":9,"column":24}},"range":[221,228],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"14":{"loc":{"start":{"line":9,"column":25},"end":{"line":9,"column":48}},"range":[229,252],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"15":{"loc":{"start":{"line":9,"column":17},"end":{"line":9,"column":62}},"range":[221,266],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"16":{"loc":{"start":{"line":10,"column":23},"end":{"line":10,"column":30}},"range":[291,298],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"17":{"loc":{"start":{"line":10,"column":31},"end":{"line":10,"column":54}},"range":[299,322],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"18":{"loc":{"start":{"line":10,"column":23},"end":{"line":10,"column":75}},"range":[291,343],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"19":{"loc":{"start":{"line":11,"column":17},"end":{"line":11,"column":24}},"range":[362,369],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"20":{"loc":{"start":{"line":11,"column":25},"end":{"line":11,"column":48}},"range":[370,393],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"21":{"loc":{"start":{"line":11,"column":17},"end":{"line":11,"column":62}},"range":[362,407],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"22":{"loc":{"start":{"line":12,"column":19},"end":{"line":12,"column":26}},"range":[428,435],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"23":{"loc":{"start":{"line":12,"column":27},"end":{"line":12,"column":50}},"range":[436,459],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"24":{"loc":{"start":{"line":12,"column":19},"end":{"line":12,"column":66}},"range":[428,475],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"25":{"loc":{"start":{"line":45,"column":12},"end":{"line":45,"column":20}},"range":[1376,1384],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","LogicalExpression","Identifier"],"type":"Identifier","line":false},"26":{"loc":{"start":{"line":45,"column":24},"end":{"line":45,"column":43}},"range":[1388,1407],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","LogicalExpression","Literal"],"type":"Literal","line":false},"27":{"loc":{"start":{"line":45,"column":12},"end":{"line":45,"column":43}},"range":[1376,1407],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","LogicalExpression"],"type":"LogicalExpression","line":false},"28":{"loc":{"start":{"line":45,"column":1},"end":{"line":45,"column":43}},"range":[1365,1407],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"29":{"loc":{"start":{"line":46,"column":18},"end":{"line":46,"column":32}},"range":[1427,1441],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"30":{"loc":{"start":{"line":46,"column":37},"end":{"line":46,"column":42}},"range":[1446,1451],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"31":{"loc":{"start":{"line":46,"column":18},"end":{"line":46,"column":42}},"range":[1427,1451],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"32":{"loc":{"start":{"line":46,"column":45},"end":{"line":46,"column":50}},"range":[1454,1459],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Literal"],"type":"Literal","line":false},"33":{"loc":{"start":{"line":46,"column":53},"end":{"line":46,"column":57}},"range":[1462,1466],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Literal"],"type":"Literal","line":false},"34":{"loc":{"start":{"line":46,"column":18},"end":{"line":46,"column":57}},"range":[1427,1466],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression"],"type":"ConditionalExpression","line":false},"35":{"loc":{"start":{"line":46,"column":1},"end":{"line":46,"column":57}},"range":[1410,1466],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"36":{"loc":{"start":{"line":48,"column":28},"end":{"line":48,"column":32}},"range":[1497,1501],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"37":{"loc":{"start":{"line":48,"column":41},"end":{"line":48,"column":45}},"range":[1510,1514],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"38":{"loc":{"start":{"line":48,"column":52},"end":{"line":48,"column":56}},"range":[1521,1525],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"39":{"loc":{"start":{"line":48,"column":67},"end":{"line":48,"column":71}},"range":[1536,1540],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"40":{"loc":{"start":{"line":48,"column":22},"end":{"line":48,"column":72}},"range":[1491,1541],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression"],"type":"ObjectExpression","line":false},"41":{"loc":{"start":{"line":49,"column":22},"end":{"line":49,"column":26}},"range":[1565,1569],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"42":{"loc":{"start":{"line":49,"column":28},"end":{"line":49,"column":42}},"range":[1571,1585],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"43":{"loc":{"start":{"line":49,"column":8},"end":{"line":49,"column":43}},"range":[1551,1586],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"44":{"loc":{"start":{"line":50,"column":12},"end":{"line":50,"column":18}},"range":[1600,1606],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"45":{"loc":{"start":{"line":50,"column":19},"end":{"line":50,"column":27}},"range":[1607,1615],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"46":{"loc":{"start":{"line":50,"column":12},"end":{"line":50,"column":28}},"range":[1600,1616],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"47":{"loc":{"start":{"line":51,"column":13},"end":{"line":51,"column":25}},"range":[1631,1643],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"48":{"loc":{"start":{"line":54,"column":7},"end":{"line":54,"column":8}},"range":[1676,1677],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"49":{"loc":{"start":{"line":55,"column":9},"end":{"line":55,"column":13}},"range":[1688,1692],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"50":{"loc":{"start":{"line":56,"column":14},"end":{"line":56,"column":16}},"range":[1708,1710],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression"],"type":"ObjectExpression","line":false},"51":{"loc":{"start":{"line":59,"column":21},"end":{"line":59,"column":28}},"range":[1751,1758],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","Identifier"],"type":"Identifier","line":false},"52":{"loc":{"start":{"line":59,"column":1},"end":{"line":59,"column":28}},"range":[1731,1758],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"53":{"loc":{"start":{"line":60,"column":22},"end":{"line":60,"column":30}},"range":[1782,1790],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","Identifier"],"type":"Identifier","line":false},"54":{"loc":{"start":{"line":60,"column":1},"end":{"line":60,"column":30}},"range":[1761,1790],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"55":{"loc":{"start":{"line":63,"column":20},"end":{"line":63,"column":26}},"range":[1836,1842],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"56":{"loc":{"start":{"line":63,"column":27},"end":{"line":63,"column":31}},"range":[1843,1847],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"57":{"loc":{"start":{"line":63,"column":20},"end":{"line":63,"column":32}},"range":[1836,1848],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"58":{"loc":{"start":{"line":63,"column":1},"end":{"line":63,"column":32}},"range":[1817,1848],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"59":{"loc":{"start":{"line":66,"column":25},"end":{"line":66,"column":27}},"range":[1903,1905],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"60":{"loc":{"start":{"line":66,"column":1},"end":{"line":66,"column":27}},"range":[1879,1905],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"61":{"loc":{"start":{"line":70,"column":9},"end":{"line":70,"column":22}},"range":[2008,2021],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","MemberExpression"],"type":"MemberExpression","line":false},"62":{"loc":{"start":{"line":70,"column":2},"end":{"line":70,"column":23}},"range":[2001,2022],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"63":{"loc":{"start":{"line":69,"column":35},"end":{"line":71,"column":2}},"range":[1979,2025],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"64":{"loc":{"start":{"line":69,"column":22},"end":{"line":71,"column":3}},"range":[1966,2026],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"65":{"loc":{"start":{"line":69,"column":1},"end":{"line":71,"column":3}},"range":[1945,2026],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"66":{"loc":{"start":{"line":74,"column":6},"end":{"line":74,"column":31}},"range":[2076,2101],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","MemberExpression"],"type":"MemberExpression","line":false},"67":{"loc":{"start":{"line":75,"column":13},"end":{"line":75,"column":18}},"range":[2116,2121],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","ThrowStatement","NewExpression","Identifier"],"type":"Identifier","line":false},"68":{"loc":{"start":{"line":75,"column":19},"end":{"line":75,"column":65}},"range":[2122,2168],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","ThrowStatement","NewExpression","Literal"],"type":"Literal","line":false},"69":{"loc":{"start":{"line":75,"column":9},"end":{"line":75,"column":66}},"range":[2112,2169],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","ThrowStatement","NewExpression"],"type":"NewExpression","line":false},"70":{"loc":{"start":{"line":78,"column":10},"end":{"line":78,"column":18}},"range":[2214,2222],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"71":{"loc":{"start":{"line":79,"column":12},"end":{"line":79,"column":22}},"range":[2236,2246],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"72":{"loc":{"start":{"line":80,"column":14},"end":{"line":80,"column":16}},"range":[2262,2264],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression","Property","ArrayExpression"],"type":"ArrayExpression","line":false},"73":{"loc":{"start":{"line":81,"column":12},"end":{"line":81,"column":26}},"range":[2278,2292],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"74":{"loc":{"start":{"line":82,"column":11},"end":{"line":82,"column":20}},"range":[2305,2314],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"75":{"loc":{"start":{"line":83,"column":11},"end":{"line":83,"column":17}},"range":[2327,2333],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression","Property","UnaryExpression"],"type":"UnaryExpression","line":false},"76":{"loc":{"start":{"line":77,"column":30},"end":{"line":84,"column":3}},"range":[2202,2337],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"77":{"loc":{"start":{"line":77,"column":2},"end":{"line":84,"column":3}},"range":[2174,2337],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"78":{"loc":{"start":{"line":74,"column":2},"end":{"line":75,"column":67}},"range":[2072,2170],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"79":{"loc":{"start":{"line":77,"column":2},"end":{"line":84,"column":4}},"range":[2174,2338],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"80":{"loc":{"start":{"line":88,"column":20},"end":{"line":88,"column":23}},"range":[2379,2382],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"81":{"loc":{"start":{"line":98,"column":9},"end":{"line":98,"column":23}},"range":[2707,2721],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"82":{"loc":{"start":{"line":105,"column":7},"end":{"line":105,"column":16}},"range":[2878,2887],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","MemberExpression"],"type":"MemberExpression","line":false},"83":{"loc":{"start":{"line":105,"column":20},"end":{"line":105,"column":36}},"range":[2891,2907],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","MemberExpression"],"type":"MemberExpression","line":false},"84":{"loc":{"start":{"line":105,"column":7},"end":{"line":105,"column":36}},"range":[2878,2907],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"85":{"loc":{"start":{"line":107,"column":19},"end":{"line":107,"column":28}},"range":[2946,2955],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"86":{"loc":{"start":{"line":107,"column":30},"end":{"line":107,"column":31}},"range":[2957,2958],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"87":{"loc":{"start":{"line":110,"column":35},"end":{"line":110,"column":44}},"range":[3032,3041],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression","UnaryExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"88":{"loc":{"start":{"line":110,"column":10},"end":{"line":110,"column":45}},"range":[3007,3042],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"89":{"loc":{"start":{"line":111,"column":26},"end":{"line":111,"column":30}},"range":[3070,3074],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"90":{"loc":{"start":{"line":111,"column":14},"end":{"line":111,"column":31}},"range":[3058,3075],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"91":{"loc":{"start":{"line":113,"column":6},"end":{"line":113,"column":10}},"range":[3084,3088],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","UpdateExpression"],"type":"UpdateExpression","line":false},"92":{"loc":{"start":{"line":114,"column":6},"end":{"line":114,"column":18}},"range":[3096,3108],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"93":{"loc":{"start":{"line":114,"column":19},"end":{"line":114,"column":21}},"range":[3109,3111],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"94":{"loc":{"start":{"line":114,"column":23},"end":{"line":114,"column":27}},"range":[3113,3117],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"95":{"loc":{"start":{"line":114,"column":29},"end":{"line":114,"column":33}},"range":[3119,3123],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"96":{"loc":{"start":{"line":114,"column":6},"end":{"line":114,"column":34}},"range":[3096,3124],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"97":{"loc":{"start":{"line":117,"column":7},"end":{"line":117,"column":26}},"range":[3160,3179],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"98":{"loc":{"start":{"line":118,"column":8},"end":{"line":118,"column":22}},"range":[3189,3203],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"99":{"loc":{"start":{"line":118,"column":23},"end":{"line":118,"column":30}},"range":[3204,3211],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"100":{"loc":{"start":{"line":118,"column":32},"end":{"line":118,"column":34}},"range":[3213,3215],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"101":{"loc":{"start":{"line":118,"column":8},"end":{"line":118,"column":35}},"range":[3189,3216],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression"],"type":"CallExpression","line":false},"102":{"loc":{"start":{"line":117,"column":7},"end":{"line":119,"column":8}},"range":[3160,3225],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression"],"type":"CallExpression","line":false},"103":{"loc":{"start":{"line":120,"column":7},"end":{"line":120,"column":11}},"range":[3234,3238],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"104":{"loc":{"start":{"line":116,"column":13},"end":{"line":121,"column":7}},"range":[3140,3246],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"105":{"loc":{"start":{"line":110,"column":6},"end":{"line":111,"column":32}},"range":[3003,3076],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"106":{"loc":{"start":{"line":113,"column":6},"end":{"line":113,"column":11}},"range":[3084,3089],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"107":{"loc":{"start":{"line":114,"column":6},"end":{"line":114,"column":35}},"range":[3096,3125],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"108":{"loc":{"start":{"line":116,"column":6},"end":{"line":121,"column":8}},"range":[3133,3247],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"109":{"loc":{"start":{"line":108,"column":13},"end":{"line":123,"column":6}},"range":[2973,3255],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"110":{"loc":{"start":{"line":123,"column":8},"end":{"line":123,"column":10}},"range":[3257,3259],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"111":{"loc":{"start":{"line":107,"column":5},"end":{"line":123,"column":11}},"range":[2932,3260],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"112":{"loc":{"start":{"line":106,"column":4},"end":{"line":123,"column":11}},"range":[2915,3260],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"113":{"loc":{"start":{"line":106,"column":4},"end":{"line":123,"column":12}},"range":[2915,3261],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"114":{"loc":{"start":{"line":126,"column":7},"end":{"line":126,"column":21}},"range":[3275,3289],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","MemberExpression"],"type":"MemberExpression","line":false},"115":{"loc":{"start":{"line":132,"column":36},"end":{"line":132,"column":45}},"range":[3426,3435],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","UnaryExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"116":{"loc":{"start":{"line":132,"column":7},"end":{"line":132,"column":46}},"range":[3397,3436],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"117":{"loc":{"start":{"line":134,"column":5},"end":{"line":134,"column":10}},"range":[3456,3461],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","UpdateExpression"],"type":"UpdateExpression","line":false},"118":{"loc":{"start":{"line":135,"column":5},"end":{"line":135,"column":17}},"range":[3468,3480],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"119":{"loc":{"start":{"line":135,"column":18},"end":{"line":135,"column":20}},"range":[3481,3483],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"120":{"loc":{"start":{"line":135,"column":22},"end":{"line":135,"column":26}},"range":[3485,3489],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"121":{"loc":{"start":{"line":135,"column":28},"end":{"line":135,"column":33}},"range":[3491,3496],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Literal"],"type":"Literal","line":false},"122":{"loc":{"start":{"line":135,"column":5},"end":{"line":135,"column":34}},"range":[3468,3497],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression"],"type":"CallExpression","line":false},"123":{"loc":{"start":{"line":136,"column":5},"end":{"line":136,"column":19}},"range":[3504,3518],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"124":{"loc":{"start":{"line":136,"column":20},"end":{"line":136,"column":27}},"range":[3519,3526],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"125":{"loc":{"start":{"line":136,"column":29},"end":{"line":136,"column":31}},"range":[3528,3530],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"126":{"loc":{"start":{"line":136,"column":33},"end":{"line":136,"column":37}},"range":[3532,3536],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"127":{"loc":{"start":{"line":136,"column":5},"end":{"line":136,"column":38}},"range":[3504,3537],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression"],"type":"CallExpression","line":false},"128":{"loc":{"start":{"line":105,"column":3},"end":{"line":124,"column":4}},"range":[2874,3266],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"129":{"loc":{"start":{"line":126,"column":3},"end":{"line":127,"column":11}},"range":[3271,3302],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"130":{"loc":{"start":{"line":132,"column":3},"end":{"line":137,"column":6}},"range":[3393,3544],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"131":{"loc":{"start":{"line":102,"column":9},"end":{"line":138,"column":3}},"range":[2783,3548],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","FunctionExpression"],"type":"FunctionExpression","line":false},"132":{"loc":{"start":{"line":88,"column":25},"end":{"line":139,"column":2}},"range":[2384,3551],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"133":{"loc":{"start":{"line":88,"column":1},"end":{"line":139,"column":3}},"range":[2360,3552],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"134":{"loc":{"start":{"line":141,"column":27},"end":{"line":141,"column":30}},"range":[3582,3585],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"135":{"loc":{"start":{"line":141,"column":8},"end":{"line":141,"column":31}},"range":[3563,3586],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"136":{"loc":{"start":{"line":141,"column":1},"end":{"line":141,"column":31}},"range":[3556,3586],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"137":{"loc":{"start":{"line":143,"column":5},"end":{"line":143,"column":19}},"range":[3594,3608],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"138":{"loc":{"start":{"line":144,"column":9},"end":{"line":144,"column":31}},"range":[3619,3641],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"139":{"loc":{"start":{"line":144,"column":32},"end":{"line":144,"column":36}},"range":[3642,3646],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"140":{"loc":{"start":{"line":144,"column":38},"end":{"line":144,"column":47}},"range":[3648,3657],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"141":{"loc":{"start":{"line":144,"column":9},"end":{"line":144,"column":48}},"range":[3619,3658],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"142":{"loc":{"start":{"line":144,"column":2},"end":{"line":144,"column":48}},"range":[3612,3658],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"143":{"loc":{"start":{"line":146,"column":5},"end":{"line":146,"column":19}},"range":[3666,3680],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"144":{"loc":{"start":{"line":147,"column":19},"end":{"line":147,"column":23}},"range":[3701,3705],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"145":{"loc":{"start":{"line":147,"column":32},"end":{"line":147,"column":41}},"range":[3714,3723],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"146":{"loc":{"start":{"line":147,"column":9},"end":{"line":147,"column":43}},"range":[3691,3725],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"147":{"loc":{"start":{"line":149,"column":8},"end":{"line":149,"column":12}},"range":[3736,3740],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"148":{"loc":{"start":{"line":45,"column":1},"end":{"line":45,"column":44}},"range":[1365,1408],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"149":{"loc":{"start":{"line":46,"column":1},"end":{"line":46,"column":58}},"range":[1410,1467],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"150":{"loc":{"start":{"line":48,"column":1},"end":{"line":56,"column":17}},"range":[1470,1711],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"151":{"loc":{"start":{"line":59,"column":1},"end":{"line":59,"column":29}},"range":[1731,1759],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"152":{"loc":{"start":{"line":60,"column":1},"end":{"line":60,"column":31}},"range":[1761,1791],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"153":{"loc":{"start":{"line":63,"column":1},"end":{"line":63,"column":33}},"range":[1817,1849],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"154":{"loc":{"start":{"line":66,"column":1},"end":{"line":66,"column":28}},"range":[1879,1906],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"155":{"loc":{"start":{"line":69,"column":1},"end":{"line":71,"column":4}},"range":[1945,2027],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"156":{"loc":{"start":{"line":73,"column":1},"end":{"line":85,"column":2}},"range":[2030,2341],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration"],"type":"FunctionDeclaration","line":true},"157":{"loc":{"start":{"line":88,"column":1},"end":{"line":139,"column":4}},"range":[2360,3553],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"158":{"loc":{"start":{"line":141,"column":1},"end":{"line":141,"column":32}},"range":[3556,3587],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"159":{"loc":{"start":{"line":143,"column":1},"end":{"line":144,"column":49}},"range":[3590,3659],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"160":{"loc":{"start":{"line":146,"column":1},"end":{"line":147,"column":44}},"range":[3662,3726],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"161":{"loc":{"start":{"line":149,"column":1},"end":{"line":149,"column":13}},"range":[3729,3741],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"162":{"loc":{"start":{"line":43,"column":1},"end":{"line":150,"column":1}},"range":[1289,3743],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"163":{"loc":{"start":{"line":42,"column":0},"end":{"line":150,"column":1}},"range":[1271,3743],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"164":{"loc":{"start":{"line":153,"column":23},"end":{"line":153,"column":27}},"range":[3837,3841],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"165":{"loc":{"start":{"line":153,"column":29},"end":{"line":153,"column":37}},"range":[3843,3851],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"166":{"loc":{"start":{"line":153,"column":39},"end":{"line":153,"column":53}},"range":[3853,3867],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"167":{"loc":{"start":{"line":153,"column":55},"end":{"line":153,"column":59}},"range":[3869,3873],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Literal"],"type":"Literal","line":false},"168":{"loc":{"start":{"line":153,"column":8},"end":{"line":153,"column":60}},"range":[3822,3874],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"169":{"loc":{"start":{"line":153,"column":1},"end":{"line":153,"column":61}},"range":[3815,3875],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"170":{"loc":{"start":{"line":152,"column":25},"end":{"line":154,"column":1}},"range":[3771,3877],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"171":{"loc":{"start":{"line":152,"column":0},"end":{"line":154,"column":1}},"range":[3746,3877],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"172":{"loc":{"start":{"line":173,"column":5},"end":{"line":173,"column":20}},"range":[4212,4227],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"173":{"loc":{"start":{"line":174,"column":20},"end":{"line":174,"column":29}},"range":[4249,4258],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","ArrayExpression","MemberExpression"],"type":"MemberExpression","line":false},"174":{"loc":{"start":{"line":174,"column":19},"end":{"line":174,"column":30}},"range":[4248,4259],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"175":{"loc":{"start":{"line":174,"column":2},"end":{"line":174,"column":30}},"range":[4231,4259],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"176":{"loc":{"start":{"line":177,"column":13},"end":{"line":177,"column":17}},"range":[4308,4312],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"177":{"loc":{"start":{"line":178,"column":13},"end":{"line":178,"column":26}},"range":[4355,4368],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"178":{"loc":{"start":{"line":182,"column":7},"end":{"line":182,"column":12}},"range":[4453,4458],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","LogicalExpression","LogicalExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"179":{"loc":{"start":{"line":182,"column":16},"end":{"line":182,"column":27}},"range":[4462,4473],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","LogicalExpression","LogicalExpression","BinaryExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"180":{"loc":{"start":{"line":182,"column":32},"end":{"line":182,"column":40}},"range":[4478,4486],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","LogicalExpression","LogicalExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"181":{"loc":{"start":{"line":182,"column":16},"end":{"line":182,"column":40}},"range":[4462,4486],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","LogicalExpression","LogicalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"182":{"loc":{"start":{"line":182,"column":7},"end":{"line":182,"column":40}},"range":[4453,4486],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","LogicalExpression","LogicalExpression"],"type":"LogicalExpression","line":false},"183":{"loc":{"start":{"line":182,"column":44},"end":{"line":182,"column":54}},"range":[4490,4500],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","LogicalExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"184":{"loc":{"start":{"line":182,"column":7},"end":{"line":182,"column":54}},"range":[4453,4500],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"185":{"loc":{"start":{"line":184,"column":42},"end":{"line":184,"column":51}},"range":[4553,4562],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"186":{"loc":{"start":{"line":184,"column":20},"end":{"line":184,"column":52}},"range":[4531,4563],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"187":{"loc":{"start":{"line":184,"column":3},"end":{"line":184,"column":52}},"range":[4514,4563],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"188":{"loc":{"start":{"line":182,"column":3},"end":{"line":182,"column":63}},"range":[4449,4509],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"189":{"loc":{"start":{"line":184,"column":3},"end":{"line":184,"column":53}},"range":[4514,4564],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"190":{"loc":{"start":{"line":187,"column":20},"end":{"line":187,"column":24}},"range":[4590,4594],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"191":{"loc":{"start":{"line":187,"column":6},"end":{"line":187,"column":25}},"range":[4576,4595],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","CallExpression"],"type":"CallExpression","line":false},"192":{"loc":{"start":{"line":188,"column":16},"end":{"line":188,"column":25}},"range":[4613,4622],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"193":{"loc":{"start":{"line":188,"column":3},"end":{"line":188,"column":26}},"range":[4600,4623],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"194":{"loc":{"start":{"line":190,"column":2},"end":{"line":190,"column":11}},"range":[4628,4637],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"195":{"loc":{"start":{"line":190,"column":12},"end":{"line":190,"column":16}},"range":[4638,4642],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"196":{"loc":{"start":{"line":190,"column":2},"end":{"line":190,"column":17}},"range":[4628,4643],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"197":{"loc":{"start":{"line":178,"column":2},"end":{"line":178,"column":27}},"range":[4344,4369],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"198":{"loc":{"start":{"line":180,"column":2},"end":{"line":185,"column":3}},"range":[4373,4568],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration"],"type":"FunctionDeclaration","line":true},"199":{"loc":{"start":{"line":187,"column":2},"end":{"line":188,"column":27}},"range":[4572,4624],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"200":{"loc":{"start":{"line":190,"column":2},"end":{"line":190,"column":18}},"range":[4628,4644],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"201":{"loc":{"start":{"line":177,"column":27},"end":{"line":191,"column":2}},"range":[4322,4647],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"202":{"loc":{"start":{"line":177,"column":1},"end":{"line":191,"column":3}},"range":[4296,4648],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"203":{"loc":{"start":{"line":193,"column":16},"end":{"line":193,"column":39}},"range":[4667,4690],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"204":{"loc":{"start":{"line":195,"column":5},"end":{"line":195,"column":14}},"range":[4698,4707],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"205":{"loc":{"start":{"line":203,"column":5},"end":{"line":203,"column":13}},"range":[4911,4919],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"206":{"loc":{"start":{"line":203,"column":25},"end":{"line":203,"column":31}},"range":[4931,4937],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"207":{"loc":{"start":{"line":203,"column":5},"end":{"line":203,"column":31}},"range":[4911,4937],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"208":{"loc":{"start":{"line":204,"column":14},"end":{"line":204,"column":22}},"range":[4953,4961],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","ArrayExpression","Identifier"],"type":"Identifier","line":false},"209":{"loc":{"start":{"line":204,"column":13},"end":{"line":204,"column":23}},"range":[4952,4962],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"210":{"loc":{"start":{"line":204,"column":2},"end":{"line":204,"column":23}},"range":[4941,4962],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"211":{"loc":{"start":{"line":206,"column":5},"end":{"line":206,"column":13}},"range":[4970,4978],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"212":{"loc":{"start":{"line":206,"column":25},"end":{"line":206,"column":30}},"range":[4990,4995],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"213":{"loc":{"start":{"line":206,"column":5},"end":{"line":206,"column":30}},"range":[4970,4995],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"214":{"loc":{"start":{"line":208,"column":7},"end":{"line":208,"column":22}},"range":[5046,5061],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"215":{"loc":{"start":{"line":210,"column":7},"end":{"line":210,"column":21}},"range":[5079,5093],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"216":{"loc":{"start":{"line":210,"column":33},"end":{"line":210,"column":38}},"range":[5105,5110],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"217":{"loc":{"start":{"line":210,"column":7},"end":{"line":210,"column":38}},"range":[5079,5110],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"218":{"loc":{"start":{"line":212,"column":22},"end":{"line":212,"column":26}},"range":[5185,5189],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","Literal"],"type":"Literal","line":false},"219":{"loc":{"start":{"line":212,"column":5},"end":{"line":212,"column":26}},"range":[5168,5189],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"220":{"loc":{"start":{"line":212,"column":5},"end":{"line":212,"column":27}},"range":[5168,5190],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"221":{"loc":{"start":{"line":211,"column":34},"end":{"line":213,"column":5}},"range":[5146,5196],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"222":{"loc":{"start":{"line":211,"column":11},"end":{"line":213,"column":6}},"range":[5123,5197],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"223":{"loc":{"start":{"line":215,"column":30},"end":{"line":215,"column":34}},"range":[5230,5234],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","Literal"],"type":"Literal","line":false},"224":{"loc":{"start":{"line":215,"column":3},"end":{"line":215,"column":34}},"range":[5203,5234],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"225":{"loc":{"start":{"line":208,"column":3},"end":{"line":208,"column":31}},"range":[5042,5070],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"226":{"loc":{"start":{"line":210,"column":3},"end":{"line":213,"column":7}},"range":[5075,5198],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"227":{"loc":{"start":{"line":215,"column":3},"end":{"line":215,"column":35}},"range":[5203,5235],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"228":{"loc":{"start":{"line":207,"column":19},"end":{"line":216,"column":3}},"range":[5018,5239],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"229":{"loc":{"start":{"line":207,"column":2},"end":{"line":216,"column":4}},"range":[5001,5240],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"230":{"loc":{"start":{"line":207,"column":2},"end":{"line":216,"column":5}},"range":[5001,5241],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"231":{"loc":{"start":{"line":222,"column":5},"end":{"line":222,"column":13}},"range":[5346,5354],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"232":{"loc":{"start":{"line":222,"column":25},"end":{"line":222,"column":31}},"range":[5366,5372],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"233":{"loc":{"start":{"line":222,"column":5},"end":{"line":222,"column":31}},"range":[5346,5372],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"234":{"loc":{"start":{"line":223,"column":14},"end":{"line":223,"column":22}},"range":[5390,5398],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"235":{"loc":{"start":{"line":224,"column":7},"end":{"line":224,"column":22}},"range":[5436,5451],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"236":{"loc":{"start":{"line":228,"column":11},"end":{"line":228,"column":29}},"range":[5501,5519],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","MemberExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"237":{"loc":{"start":{"line":230,"column":10},"end":{"line":230,"column":15}},"range":[5566,5571],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"238":{"loc":{"start":{"line":230,"column":24},"end":{"line":230,"column":28}},"range":[5580,5584],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"239":{"loc":{"start":{"line":232,"column":7},"end":{"line":232,"column":26}},"range":[5608,5627],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"240":{"loc":{"start":{"line":233,"column":8},"end":{"line":233,"column":31}},"range":[5640,5663],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"241":{"loc":{"start":{"line":232,"column":7},"end":{"line":233,"column":31}},"range":[5608,5663],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"242":{"loc":{"start":{"line":230,"column":6},"end":{"line":230,"column":29}},"range":[5562,5585],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"243":{"loc":{"start":{"line":231,"column":6},"end":{"line":233,"column":33}},"range":[5592,5665],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"244":{"loc":{"start":{"line":229,"column":13},"end":{"line":234,"column":6}},"range":[5534,5672],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"245":{"loc":{"start":{"line":234,"column":8},"end":{"line":234,"column":12}},"range":[5674,5678],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"246":{"loc":{"start":{"line":227,"column":4},"end":{"line":234,"column":13}},"range":[5483,5679],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"247":{"loc":{"start":{"line":236,"column":7},"end":{"line":236,"column":14}},"range":[5689,5696],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"248":{"loc":{"start":{"line":236,"column":43},"end":{"line":236,"column":47}},"range":[5725,5729],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","Literal"],"type":"Literal","line":false},"249":{"loc":{"start":{"line":236,"column":16},"end":{"line":236,"column":47}},"range":[5698,5729],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"250":{"loc":{"start":{"line":224,"column":3},"end":{"line":224,"column":31}},"range":[5432,5460],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"251":{"loc":{"start":{"line":226,"column":3},"end":{"line":234,"column":14}},"range":[5465,5680],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"252":{"loc":{"start":{"line":236,"column":3},"end":{"line":236,"column":48}},"range":[5685,5730],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"253":{"loc":{"start":{"line":223,"column":32},"end":{"line":237,"column":3}},"range":[5408,5734],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"254":{"loc":{"start":{"line":223,"column":2},"end":{"line":237,"column":4}},"range":[5378,5735],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"255":{"loc":{"start":{"line":223,"column":2},"end":{"line":237,"column":5}},"range":[5378,5736],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"256":{"loc":{"start":{"line":173,"column":1},"end":{"line":174,"column":31}},"range":[4208,4260],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"257":{"loc":{"start":{"line":177,"column":1},"end":{"line":191,"column":4}},"range":[4296,4649],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"258":{"loc":{"start":{"line":193,"column":1},"end":{"line":193,"column":40}},"range":[4652,4691],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"259":{"loc":{"start":{"line":195,"column":1},"end":{"line":195,"column":23}},"range":[4694,4716],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"260":{"loc":{"start":{"line":203,"column":1},"end":{"line":204,"column":24}},"range":[4907,4963],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"261":{"loc":{"start":{"line":206,"column":1},"end":{"line":217,"column":2}},"range":[4966,5244],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"262":{"loc":{"start":{"line":222,"column":1},"end":{"line":238,"column":2}},"range":[5342,5739],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"263":{"loc":{"start":{"line":171,"column":1},"end":{"line":239,"column":1}},"range":[4174,5741],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"264":{"loc":{"start":{"line":170,"column":17},"end":{"line":239,"column":1}},"range":[4141,5741],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression"],"type":"AssignmentExpression","line":false},"265":{"loc":{"start":{"line":170,"column":0},"end":{"line":239,"column":1}},"range":[4124,5741],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"266":{"loc":{"start":{"line":262,"column":11},"end":{"line":262,"column":20}},"range":[6279,6288],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ArrayExpression","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"267":{"loc":{"start":{"line":263,"column":12},"end":{"line":263,"column":19}},"range":[6302,6309],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ArrayExpression","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"268":{"loc":{"start":{"line":261,"column":2},"end":{"line":264,"column":3}},"range":[6266,6313],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ArrayExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"269":{"loc":{"start":{"line":266,"column":11},"end":{"line":266,"column":20}},"range":[6330,6339],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ArrayExpression","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"270":{"loc":{"start":{"line":267,"column":12},"end":{"line":267,"column":14}},"range":[6353,6355],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ArrayExpression","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"271":{"loc":{"start":{"line":265,"column":2},"end":{"line":268,"column":3}},"range":[6317,6359],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ArrayExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"272":{"loc":{"start":{"line":260,"column":16},"end":{"line":269,"column":2}},"range":[6262,6362],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ArrayExpression"],"type":"ArrayExpression","line":false},"273":{"loc":{"start":{"line":271,"column":5},"end":{"line":271,"column":11}},"range":[6370,6376],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"274":{"loc":{"start":{"line":271,"column":27},"end":{"line":271,"column":31}},"range":[6392,6396],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"275":{"loc":{"start":{"line":271,"column":13},"end":{"line":271,"column":32}},"range":[6378,6397],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"276":{"loc":{"start":{"line":274,"column":10},"end":{"line":274,"column":26}},"range":[6420,6436],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"277":{"loc":{"start":{"line":276,"column":11},"end":{"line":276,"column":23}},"range":[6463,6475],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"278":{"loc":{"start":{"line":277,"column":11},"end":{"line":277,"column":32}},"range":[6488,6509],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"279":{"loc":{"start":{"line":275,"column":12},"end":{"line":278,"column":3}},"range":[6450,6513],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","ObjectExpression"],"type":"ObjectExpression","line":false},"280":{"loc":{"start":{"line":279,"column":15},"end":{"line":279,"column":23}},"range":[6530,6538],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"281":{"loc":{"start":{"line":273,"column":8},"end":{"line":280,"column":2}},"range":[6408,6541],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"282":{"loc":{"start":{"line":260,"column":1},"end":{"line":269,"column":3}},"range":[6247,6363],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"283":{"loc":{"start":{"line":271,"column":1},"end":{"line":271,"column":33}},"range":[6366,6398],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"284":{"loc":{"start":{"line":273,"column":1},"end":{"line":280,"column":3}},"range":[6401,6542],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"285":{"loc":{"start":{"line":258,"column":1},"end":{"line":281,"column":1}},"range":[6200,6544],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"286":{"loc":{"start":{"line":257,"column":17},"end":{"line":281,"column":1}},"range":[6167,6544],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression"],"type":"AssignmentExpression","line":false},"287":{"loc":{"start":{"line":257,"column":0},"end":{"line":281,"column":1}},"range":[6150,6544],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"288":{"loc":{"start":{"line":300,"column":10},"end":{"line":300,"column":31}},"range":[6916,6937],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"289":{"loc":{"start":{"line":301,"column":16},"end":{"line":301,"column":20}},"range":[6955,6959],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"290":{"loc":{"start":{"line":299,"column":8},"end":{"line":302,"column":2}},"range":[6904,6962],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"291":{"loc":{"start":{"line":299,"column":1},"end":{"line":302,"column":3}},"range":[6897,6963],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"292":{"loc":{"start":{"line":298,"column":1},"end":{"line":303,"column":1}},"range":[6859,6965],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"293":{"loc":{"start":{"line":297,"column":22},"end":{"line":303,"column":1}},"range":[6821,6965],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression"],"type":"AssignmentExpression","line":false},"294":{"loc":{"start":{"line":297,"column":0},"end":{"line":303,"column":1}},"range":[6799,6965],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"295":{"loc":{"start":{"line":328,"column":15},"end":{"line":328,"column":32}},"range":[7744,7761],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"296":{"loc":{"start":{"line":329,"column":23},"end":{"line":329,"column":32}},"range":[7786,7795],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"297":{"loc":{"start":{"line":329,"column":8},"end":{"line":329,"column":33}},"range":[7771,7796],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"298":{"loc":{"start":{"line":334,"column":2},"end":{"line":334,"column":54}},"range":[7836,7888],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"299":{"loc":{"start":{"line":335,"column":3},"end":{"line":335,"column":31}},"range":[7893,7921],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"300":{"loc":{"start":{"line":336,"column":2},"end":{"line":336,"column":5}},"range":[7925,7928],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"301":{"loc":{"start":{"line":339,"column":2},"end":{"line":339,"column":36}},"range":[7950,7984],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"302":{"loc":{"start":{"line":339,"column":39},"end":{"line":339,"column":46}},"range":[7987,7994],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"303":{"loc":{"start":{"line":339,"column":2},"end":{"line":339,"column":46}},"range":[7950,7994],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"304":{"loc":{"start":{"line":339,"column":49},"end":{"line":339,"column":70}},"range":[7997,8018],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"305":{"loc":{"start":{"line":339,"column":2},"end":{"line":339,"column":70}},"range":[7950,8018],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"306":{"loc":{"start":{"line":340,"column":3},"end":{"line":340,"column":26}},"range":[8023,8046],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","BinaryExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"307":{"loc":{"start":{"line":340,"column":29},"end":{"line":340,"column":36}},"range":[8049,8056],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"308":{"loc":{"start":{"line":340,"column":3},"end":{"line":340,"column":36}},"range":[8023,8056],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"309":{"loc":{"start":{"line":340,"column":39},"end":{"line":340,"column":44}},"range":[8059,8064],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"310":{"loc":{"start":{"line":340,"column":3},"end":{"line":340,"column":44}},"range":[8023,8064],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"311":{"loc":{"start":{"line":340,"column":47},"end":{"line":340,"column":50}},"range":[8067,8070],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"312":{"loc":{"start":{"line":340,"column":3},"end":{"line":340,"column":50}},"range":[8023,8070],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"313":{"loc":{"start":{"line":340,"column":53},"end":{"line":340,"column":56}},"range":[8073,8076],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"314":{"loc":{"start":{"line":340,"column":3},"end":{"line":340,"column":56}},"range":[8023,8076],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"315":{"loc":{"start":{"line":341,"column":2},"end":{"line":341,"column":5}},"range":[8080,8083],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"316":{"loc":{"start":{"line":344,"column":2},"end":{"line":344,"column":9}},"range":[8106,8113],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"317":{"loc":{"start":{"line":344,"column":10},"end":{"line":344,"column":33}},"range":[8114,8137],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","CallExpression","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"318":{"loc":{"start":{"line":344,"column":2},"end":{"line":344,"column":45}},"range":[8106,8149],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","CallExpression"],"type":"CallExpression","line":false},"319":{"loc":{"start":{"line":347,"column":2},"end":{"line":347,"column":6}},"range":[8176,8180],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","ArrayExpression","Identifier"],"type":"Identifier","line":false},"320":{"loc":{"start":{"line":349,"column":8},"end":{"line":349,"column":12}},"range":[8190,8194],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Literal"],"type":"Literal","line":false},"321":{"loc":{"start":{"line":331,"column":8},"end":{"line":349,"column":13}},"range":[7807,8195],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"322":{"loc":{"start":{"line":328,"column":1},"end":{"line":329,"column":34}},"range":[7730,7797],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"323":{"loc":{"start":{"line":331,"column":1},"end":{"line":349,"column":14}},"range":[7800,8196],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"324":{"loc":{"start":{"line":326,"column":1},"end":{"line":350,"column":1}},"range":[7677,8198],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"325":{"loc":{"start":{"line":325,"column":25},"end":{"line":350,"column":1}},"range":[7636,8198],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression"],"type":"AssignmentExpression","line":false},"326":{"loc":{"start":{"line":325,"column":0},"end":{"line":350,"column":1}},"range":[7611,8198],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"327":{"loc":{"start":{"line":369,"column":8},"end":{"line":369,"column":11}},"range":[8504,8507],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","Literal"],"type":"Literal","line":false},"328":{"loc":{"start":{"line":369,"column":32},"end":{"line":369,"column":37}},"range":[8528,8533],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","MemberExpression","CallExpression","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"329":{"loc":{"start":{"line":369,"column":46},"end":{"line":369,"column":48}},"range":[8542,8544],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"330":{"loc":{"start":{"line":369,"column":57},"end":{"line":369,"column":62}},"range":[8553,8558],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Literal"],"type":"Literal","line":false},"331":{"loc":{"start":{"line":369,"column":14},"end":{"line":369,"column":63}},"range":[8510,8559],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression"],"type":"CallExpression","line":false},"332":{"loc":{"start":{"line":369,"column":8},"end":{"line":369,"column":63}},"range":[8504,8559],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"333":{"loc":{"start":{"line":369,"column":1},"end":{"line":369,"column":64}},"range":[8497,8560],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"334":{"loc":{"start":{"line":367,"column":1},"end":{"line":370,"column":1}},"range":[8473,8562],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"335":{"loc":{"start":{"line":366,"column":9},"end":{"line":370,"column":1}},"range":[8448,8562],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","AssignmentExpression"],"type":"AssignmentExpression","line":false},"336":{"loc":{"start":{"line":366,"column":0},"end":{"line":370,"column":1}},"range":[8439,8562],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"337":{"loc":{"start":{"line":3,"column":0},"end":{"line":12,"column":67}},"range":[39,476],"results":[],"stack":["Program","VariableDeclaration"],"type":"VariableDeclaration","line":true},"338":{"loc":{"start":{"line":15,"column":0},"end":{"line":19,"column":16}},"range":[526,618],"results":[],"stack":["Program","VariableDeclaration"],"type":"VariableDeclaration","line":true},"339":{"loc":{"start":{"line":42,"column":0},"end":{"line":150,"column":2}},"range":[1271,3744],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"340":{"loc":{"start":{"line":152,"column":0},"end":{"line":154,"column":2}},"range":[3746,3878],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"341":{"loc":{"start":{"line":170,"column":0},"end":{"line":239,"column":2}},"range":[4124,5742],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"342":{"loc":{"start":{"line":257,"column":0},"end":{"line":281,"column":2}},"range":[6150,6545],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"343":{"loc":{"start":{"line":297,"column":0},"end":{"line":303,"column":2}},"range":[6799,6966],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"344":{"loc":{"start":{"line":325,"column":0},"end":{"line":350,"column":2}},"range":[7611,8199],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"345":{"loc":{"start":{"line":366,"column":0},"end":{"line":370,"column":2}},"range":[8439,8563],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true}},"comments":[[0,37],[178,203],[478,525],[620,1269],[1648,1668],[1714,1729],[1794,1815],[1852,1877],[1909,1943],[2344,2358],[2389,2433],[2436,2503],[2506,2534],[2537,2539],[2542,2607],[2610,2680],[2683,2696],[2726,2772],[2805,2870],[3307,3347],[3351,3388],[3880,4122],[4263,4294],[4403,4445],[4719,4776],[4778,4796],[4798,4800],[4802,4862],[4864,4904],[5247,5295],[5297,5339],[5744,6148],[6547,6797],[6968,7610],[7812,7833],[7933,7947],[8088,8103],[8154,8173],[8201,8437]]};
}
function instrumentor_record(filetag, id, expression) {
	var scope = global || window;
	scope.st_timeCommenced = scope.st_timeCommenced || Date.now();
	scope.st_iterator = (scope.st_iterator || 0);

	var depth =
		(new Error()).stack.split("\n")
			.slice(1)
			.length;

	if (!scope.mem)
		scope.mem = (
			typeof process !== "undefined" ? process.memoryUsage : function(){}
		);

	__instrumentor_map__[filetag].instruments[id].results.push({
		"depth": depth,
		"time": Date.now(),
		"timeOffset": Date.now() - scope.st_timeCommenced,
		"memoryUsage": scope.mem(),
		"invocation": scope.st_iterator,
		"milestone": scope.__steamshovel_milestone,
		"value": scope.__steamshovel_record_expression ? expression : null
	});

	scope.st_iterator ++;

	// Make sure we return the original expresison result
	return expression;
}
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 337);
var crypto = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 3, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 1, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 2, 'crypto'))), esprima = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 6, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 4, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 5, 'esprima'))), escodegen = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 9, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 7, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 8, 'escodegen'))), estraverse = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 12, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 10, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 11, 'estraverse'))), noReplaceMap = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 15, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 13, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 14, './instrumentor-config')).noReplaceMap), allowedReplacements = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 18, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 16, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 17, './instrumentor-config')).allowedReplacements), noPrependMap = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 21, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 19, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 20, './instrumentor-config')).noPrependMap), allowedPrepend = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 24, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 22, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 23, './instrumentor-config')).allowedPrepend);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 338);
var callExpression, expressionStatement, fileID, prependInstrumentorMap, preprocessNode;
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 339);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 163, module.exports = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 162, function instrumentCode(data, filename, incorporateMap, returnAsObject) {
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 148);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 28, filename = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 27, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 25, filename) || instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 26, 'implicit-filename')));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 149);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 35, incorporateMap = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 34, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 31, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 29, incorporateMap) === instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 30, false)) ? instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 32, false) : instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 33, true)));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 150);
    var esprimaOptions = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 40, {
            loc: instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 36, true),
            range: instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 37, true),
            raw: instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 38, true),
            comment: instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 39, true)
        }), ast = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 43, esprima.parse(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 41, data), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 42, esprimaOptions))), filetag = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 46, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 44, fileID)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 45, filename))), comments = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 47, ast.comments), id = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 48, 0), code = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 49, null), sourceMap = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 50, {});
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 151);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 52, sourceMap.filetag = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 51, filetag));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 152);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 54, sourceMap.filename = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 53, filename));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 153);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 58, sourceMap.source = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 57, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 55, String)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 56, data))));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 154);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 60, sourceMap.instruments = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 59, {}));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 155);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 65, sourceMap.comments = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 64, comments.map(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 63, function (comment) {
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 62);
        return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 61, comment.range);
    }))));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 156);
    function sourceMapAdd(id, node, line) {
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 78);
        if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 66, sourceMap.instruments[id]))
            throw instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 69, new (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 67, Error))(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 68, 'Instrument error: Instrument already exists!')));
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 79);
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 77, sourceMap.instruments[id] = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 76, {
            'loc': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 70, node.loc),
            'range': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 71, node.range),
            'results': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 72, []),
            'stack': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 73, node.stackPath),
            'type': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 74, node.type),
            'line': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 75, !!line)
        }));
    }
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 157);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 133, estraverse.replace(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 80, ast), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 132, {
        enter: instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 81, preprocessNode),
        leave: instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 131, function (node) {
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 128);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 84, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 82, node.body) && instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 83, node.body.length))) {
                instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 113);
                instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 112, node.body = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 111, [].slice.call(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 85, node.body), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 86, 0)).reduce(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 109, function (body, node) {
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 105);
                    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 88, !~allowedPrepend.indexOf(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 87, node.type))))
                        return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 90, body.concat(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 89, node)));
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 106);
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 91, id++);
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 107);
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 96, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 92, sourceMapAdd)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 93, id), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 94, node), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 95, true)));
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 108);
                    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 104, body.concat(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 102, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 97, expressionStatement)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 101, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 98, callExpression)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 99, filetag), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 100, id))))), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 103, node)));
                }), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 110, []))));
            }
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 129);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 114, node.noReplace))
                return;
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 130);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 116, ~allowedReplacements.indexOf(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 115, node.type))))
                return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 117, id++), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 122, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 118, sourceMapAdd)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 119, id), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 120, node), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 121, false))), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 127, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 123, callExpression)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 124, filetag), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 125, id), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 126, node)));
        })
    })));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 158);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 136, code = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 135, escodegen.generate(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 134, ast))));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 159);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 137, incorporateMap))
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 142, code = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 141, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 138, prependInstrumentorMap)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 139, code), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 140, sourceMap))));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 160);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 143, returnAsObject))
        return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 146, {
            'code': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 144, code),
            'map': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 145, sourceMap)
        });
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 161);
    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 147, code);
}));
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 340);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 171, module.exports.withmap = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 170, function (data, filename, incorporateMap) {
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 169);
    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 168, module.exports(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 164, data), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 165, filename), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 166, incorporateMap), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 167, true)));
}));
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 341);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 265, preprocessNode = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 264, module.exports.preprocessNode = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 263, function preprocessNode(node) {
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 256);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 172, !node.stackPath))
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 175, node.stackPath = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 174, [instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 173, node.type)]));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 257);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 202, Object.keys(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 176, node)).forEach(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 201, function (nodekey) {
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 197);
        var prop = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 177, node[nodekey]);
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 198);
        function saveStack(prop) {
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 188);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 184, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 182, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 178, !prop) || instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 181, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 179, typeof prop) !== instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 180, 'object'))) || instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 183, !prop.type)))
                return;
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 189);
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 187, prop.stackPath = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 186, node.stackPath.concat(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 185, prop.type))));
        }
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 199);
        if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 191, Array.isArray(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 190, prop))))
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 193, prop.forEach(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 192, saveStack)));
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 200);
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 196, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 194, saveStack)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 195, prop)));
    })));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 258);
    var nodeRule = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 203, noReplaceMap[node.type]);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 259);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 204, !nodeRule))
        return;
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 260);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 207, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 205, nodeRule) instanceof instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 206, String)))
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 210, nodeRule = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 209, [instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 208, nodeRule)]));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 261);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 213, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 211, nodeRule) instanceof instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 212, Array))) {
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 230);
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 229, nodeRule.forEach(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 228, function (property) {
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 225);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 214, !node[property]))
                return;
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 226);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 217, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 215, node[property]) instanceof instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 216, Array)))
                return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 222, node[property].forEach(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 221, function (item) {
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 220);
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 219, item.noReplace = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 218, true));
                })));
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 227);
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 224, node[property].noReplace = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 223, true));
        })));
    }
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 262);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 233, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 231, nodeRule) instanceof instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 232, Object))) {
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 255);
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 254, Object.keys(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 234, nodeRule)).forEach(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 253, function (property) {
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 250);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 235, !node[property]))
                return;
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 251);
            var exclude = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 246, Object.keys(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 236, nodeRule[property])).reduce(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 244, function (prev, cur) {
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 242);
                    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 237, !prev))
                        return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 238, prev);
                    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 243);
                    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 241, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 239, node[property][cur]) === instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 240, nodeRule[property][cur]));
                }), instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 245, true)));
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 252);
            if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 247, exclude))
                instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 249, node[property].noReplace = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 248, true));
        })));
    }
})));
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 342);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 287, callExpression = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 286, module.exports.callExpression = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 285, function callExpression(filetag, id, node) {
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 282);
    var callArgs = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 272, [
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 268, {
                'type': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 266, 'Literal'),
                'value': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 267, filetag)
            }),
            instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 271, {
                'type': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 269, 'Literal'),
                'value': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 270, id)
            })
        ]);
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 283);
    if (instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 273, !!node))
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 275, callArgs.push(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 274, node)));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 284);
    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 281, {
        'type': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 276, 'CallExpression'),
        'callee': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 279, {
            'type': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 277, 'Identifier'),
            'name': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 278, 'instrumentor_record')
        }),
        'arguments': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 280, callArgs)
    });
})));
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 343);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 294, expressionStatement = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 293, module.exports.expressionStatement = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 292, function expressionStatement(node) {
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 291);
    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 290, {
        'type': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 288, 'ExpressionStatement'),
        'expression': instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 289, node)
    });
})));
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 344);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 326, prependInstrumentorMap = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 325, module.exports.prependInstrumentorMap = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 324, function prependInstrumentorMap(code, sourceMap) {
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 322);
    var filetag = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 295, sourceMap.filetag), map = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 297, JSON.stringify(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 296, sourceMap)));
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 323);
    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 321, [
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 298, 'if (typeof __instrumentor_map__ === \'undefined\') {'),
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 299, '__instrumentor_map__ = {};'),
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 300, '}'),
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 305, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 303, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 301, 'if (typeof __instrumentor_map__.') + instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 302, filetag)) + instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 304, ' === \'undefined\') {')),
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 314, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 312, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 310, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 308, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 306, '__instrumentor_map__.') + instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 307, filetag)) + instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 309, ' = ')) + instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 311, map)) + instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 313, ';')),
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 315, '}'),
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 318, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 316, require)(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 317, './instrumentor-record')).toString()),
        instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 319, code)
    ].join(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 320, '\n')));
})));
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 345);
instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 336, fileID = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 335, module.exports.fileID = instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 334, function fileID(id) {
    instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 333);
    return instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 332, instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 327, 'i') + instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 331, crypto.createHash(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 328, 'md5')).update(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 329, id)).digest(instrumentor_record('ic7db2deba541272893290ef6a69a34ee', 330, 'hex'))));
})));