if (typeof __instrumentor_map__ === 'undefined') {
__instrumentor_map__ = {};
}
if (typeof __instrumentor_map__.ibdd0ef4c9ff8ae316997d80042bcc53d === 'undefined') {
__instrumentor_map__.ibdd0ef4c9ff8ae316997d80042bcc53d = {"filetag":"ibdd0ef4c9ff8ae316997d80042bcc53d","filename":"lib/process.js","source":"\"use strict\";\n\nvar fs = require(\"fs\"),\n\tpath = require(\"path\"),\n\tmkdirp = require(\"mkdirp\"),\n\tinstrument = require(\"./instrumentor\"),\n\tassert = require(\"assert\"),\n\tEventEmitter = require(\"events\").EventEmitter;\n\n/*\n\tPublic: Instrument a file or folder using the SteamShovel instrumentor.\n\n\tinFile\t\t\t-\tThe filesystem location of the input resource\n\toutFile\t\t\t-\tThe filesystem destination for the instrumented resource\n\temitter\t\t\t-\tAn optional pre-defined event emitter to use when\n\t\t\t\t\t\temitting status events.\n\n\tReturns\n\n\t\temitter\t\t-\tAn event emitter from which information about the\n\t\t\t\t\t\tinstrumentation progress is dispatched.\n\n\tExamples\n\n\t\tinstrumentor(\"myfile.js\", \"myfile-instrumented.js\");\n\n*/\n\nmodule.exports = function instrumentTree(inFile, outFile, emitter) {\n\tvar combinedMap = {}, isDirectory = false;\n\n\t// Is this the first call in the stack?\n\t// We know because every recursion will have an emitter.\n\tvar firstCall = !emitter;\n\n\temitter =\n\t\temitter && emitter instanceof EventEmitter ? emitter :\n\t\t\tnew EventEmitter();\n\n\tif (!inFile)\n\t\tthrow new Error(\"You must specify an input file/directory.\");\n\n\tif (!outFile)\n\t\tthrow new Error(\"You must specify an output file/directory.\");\n\n\tif (inFile instanceof Array) {\n\t\tassert(outFile instanceof Array,\n\t\t\t\t\"If an array if input resources is provided, \" +\n\t\t\t\t\"the output must also be an array.\");\n\n\t\tassert(outFile.length === inFile.length,\n\t\t\t\t\"The lengths of the input and destination arrays must match.\");\n\n\n\t\t// Loop through both arrays, but retain the same emitter\n\t\tinFile.forEach(function(file, index) {\n\t\t\tinstrumentTree(file, outFile[index], emitter);\n\t\t});\n\n\t\treturn emitter;\n\n\t} else {\n\t\tassert(typeof outFile === \"string\",\n\t\t\t\"If the first parameter is not an array, the second mustn't be.\");\n\t}\n\n\tfs.stat(inFile, function(err, stats) {\n\t\tif (err) return emitter.emit(\"error\", err, inFile);\n\n\t\tif (stats.isDirectory()) {\n\t\t\tisDirectory = true;\n\n\t\t\tif (firstCall)\n\t\t\t\temitter.on(\"dircomplete\", function(dir) {\n\t\t\t\t\tif (dir === inFile) emitter.emit(\"_int_complete\", inFile);\n\t\t\t\t});\n\n\t\t\treturn module.exports.processDir(inFile, outFile, emitter);\n\t\t}\n\n\t\tif (firstCall)\n\t\t\temitter.on(\"writefile\", function(file) {\n\t\t\t\tif (file === inFile) emitter.emit(\"complete\", inFile);\n\t\t\t});\n\n\t\tmodule.exports.processFile(inFile, outFile, emitter);\n\t});\n\n\t// This chunk writes an 'instruments.shovelmap' file into the directory\n\t// being instrumented. This file contains a map of all the data from all\n\t// the files which were instrumented, so if a given file is not included\n\t// in the test suite, its data will still appear in output.\n\t\n\tif (firstCall) {\n\t\temitter.on(\"map\", function(sourceMap) {\n\t\t\tcombinedMap[sourceMap.filetag] = sourceMap;\n\t\t});\n\n\t\temitter.on(\"_int_complete\", function(inFile) {\n\t\t\tvar writePath = path.join(outFile, \"./instruments.shovelmap\");\n\n\t\t\tfs.writeFile(\n\t\t\t\twritePath,\n\t\t\t\t\"__instrumentor_map__ = \" + JSON.stringify(combinedMap),\n\t\t\t\tfunction(err) {\n\t\t\t\t\tif (err) emitter.emit(\"maperror\", err);\n\t\t\t\t\temitter.emit(\"complete\", inFile);\n\t\t\t\t});\n\t\t});\n\t}\n\n\treturn emitter;\n};\n\n/*\n\tPublic: Recursively instrument a folder/directory using the SteamShovel\n\tinstrumentor.\n\n\tinDir\t\t\t-\tThe filesystem location of the input resource\n\toutDir\t\t\t-\tThe filesystem destination for the instrumented resource\n\temitter\t\t\t-\tAn optional pre-defined event emitter to use when\n\t\t\t\t\t\temitting status events.\n\n\tReturns\n\n\t\temitter\t\t-\tAn event emitter from which information about the\n\t\t\t\t\t\tinstrumentation progress is dispatched.\n\n\tExamples\n\n\t\tinstrumentor(\"./lib\", \"./lib-cov\");\n\n*/\n\nmodule.exports.processDir = function processDir(dir, out, emitter) {\n\n\temitter =\n\t\temitter && emitter instanceof EventEmitter ? emitter :\n\t\t\tnew EventEmitter();\n\n\temitter.emit(\"processdir\", dir, out);\n\n\ttry { fs.statSync(out); }\n\tcatch (e) {\n\t\temitter.emit(\"mkdir\", out);\n\t\tmkdirp.mkdirp(out);\n\t}\n\n\tfs.readdir(dir, function(err, dirContents) {\n\t\tif (err) return emitter.emit(\"error\", err, dir);\n\n\t\temitter.emit(\"readdir\", dir);\n\n\t\tdirContents.forEach(function(file) {\n\n\t\t\tvar filePath = path.join(dir, file),\n\t\t\t\toutPath = path.join(out, file);\n\n\t\t\tmodule.exports(filePath, outPath, emitter);\n\t\t});\n\n\t\tvar instrumentedFiles = 0;\n\t\tfunction checkComplete(file) {\n\t\t\tif (file instanceof Error)\n\t\t\t\tfile = arguments[1];\n\n\t\t\tif (dirContents.reduce(function(acc, cur) {\n\t\t\t\t\treturn acc || (dir + \"/\" + cur === file);\n\t\t\t\t}, false))\n\t\t\t\tinstrumentedFiles ++;\n\n\t\t\tif (instrumentedFiles === dirContents.length) {\n\t\t\t\temitter.removeListener(\"writefile\", checkComplete);\n\t\t\t\temitter.removeListener(\"dircomplete\", checkComplete);\n\t\t\t\temitter.emit(\"dircomplete\", dir);\n\t\t\t}\n\t\t}\n\n\t\temitter.on(\"writefile\", \tcheckComplete);\n\t\temitter.on(\"dircomplete\",\tcheckComplete);\n\t});\n\n\treturn emitter;\n};\n\n/*\n\tPublic: Instrument a single file using the SteamShovel instrumentor.\n\n\tThis function will ignore files that the instrumentor cannot parse, files\n\twhich do not have a `.js` file extension, and files which contain\n\n\tinFile\t\t\t-\tThe filesystem location of the input resource\n\toutFile\t\t\t-\tThe filesystem destination for the instrumented resource\n\temitter\t\t\t-\tAn optional pre-defined event emitter to use when\n\t\t\t\t\t\temitting status events.\n\n\tReturns\n\n\t\temitter\t\t-\tAn event emitter from which information about the\n\t\t\t\t\t\tinstrumentation progress is dispatched.\n\n\tExamples\n\n\t\tinstrumentor(\"./lib/myfile.js\", \"./lib-cov/myfile.js\");\n\n*/\n\nmodule.exports.processFile = function processFile(file, out, emitter) {\n\n\temitter =\n\t\temitter && emitter instanceof EventEmitter ? emitter :\n\t\t\tnew EventEmitter();\n\n\tif (!file.match(/\\.js$/i))\n\t\treturn\temitter.emit(\"nojs\", file),\n\t\t\t\tfs\t.createReadStream(file)\n\t\t\t\t\t.pipe(fs.createWriteStream(out))\n\t\t\t\t\t.on(\"close\", function() {\n\t\t\t\t\t\temitter.emit(\"writefile\", file);\n\t\t\t\t\t});\n\n\tfs.readFile(file, function(err, data) {\n\t\tif (err) return emitter.emit(\"error\", err, file);\n\n\t\temitter.emit(\"readfile\", file);\n\n\t\tvar instrumentResult,\n\t\t\tcode;\n\t\t\tdata = String(data);\n\n\t\tif (~data.indexOf(\"steamShovel:\" + \"ignore\")) {\n\t\t\temitter.emit(\"ignore\", file);\n\n\t\t} else {\n\t\t\ttry {\n\t\t\t\tinstrumentResult = instrument.withmap(data, file);\n\t\t\t\tcode = instrumentResult.code;\n\n\t\t\t} catch (err) {\n\t\t\t\temitter.emit(\"instrumenterror\", err, file);\n\t\t\t}\n\t\t}\n\n\t\tfs.writeFile(out, code || data, function(err) {\n\t\t\tif (err) return emitter.emit(\"error\", err, file);\n\n\t\t\tif (instrumentResult && instrumentResult.map) {\n\t\t\t\temitter.emit(\"map\", instrumentResult.map);\n\t\t\t}\n\n\t\t\temitter.emit(\"writefile\", file);\n\t\t});\n\t});\n\n\treturn emitter;\n};","instruments":{"1":{"loc":{"start":{"line":1,"column":0},"end":{"line":1,"column":12}},"range":[0,12],"results":[],"stack":["Program","ExpressionStatement","Literal"],"type":"Literal","line":false},"2":{"loc":{"start":{"line":3,"column":9},"end":{"line":3,"column":16}},"range":[24,31],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"3":{"loc":{"start":{"line":3,"column":17},"end":{"line":3,"column":21}},"range":[32,36],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"4":{"loc":{"start":{"line":3,"column":9},"end":{"line":3,"column":22}},"range":[24,37],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"5":{"loc":{"start":{"line":4,"column":8},"end":{"line":4,"column":15}},"range":[47,54],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"6":{"loc":{"start":{"line":4,"column":16},"end":{"line":4,"column":22}},"range":[55,61],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"7":{"loc":{"start":{"line":4,"column":8},"end":{"line":4,"column":23}},"range":[47,62],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"8":{"loc":{"start":{"line":5,"column":10},"end":{"line":5,"column":17}},"range":[74,81],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"9":{"loc":{"start":{"line":5,"column":18},"end":{"line":5,"column":26}},"range":[82,90],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"10":{"loc":{"start":{"line":5,"column":10},"end":{"line":5,"column":27}},"range":[74,91],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"11":{"loc":{"start":{"line":6,"column":14},"end":{"line":6,"column":21}},"range":[107,114],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"12":{"loc":{"start":{"line":6,"column":22},"end":{"line":6,"column":38}},"range":[115,131],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"13":{"loc":{"start":{"line":6,"column":14},"end":{"line":6,"column":39}},"range":[107,132],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"14":{"loc":{"start":{"line":7,"column":10},"end":{"line":7,"column":17}},"range":[144,151],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"15":{"loc":{"start":{"line":7,"column":18},"end":{"line":7,"column":26}},"range":[152,160],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"16":{"loc":{"start":{"line":7,"column":10},"end":{"line":7,"column":27}},"range":[144,161],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"17":{"loc":{"start":{"line":8,"column":16},"end":{"line":8,"column":23}},"range":[179,186],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"18":{"loc":{"start":{"line":8,"column":24},"end":{"line":8,"column":32}},"range":[187,195],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","Literal"],"type":"Literal","line":false},"19":{"loc":{"start":{"line":8,"column":16},"end":{"line":8,"column":46}},"range":[179,209],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"20":{"loc":{"start":{"line":30,"column":19},"end":{"line":30,"column":21}},"range":[790,792],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression"],"type":"ObjectExpression","line":false},"21":{"loc":{"start":{"line":30,"column":37},"end":{"line":30,"column":42}},"range":[808,813],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"22":{"loc":{"start":{"line":34,"column":17},"end":{"line":34,"column":25}},"range":[932,940],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","UnaryExpression"],"type":"UnaryExpression","line":false},"23":{"loc":{"start":{"line":37,"column":2},"end":{"line":37,"column":9}},"range":[956,963],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","Identifier"],"type":"Identifier","line":false},"24":{"loc":{"start":{"line":37,"column":13},"end":{"line":37,"column":20}},"range":[967,974],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"25":{"loc":{"start":{"line":37,"column":32},"end":{"line":37,"column":44}},"range":[986,998],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"26":{"loc":{"start":{"line":37,"column":13},"end":{"line":37,"column":44}},"range":[967,998],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"27":{"loc":{"start":{"line":37,"column":2},"end":{"line":37,"column":44}},"range":[956,998],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression"],"type":"LogicalExpression","line":false},"28":{"loc":{"start":{"line":37,"column":47},"end":{"line":37,"column":54}},"range":[1001,1008],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Identifier"],"type":"Identifier","line":false},"29":{"loc":{"start":{"line":38,"column":7},"end":{"line":38,"column":19}},"range":[1018,1030],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","NewExpression","Identifier"],"type":"Identifier","line":false},"30":{"loc":{"start":{"line":38,"column":3},"end":{"line":38,"column":21}},"range":[1014,1032],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","NewExpression"],"type":"NewExpression","line":false},"31":{"loc":{"start":{"line":37,"column":2},"end":{"line":38,"column":21}},"range":[956,1032],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression"],"type":"ConditionalExpression","line":false},"32":{"loc":{"start":{"line":36,"column":1},"end":{"line":38,"column":21}},"range":[944,1032],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"33":{"loc":{"start":{"line":40,"column":5},"end":{"line":40,"column":12}},"range":[1040,1047],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"34":{"loc":{"start":{"line":41,"column":12},"end":{"line":41,"column":17}},"range":[1061,1066],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression","Identifier"],"type":"Identifier","line":false},"35":{"loc":{"start":{"line":41,"column":18},"end":{"line":41,"column":61}},"range":[1067,1110],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression","Literal"],"type":"Literal","line":false},"36":{"loc":{"start":{"line":41,"column":8},"end":{"line":41,"column":62}},"range":[1057,1111],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression"],"type":"NewExpression","line":false},"37":{"loc":{"start":{"line":43,"column":5},"end":{"line":43,"column":13}},"range":[1119,1127],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"38":{"loc":{"start":{"line":44,"column":12},"end":{"line":44,"column":17}},"range":[1141,1146],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression","Identifier"],"type":"Identifier","line":false},"39":{"loc":{"start":{"line":44,"column":18},"end":{"line":44,"column":62}},"range":[1147,1191],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression","Literal"],"type":"Literal","line":false},"40":{"loc":{"start":{"line":44,"column":8},"end":{"line":44,"column":63}},"range":[1137,1192],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression"],"type":"NewExpression","line":false},"41":{"loc":{"start":{"line":46,"column":5},"end":{"line":46,"column":11}},"range":[1200,1206],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"42":{"loc":{"start":{"line":46,"column":23},"end":{"line":46,"column":28}},"range":[1218,1223],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"43":{"loc":{"start":{"line":46,"column":5},"end":{"line":46,"column":28}},"range":[1200,1223],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"44":{"loc":{"start":{"line":47,"column":2},"end":{"line":47,"column":8}},"range":[1229,1235],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"45":{"loc":{"start":{"line":47,"column":9},"end":{"line":47,"column":16}},"range":[1236,1243],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"46":{"loc":{"start":{"line":47,"column":28},"end":{"line":47,"column":33}},"range":[1255,1260],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"47":{"loc":{"start":{"line":47,"column":9},"end":{"line":47,"column":33}},"range":[1236,1260],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"48":{"loc":{"start":{"line":48,"column":4},"end":{"line":48,"column":50}},"range":[1266,1312],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"49":{"loc":{"start":{"line":49,"column":4},"end":{"line":49,"column":39}},"range":[1319,1354],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"50":{"loc":{"start":{"line":48,"column":4},"end":{"line":49,"column":39}},"range":[1266,1354],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"51":{"loc":{"start":{"line":47,"column":2},"end":{"line":49,"column":40}},"range":[1229,1355],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"52":{"loc":{"start":{"line":51,"column":2},"end":{"line":51,"column":8}},"range":[1360,1366],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"53":{"loc":{"start":{"line":51,"column":9},"end":{"line":51,"column":23}},"range":[1367,1381],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"54":{"loc":{"start":{"line":51,"column":28},"end":{"line":51,"column":41}},"range":[1386,1399],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"55":{"loc":{"start":{"line":51,"column":9},"end":{"line":51,"column":41}},"range":[1367,1399],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"56":{"loc":{"start":{"line":52,"column":4},"end":{"line":52,"column":65}},"range":[1405,1466],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"57":{"loc":{"start":{"line":51,"column":2},"end":{"line":52,"column":66}},"range":[1360,1467],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"58":{"loc":{"start":{"line":57,"column":3},"end":{"line":57,"column":17}},"range":[1574,1588],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"59":{"loc":{"start":{"line":57,"column":18},"end":{"line":57,"column":22}},"range":[1589,1593],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"60":{"loc":{"start":{"line":57,"column":24},"end":{"line":57,"column":38}},"range":[1595,1609],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"61":{"loc":{"start":{"line":57,"column":40},"end":{"line":57,"column":47}},"range":[1611,1618],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"62":{"loc":{"start":{"line":57,"column":3},"end":{"line":57,"column":48}},"range":[1574,1619],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"63":{"loc":{"start":{"line":57,"column":3},"end":{"line":57,"column":49}},"range":[1574,1620],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"64":{"loc":{"start":{"line":56,"column":17},"end":{"line":58,"column":3}},"range":[1547,1624],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"65":{"loc":{"start":{"line":56,"column":2},"end":{"line":58,"column":4}},"range":[1532,1625],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"66":{"loc":{"start":{"line":60,"column":9},"end":{"line":60,"column":16}},"range":[1637,1644],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"67":{"loc":{"start":{"line":47,"column":2},"end":{"line":49,"column":41}},"range":[1229,1356],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"68":{"loc":{"start":{"line":51,"column":2},"end":{"line":52,"column":67}},"range":[1360,1468],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"69":{"loc":{"start":{"line":56,"column":2},"end":{"line":58,"column":5}},"range":[1532,1626],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"70":{"loc":{"start":{"line":60,"column":2},"end":{"line":60,"column":17}},"range":[1630,1645],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"71":{"loc":{"start":{"line":63,"column":2},"end":{"line":63,"column":8}},"range":[1659,1665],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"72":{"loc":{"start":{"line":63,"column":9},"end":{"line":63,"column":23}},"range":[1666,1680],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"73":{"loc":{"start":{"line":63,"column":28},"end":{"line":63,"column":36}},"range":[1685,1693],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"74":{"loc":{"start":{"line":63,"column":9},"end":{"line":63,"column":36}},"range":[1666,1693],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"75":{"loc":{"start":{"line":64,"column":3},"end":{"line":64,"column":67}},"range":[1698,1762],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"76":{"loc":{"start":{"line":63,"column":2},"end":{"line":64,"column":68}},"range":[1659,1763],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"77":{"loc":{"start":{"line":63,"column":2},"end":{"line":64,"column":69}},"range":[1659,1764],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"78":{"loc":{"start":{"line":67,"column":9},"end":{"line":67,"column":15}},"range":[1778,1784],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"79":{"loc":{"start":{"line":68,"column":6},"end":{"line":68,"column":9}},"range":[1815,1818],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"80":{"loc":{"start":{"line":68,"column":31},"end":{"line":68,"column":38}},"range":[1840,1847],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Literal"],"type":"Literal","line":false},"81":{"loc":{"start":{"line":68,"column":40},"end":{"line":68,"column":43}},"range":[1849,1852],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"82":{"loc":{"start":{"line":68,"column":45},"end":{"line":68,"column":51}},"range":[1854,1860],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"83":{"loc":{"start":{"line":68,"column":18},"end":{"line":68,"column":52}},"range":[1827,1861],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"84":{"loc":{"start":{"line":70,"column":6},"end":{"line":70,"column":25}},"range":[1870,1889],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","CallExpression"],"type":"CallExpression","line":false},"85":{"loc":{"start":{"line":71,"column":17},"end":{"line":71,"column":21}},"range":[1910,1914],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression","Literal"],"type":"Literal","line":false},"86":{"loc":{"start":{"line":71,"column":3},"end":{"line":71,"column":21}},"range":[1896,1914],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"87":{"loc":{"start":{"line":73,"column":7},"end":{"line":73,"column":16}},"range":[1924,1933],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"88":{"loc":{"start":{"line":74,"column":15},"end":{"line":74,"column":28}},"range":[1950,1963],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"89":{"loc":{"start":{"line":75,"column":9},"end":{"line":75,"column":12}},"range":[1990,1993],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"90":{"loc":{"start":{"line":75,"column":17},"end":{"line":75,"column":23}},"range":[1998,2004],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"91":{"loc":{"start":{"line":75,"column":9},"end":{"line":75,"column":23}},"range":[1990,2004],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"92":{"loc":{"start":{"line":75,"column":38},"end":{"line":75,"column":53}},"range":[2019,2034],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"93":{"loc":{"start":{"line":75,"column":55},"end":{"line":75,"column":61}},"range":[2036,2042],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"94":{"loc":{"start":{"line":75,"column":25},"end":{"line":75,"column":62}},"range":[2006,2043],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"95":{"loc":{"start":{"line":75,"column":5},"end":{"line":75,"column":63}},"range":[1986,2044],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"96":{"loc":{"start":{"line":74,"column":30},"end":{"line":76,"column":5}},"range":[1965,2050],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"97":{"loc":{"start":{"line":74,"column":4},"end":{"line":76,"column":6}},"range":[1939,2051],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"98":{"loc":{"start":{"line":78,"column":36},"end":{"line":78,"column":42}},"range":[2090,2096],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"99":{"loc":{"start":{"line":78,"column":44},"end":{"line":78,"column":51}},"range":[2098,2105],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"100":{"loc":{"start":{"line":78,"column":53},"end":{"line":78,"column":60}},"range":[2107,2114],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"101":{"loc":{"start":{"line":78,"column":10},"end":{"line":78,"column":61}},"range":[2064,2115],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"102":{"loc":{"start":{"line":71,"column":3},"end":{"line":71,"column":22}},"range":[1896,1915],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"103":{"loc":{"start":{"line":73,"column":3},"end":{"line":76,"column":7}},"range":[1920,2052],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"104":{"loc":{"start":{"line":78,"column":3},"end":{"line":78,"column":62}},"range":[2057,2116],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"105":{"loc":{"start":{"line":81,"column":6},"end":{"line":81,"column":15}},"range":[2128,2137],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"106":{"loc":{"start":{"line":82,"column":14},"end":{"line":82,"column":25}},"range":[2153,2164],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"107":{"loc":{"start":{"line":83,"column":8},"end":{"line":83,"column":12}},"range":[2191,2195],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"108":{"loc":{"start":{"line":83,"column":17},"end":{"line":83,"column":23}},"range":[2200,2206],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"109":{"loc":{"start":{"line":83,"column":8},"end":{"line":83,"column":23}},"range":[2191,2206],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"110":{"loc":{"start":{"line":83,"column":38},"end":{"line":83,"column":48}},"range":[2221,2231],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"111":{"loc":{"start":{"line":83,"column":50},"end":{"line":83,"column":56}},"range":[2233,2239],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"112":{"loc":{"start":{"line":83,"column":25},"end":{"line":83,"column":57}},"range":[2208,2240],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"113":{"loc":{"start":{"line":83,"column":4},"end":{"line":83,"column":58}},"range":[2187,2241],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"114":{"loc":{"start":{"line":82,"column":27},"end":{"line":84,"column":4}},"range":[2166,2246],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"115":{"loc":{"start":{"line":82,"column":3},"end":{"line":84,"column":5}},"range":[2142,2247],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"116":{"loc":{"start":{"line":86,"column":29},"end":{"line":86,"column":35}},"range":[2279,2285],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"117":{"loc":{"start":{"line":86,"column":37},"end":{"line":86,"column":44}},"range":[2287,2294],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"118":{"loc":{"start":{"line":86,"column":46},"end":{"line":86,"column":53}},"range":[2296,2303],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"119":{"loc":{"start":{"line":86,"column":2},"end":{"line":86,"column":54}},"range":[2252,2304],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"120":{"loc":{"start":{"line":68,"column":2},"end":{"line":68,"column":53}},"range":[1811,1862],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"121":{"loc":{"start":{"line":70,"column":2},"end":{"line":79,"column":3}},"range":[1866,2120],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"122":{"loc":{"start":{"line":81,"column":2},"end":{"line":84,"column":6}},"range":[2124,2248],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"123":{"loc":{"start":{"line":86,"column":2},"end":{"line":86,"column":55}},"range":[2252,2305],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"124":{"loc":{"start":{"line":67,"column":17},"end":{"line":87,"column":2}},"range":[1786,2308],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"125":{"loc":{"start":{"line":67,"column":1},"end":{"line":87,"column":3}},"range":[1770,2309],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"126":{"loc":{"start":{"line":94,"column":5},"end":{"line":94,"column":14}},"range":[2601,2610],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"127":{"loc":{"start":{"line":95,"column":13},"end":{"line":95,"column":18}},"range":[2627,2632],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"128":{"loc":{"start":{"line":96,"column":36},"end":{"line":96,"column":45}},"range":[2692,2701],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","Identifier"],"type":"Identifier","line":false},"129":{"loc":{"start":{"line":96,"column":3},"end":{"line":96,"column":45}},"range":[2659,2701],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"130":{"loc":{"start":{"line":96,"column":3},"end":{"line":96,"column":46}},"range":[2659,2702],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"131":{"loc":{"start":{"line":95,"column":20},"end":{"line":97,"column":3}},"range":[2634,2706],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"132":{"loc":{"start":{"line":95,"column":2},"end":{"line":97,"column":4}},"range":[2616,2707],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"133":{"loc":{"start":{"line":99,"column":13},"end":{"line":99,"column":28}},"range":[2723,2738],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"134":{"loc":{"start":{"line":100,"column":29},"end":{"line":100,"column":36}},"range":[2788,2795],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"135":{"loc":{"start":{"line":100,"column":38},"end":{"line":100,"column":63}},"range":[2797,2822],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"136":{"loc":{"start":{"line":100,"column":19},"end":{"line":100,"column":64}},"range":[2778,2823],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"137":{"loc":{"start":{"line":103,"column":4},"end":{"line":103,"column":13}},"range":[2847,2856],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"138":{"loc":{"start":{"line":104,"column":4},"end":{"line":104,"column":29}},"range":[2862,2887],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"139":{"loc":{"start":{"line":104,"column":47},"end":{"line":104,"column":58}},"range":[2905,2916],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"140":{"loc":{"start":{"line":104,"column":32},"end":{"line":104,"column":59}},"range":[2890,2917],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","CallExpression"],"type":"CallExpression","line":false},"141":{"loc":{"start":{"line":104,"column":4},"end":{"line":104,"column":59}},"range":[2862,2917],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"142":{"loc":{"start":{"line":106,"column":9},"end":{"line":106,"column":12}},"range":[2948,2951],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"143":{"loc":{"start":{"line":106,"column":27},"end":{"line":106,"column":37}},"range":[2966,2976],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"144":{"loc":{"start":{"line":106,"column":39},"end":{"line":106,"column":42}},"range":[2978,2981],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"145":{"loc":{"start":{"line":106,"column":14},"end":{"line":106,"column":43}},"range":[2953,2982],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"146":{"loc":{"start":{"line":107,"column":18},"end":{"line":107,"column":28}},"range":[3002,3012],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"147":{"loc":{"start":{"line":107,"column":30},"end":{"line":107,"column":36}},"range":[3014,3020],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"148":{"loc":{"start":{"line":107,"column":5},"end":{"line":107,"column":37}},"range":[2989,3021],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"149":{"loc":{"start":{"line":106,"column":5},"end":{"line":106,"column":44}},"range":[2944,2983],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"150":{"loc":{"start":{"line":107,"column":5},"end":{"line":107,"column":38}},"range":[2989,3022],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"151":{"loc":{"start":{"line":105,"column":4},"end":{"line":108,"column":5}},"range":[2923,3028],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"152":{"loc":{"start":{"line":102,"column":3},"end":{"line":108,"column":6}},"range":[2829,3029],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"153":{"loc":{"start":{"line":100,"column":3},"end":{"line":100,"column":65}},"range":[2762,2824],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"154":{"loc":{"start":{"line":102,"column":3},"end":{"line":108,"column":7}},"range":[2829,3030],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"155":{"loc":{"start":{"line":99,"column":30},"end":{"line":109,"column":3}},"range":[2740,3034],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"156":{"loc":{"start":{"line":99,"column":2},"end":{"line":109,"column":4}},"range":[2712,3035],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"157":{"loc":{"start":{"line":95,"column":2},"end":{"line":97,"column":5}},"range":[2616,2708],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"158":{"loc":{"start":{"line":99,"column":2},"end":{"line":109,"column":5}},"range":[2712,3036],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"159":{"loc":{"start":{"line":112,"column":8},"end":{"line":112,"column":15}},"range":[3049,3056],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"160":{"loc":{"start":{"line":30,"column":1},"end":{"line":30,"column":43}},"range":[772,814],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"161":{"loc":{"start":{"line":34,"column":1},"end":{"line":34,"column":26}},"range":[916,941],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"162":{"loc":{"start":{"line":36,"column":1},"end":{"line":38,"column":22}},"range":[944,1033],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"163":{"loc":{"start":{"line":40,"column":1},"end":{"line":41,"column":63}},"range":[1036,1112],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"164":{"loc":{"start":{"line":43,"column":1},"end":{"line":44,"column":64}},"range":[1115,1193],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"165":{"loc":{"start":{"line":46,"column":1},"end":{"line":65,"column":2}},"range":[1196,1767],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"166":{"loc":{"start":{"line":67,"column":1},"end":{"line":87,"column":4}},"range":[1770,2310],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"167":{"loc":{"start":{"line":94,"column":1},"end":{"line":110,"column":2}},"range":[2597,3039],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"168":{"loc":{"start":{"line":112,"column":1},"end":{"line":112,"column":16}},"range":[3042,3057],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"169":{"loc":{"start":{"line":29,"column":17},"end":{"line":113,"column":1}},"range":[719,3059],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"170":{"loc":{"start":{"line":29,"column":0},"end":{"line":113,"column":1}},"range":[702,3059],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"171":{"loc":{"start":{"line":138,"column":2},"end":{"line":138,"column":9}},"range":[3631,3638],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","Identifier"],"type":"Identifier","line":false},"172":{"loc":{"start":{"line":138,"column":13},"end":{"line":138,"column":20}},"range":[3642,3649],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"173":{"loc":{"start":{"line":138,"column":32},"end":{"line":138,"column":44}},"range":[3661,3673],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"174":{"loc":{"start":{"line":138,"column":13},"end":{"line":138,"column":44}},"range":[3642,3673],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"175":{"loc":{"start":{"line":138,"column":2},"end":{"line":138,"column":44}},"range":[3631,3673],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression"],"type":"LogicalExpression","line":false},"176":{"loc":{"start":{"line":138,"column":47},"end":{"line":138,"column":54}},"range":[3676,3683],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Identifier"],"type":"Identifier","line":false},"177":{"loc":{"start":{"line":139,"column":7},"end":{"line":139,"column":19}},"range":[3693,3705],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","NewExpression","Identifier"],"type":"Identifier","line":false},"178":{"loc":{"start":{"line":139,"column":3},"end":{"line":139,"column":21}},"range":[3689,3707],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","NewExpression"],"type":"NewExpression","line":false},"179":{"loc":{"start":{"line":138,"column":2},"end":{"line":139,"column":21}},"range":[3631,3707],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression"],"type":"ConditionalExpression","line":false},"180":{"loc":{"start":{"line":137,"column":1},"end":{"line":139,"column":21}},"range":[3619,3707],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"181":{"loc":{"start":{"line":141,"column":14},"end":{"line":141,"column":26}},"range":[3724,3736],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"182":{"loc":{"start":{"line":141,"column":28},"end":{"line":141,"column":31}},"range":[3738,3741],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"183":{"loc":{"start":{"line":141,"column":33},"end":{"line":141,"column":36}},"range":[3743,3746],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"184":{"loc":{"start":{"line":141,"column":1},"end":{"line":141,"column":37}},"range":[3711,3747],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"185":{"loc":{"start":{"line":143,"column":19},"end":{"line":143,"column":22}},"range":[3769,3772],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"186":{"loc":{"start":{"line":143,"column":7},"end":{"line":143,"column":23}},"range":[3757,3773],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"187":{"loc":{"start":{"line":143,"column":7},"end":{"line":143,"column":24}},"range":[3757,3774],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"188":{"loc":{"start":{"line":145,"column":15},"end":{"line":145,"column":22}},"range":[3805,3812],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"189":{"loc":{"start":{"line":145,"column":24},"end":{"line":145,"column":27}},"range":[3814,3817],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"190":{"loc":{"start":{"line":145,"column":2},"end":{"line":145,"column":28}},"range":[3792,3818],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"191":{"loc":{"start":{"line":146,"column":16},"end":{"line":146,"column":19}},"range":[3836,3839],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"192":{"loc":{"start":{"line":146,"column":2},"end":{"line":146,"column":20}},"range":[3822,3840],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"193":{"loc":{"start":{"line":145,"column":2},"end":{"line":145,"column":29}},"range":[3792,3819],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"194":{"loc":{"start":{"line":146,"column":2},"end":{"line":146,"column":21}},"range":[3822,3841],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"195":{"loc":{"start":{"line":149,"column":12},"end":{"line":149,"column":15}},"range":[3858,3861],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"196":{"loc":{"start":{"line":150,"column":6},"end":{"line":150,"column":9}},"range":[3898,3901],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"197":{"loc":{"start":{"line":150,"column":31},"end":{"line":150,"column":38}},"range":[3923,3930],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Literal"],"type":"Literal","line":false},"198":{"loc":{"start":{"line":150,"column":40},"end":{"line":150,"column":43}},"range":[3932,3935],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"199":{"loc":{"start":{"line":150,"column":45},"end":{"line":150,"column":48}},"range":[3937,3940],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"200":{"loc":{"start":{"line":150,"column":18},"end":{"line":150,"column":49}},"range":[3910,3941],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"201":{"loc":{"start":{"line":152,"column":15},"end":{"line":152,"column":24}},"range":[3959,3968],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"202":{"loc":{"start":{"line":152,"column":26},"end":{"line":152,"column":29}},"range":[3970,3973],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"203":{"loc":{"start":{"line":152,"column":2},"end":{"line":152,"column":30}},"range":[3946,3974],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"204":{"loc":{"start":{"line":156,"column":28},"end":{"line":156,"column":31}},"range":[4045,4048],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"205":{"loc":{"start":{"line":156,"column":33},"end":{"line":156,"column":37}},"range":[4050,4054],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"206":{"loc":{"start":{"line":156,"column":18},"end":{"line":156,"column":38}},"range":[4035,4055],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"207":{"loc":{"start":{"line":157,"column":24},"end":{"line":157,"column":27}},"range":[4081,4084],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"208":{"loc":{"start":{"line":157,"column":29},"end":{"line":157,"column":33}},"range":[4086,4090],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"209":{"loc":{"start":{"line":157,"column":14},"end":{"line":157,"column":34}},"range":[4071,4091],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"210":{"loc":{"start":{"line":159,"column":18},"end":{"line":159,"column":26}},"range":[4112,4120],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"211":{"loc":{"start":{"line":159,"column":28},"end":{"line":159,"column":35}},"range":[4122,4129],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"212":{"loc":{"start":{"line":159,"column":37},"end":{"line":159,"column":44}},"range":[4131,4138],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"213":{"loc":{"start":{"line":159,"column":3},"end":{"line":159,"column":45}},"range":[4097,4139],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"214":{"loc":{"start":{"line":156,"column":3},"end":{"line":157,"column":35}},"range":[4020,4092],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"215":{"loc":{"start":{"line":159,"column":3},"end":{"line":159,"column":46}},"range":[4097,4140],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"216":{"loc":{"start":{"line":154,"column":22},"end":{"line":160,"column":3}},"range":[3999,4144],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"217":{"loc":{"start":{"line":154,"column":2},"end":{"line":160,"column":4}},"range":[3979,4145],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"218":{"loc":{"start":{"line":162,"column":26},"end":{"line":162,"column":27}},"range":[4174,4175],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"219":{"loc":{"start":{"line":164,"column":7},"end":{"line":164,"column":11}},"range":[4217,4221],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"220":{"loc":{"start":{"line":164,"column":23},"end":{"line":164,"column":28}},"range":[4233,4238],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"221":{"loc":{"start":{"line":164,"column":7},"end":{"line":164,"column":28}},"range":[4217,4238],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"222":{"loc":{"start":{"line":165,"column":11},"end":{"line":165,"column":23}},"range":[4251,4263],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression","MemberExpression"],"type":"MemberExpression","line":false},"223":{"loc":{"start":{"line":165,"column":4},"end":{"line":165,"column":23}},"range":[4244,4263],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"224":{"loc":{"start":{"line":168,"column":12},"end":{"line":168,"column":15}},"range":[4325,4328],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","Identifier"],"type":"Identifier","line":false},"225":{"loc":{"start":{"line":168,"column":20},"end":{"line":168,"column":23}},"range":[4333,4336],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","BinaryExpression","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"226":{"loc":{"start":{"line":168,"column":26},"end":{"line":168,"column":29}},"range":[4339,4342],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","BinaryExpression","BinaryExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"227":{"loc":{"start":{"line":168,"column":20},"end":{"line":168,"column":29}},"range":[4333,4342],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","BinaryExpression","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"228":{"loc":{"start":{"line":168,"column":32},"end":{"line":168,"column":35}},"range":[4345,4348],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"229":{"loc":{"start":{"line":168,"column":20},"end":{"line":168,"column":35}},"range":[4333,4348],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"230":{"loc":{"start":{"line":168,"column":40},"end":{"line":168,"column":44}},"range":[4353,4357],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"231":{"loc":{"start":{"line":168,"column":20},"end":{"line":168,"column":44}},"range":[4333,4357],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"232":{"loc":{"start":{"line":168,"column":12},"end":{"line":168,"column":45}},"range":[4325,4358],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"233":{"loc":{"start":{"line":168,"column":5},"end":{"line":168,"column":46}},"range":[4318,4359],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"234":{"loc":{"start":{"line":167,"column":26},"end":{"line":169,"column":5}},"range":[4292,4365],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"235":{"loc":{"start":{"line":169,"column":7},"end":{"line":169,"column":12}},"range":[4367,4372],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression","Literal"],"type":"Literal","line":false},"236":{"loc":{"start":{"line":167,"column":7},"end":{"line":169,"column":13}},"range":[4273,4373],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","CallExpression"],"type":"CallExpression","line":false},"237":{"loc":{"start":{"line":170,"column":4},"end":{"line":170,"column":24}},"range":[4379,4399],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","ExpressionStatement","UpdateExpression"],"type":"UpdateExpression","line":false},"238":{"loc":{"start":{"line":172,"column":7},"end":{"line":172,"column":24}},"range":[4409,4426],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"239":{"loc":{"start":{"line":172,"column":29},"end":{"line":172,"column":47}},"range":[4431,4449],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"240":{"loc":{"start":{"line":172,"column":7},"end":{"line":172,"column":47}},"range":[4409,4449],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"241":{"loc":{"start":{"line":173,"column":27},"end":{"line":173,"column":38}},"range":[4480,4491],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"242":{"loc":{"start":{"line":173,"column":40},"end":{"line":173,"column":53}},"range":[4493,4506],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"243":{"loc":{"start":{"line":173,"column":4},"end":{"line":173,"column":54}},"range":[4457,4507],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"244":{"loc":{"start":{"line":174,"column":27},"end":{"line":174,"column":40}},"range":[4536,4549],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"245":{"loc":{"start":{"line":174,"column":42},"end":{"line":174,"column":55}},"range":[4551,4564],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"246":{"loc":{"start":{"line":174,"column":4},"end":{"line":174,"column":56}},"range":[4513,4565],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"247":{"loc":{"start":{"line":175,"column":17},"end":{"line":175,"column":30}},"range":[4584,4597],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"248":{"loc":{"start":{"line":175,"column":32},"end":{"line":175,"column":35}},"range":[4599,4602],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"249":{"loc":{"start":{"line":175,"column":4},"end":{"line":175,"column":36}},"range":[4571,4603],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"250":{"loc":{"start":{"line":173,"column":4},"end":{"line":173,"column":55}},"range":[4457,4508],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"251":{"loc":{"start":{"line":174,"column":4},"end":{"line":174,"column":57}},"range":[4513,4566],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"252":{"loc":{"start":{"line":175,"column":4},"end":{"line":175,"column":37}},"range":[4571,4604],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"253":{"loc":{"start":{"line":164,"column":3},"end":{"line":165,"column":24}},"range":[4213,4264],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"254":{"loc":{"start":{"line":167,"column":3},"end":{"line":170,"column":25}},"range":[4269,4400],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"255":{"loc":{"start":{"line":172,"column":3},"end":{"line":176,"column":4}},"range":[4405,4609],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"256":{"loc":{"start":{"line":179,"column":13},"end":{"line":179,"column":24}},"range":[4628,4639],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"257":{"loc":{"start":{"line":179,"column":27},"end":{"line":179,"column":40}},"range":[4642,4655],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"258":{"loc":{"start":{"line":179,"column":2},"end":{"line":179,"column":41}},"range":[4617,4656],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"259":{"loc":{"start":{"line":180,"column":13},"end":{"line":180,"column":26}},"range":[4671,4684],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"260":{"loc":{"start":{"line":180,"column":28},"end":{"line":180,"column":41}},"range":[4686,4699],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"261":{"loc":{"start":{"line":180,"column":2},"end":{"line":180,"column":42}},"range":[4660,4700],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"262":{"loc":{"start":{"line":150,"column":2},"end":{"line":150,"column":50}},"range":[3894,3942],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"263":{"loc":{"start":{"line":152,"column":2},"end":{"line":152,"column":31}},"range":[3946,3975],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"264":{"loc":{"start":{"line":154,"column":2},"end":{"line":160,"column":5}},"range":[3979,4146],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"265":{"loc":{"start":{"line":162,"column":2},"end":{"line":162,"column":28}},"range":[4150,4176],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"266":{"loc":{"start":{"line":163,"column":2},"end":{"line":177,"column":3}},"range":[4179,4613],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","FunctionDeclaration"],"type":"FunctionDeclaration","line":true},"267":{"loc":{"start":{"line":179,"column":2},"end":{"line":179,"column":42}},"range":[4617,4657],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"268":{"loc":{"start":{"line":180,"column":2},"end":{"line":180,"column":43}},"range":[4660,4701],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"269":{"loc":{"start":{"line":149,"column":17},"end":{"line":181,"column":2}},"range":[3863,4704],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"270":{"loc":{"start":{"line":149,"column":1},"end":{"line":181,"column":3}},"range":[3847,4705],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"271":{"loc":{"start":{"line":183,"column":8},"end":{"line":183,"column":15}},"range":[4716,4723],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"272":{"loc":{"start":{"line":137,"column":1},"end":{"line":139,"column":22}},"range":[3619,3708],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"273":{"loc":{"start":{"line":141,"column":1},"end":{"line":141,"column":38}},"range":[3711,3748],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"274":{"loc":{"start":{"line":143,"column":1},"end":{"line":147,"column":2}},"range":[3751,3844],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","TryStatement"],"type":"TryStatement","line":true},"275":{"loc":{"start":{"line":149,"column":1},"end":{"line":181,"column":4}},"range":[3847,4706],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"276":{"loc":{"start":{"line":183,"column":1},"end":{"line":183,"column":16}},"range":[4709,4724],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"277":{"loc":{"start":{"line":135,"column":28},"end":{"line":184,"column":1}},"range":[3576,4726],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"278":{"loc":{"start":{"line":135,"column":0},"end":{"line":184,"column":1}},"range":[3548,4726],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"279":{"loc":{"start":{"line":211,"column":2},"end":{"line":211,"column":9}},"range":[5448,5455],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","Identifier"],"type":"Identifier","line":false},"280":{"loc":{"start":{"line":211,"column":13},"end":{"line":211,"column":20}},"range":[5459,5466],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"281":{"loc":{"start":{"line":211,"column":32},"end":{"line":211,"column":44}},"range":[5478,5490],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"282":{"loc":{"start":{"line":211,"column":13},"end":{"line":211,"column":44}},"range":[5459,5490],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"283":{"loc":{"start":{"line":211,"column":2},"end":{"line":211,"column":44}},"range":[5448,5490],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","LogicalExpression"],"type":"LogicalExpression","line":false},"284":{"loc":{"start":{"line":211,"column":47},"end":{"line":211,"column":54}},"range":[5493,5500],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Identifier"],"type":"Identifier","line":false},"285":{"loc":{"start":{"line":212,"column":7},"end":{"line":212,"column":19}},"range":[5510,5522],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","NewExpression","Identifier"],"type":"Identifier","line":false},"286":{"loc":{"start":{"line":212,"column":3},"end":{"line":212,"column":21}},"range":[5506,5524],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","NewExpression"],"type":"NewExpression","line":false},"287":{"loc":{"start":{"line":211,"column":2},"end":{"line":212,"column":21}},"range":[5448,5524],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression"],"type":"ConditionalExpression","line":false},"288":{"loc":{"start":{"line":210,"column":1},"end":{"line":212,"column":21}},"range":[5436,5524],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"289":{"loc":{"start":{"line":214,"column":17},"end":{"line":214,"column":25}},"range":[5544,5552],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression","CallExpression","Literal"],"type":"Literal","line":false},"290":{"loc":{"start":{"line":214,"column":5},"end":{"line":214,"column":26}},"range":[5532,5553],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"291":{"loc":{"start":{"line":215,"column":22},"end":{"line":215,"column":28}},"range":[5577,5583],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Literal"],"type":"Literal","line":false},"292":{"loc":{"start":{"line":215,"column":30},"end":{"line":215,"column":34}},"range":[5585,5589],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"293":{"loc":{"start":{"line":215,"column":9},"end":{"line":215,"column":35}},"range":[5564,5590],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression"],"type":"CallExpression","line":false},"294":{"loc":{"start":{"line":216,"column":25},"end":{"line":216,"column":29}},"range":[5617,5621],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","MemberExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"295":{"loc":{"start":{"line":217,"column":32},"end":{"line":217,"column":35}},"range":[5655,5658],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","MemberExpression","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"296":{"loc":{"start":{"line":217,"column":11},"end":{"line":217,"column":36}},"range":[5634,5659],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","MemberExpression","CallExpression","CallExpression"],"type":"CallExpression","line":false},"297":{"loc":{"start":{"line":218,"column":9},"end":{"line":218,"column":16}},"range":[5670,5677],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","Literal"],"type":"Literal","line":false},"298":{"loc":{"start":{"line":219,"column":19},"end":{"line":219,"column":30}},"range":[5711,5722],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"299":{"loc":{"start":{"line":219,"column":32},"end":{"line":219,"column":36}},"range":[5724,5728],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"300":{"loc":{"start":{"line":219,"column":6},"end":{"line":219,"column":37}},"range":[5698,5729],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"301":{"loc":{"start":{"line":219,"column":6},"end":{"line":219,"column":38}},"range":[5698,5730],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"302":{"loc":{"start":{"line":218,"column":18},"end":{"line":220,"column":6}},"range":[5679,5737],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"303":{"loc":{"start":{"line":216,"column":4},"end":{"line":220,"column":7}},"range":[5596,5738],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","SequenceExpression","CallExpression"],"type":"CallExpression","line":false},"304":{"loc":{"start":{"line":222,"column":13},"end":{"line":222,"column":17}},"range":[5754,5758],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"305":{"loc":{"start":{"line":223,"column":6},"end":{"line":223,"column":9}},"range":[5788,5791],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"306":{"loc":{"start":{"line":223,"column":31},"end":{"line":223,"column":38}},"range":[5813,5820],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Literal"],"type":"Literal","line":false},"307":{"loc":{"start":{"line":223,"column":40},"end":{"line":223,"column":43}},"range":[5822,5825],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"308":{"loc":{"start":{"line":223,"column":45},"end":{"line":223,"column":49}},"range":[5827,5831],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"309":{"loc":{"start":{"line":223,"column":18},"end":{"line":223,"column":50}},"range":[5800,5832],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"310":{"loc":{"start":{"line":225,"column":15},"end":{"line":225,"column":25}},"range":[5850,5860],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"311":{"loc":{"start":{"line":225,"column":27},"end":{"line":225,"column":31}},"range":[5862,5866],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"312":{"loc":{"start":{"line":225,"column":2},"end":{"line":225,"column":32}},"range":[5837,5867],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"313":{"loc":{"start":{"line":229,"column":10},"end":{"line":229,"column":16}},"range":[5913,5919],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"314":{"loc":{"start":{"line":229,"column":17},"end":{"line":229,"column":21}},"range":[5920,5924],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"315":{"loc":{"start":{"line":229,"column":10},"end":{"line":229,"column":22}},"range":[5913,5925],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"316":{"loc":{"start":{"line":229,"column":3},"end":{"line":229,"column":22}},"range":[5906,5925],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"317":{"loc":{"start":{"line":231,"column":20},"end":{"line":231,"column":34}},"range":[5948,5962],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression","CallExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"318":{"loc":{"start":{"line":231,"column":37},"end":{"line":231,"column":45}},"range":[5965,5973],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression","CallExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"319":{"loc":{"start":{"line":231,"column":20},"end":{"line":231,"column":45}},"range":[5948,5973],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression","CallExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"320":{"loc":{"start":{"line":231,"column":6},"end":{"line":231,"column":46}},"range":[5934,5974],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"321":{"loc":{"start":{"line":232,"column":16},"end":{"line":232,"column":24}},"range":[5994,6002],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"322":{"loc":{"start":{"line":232,"column":26},"end":{"line":232,"column":30}},"range":[6004,6008],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"323":{"loc":{"start":{"line":232,"column":3},"end":{"line":232,"column":31}},"range":[5981,6009],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"324":{"loc":{"start":{"line":232,"column":3},"end":{"line":232,"column":32}},"range":[5981,6010],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"325":{"loc":{"start":{"line":236,"column":42},"end":{"line":236,"column":46}},"range":[6074,6078],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"326":{"loc":{"start":{"line":236,"column":48},"end":{"line":236,"column":52}},"range":[6080,6084],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"327":{"loc":{"start":{"line":236,"column":23},"end":{"line":236,"column":53}},"range":[6055,6085],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"328":{"loc":{"start":{"line":236,"column":4},"end":{"line":236,"column":53}},"range":[6036,6085],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"329":{"loc":{"start":{"line":237,"column":11},"end":{"line":237,"column":32}},"range":[6098,6119],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","AssignmentExpression","MemberExpression"],"type":"MemberExpression","line":false},"330":{"loc":{"start":{"line":237,"column":4},"end":{"line":237,"column":32}},"range":[6091,6119],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"331":{"loc":{"start":{"line":236,"column":4},"end":{"line":236,"column":54}},"range":[6036,6086],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"332":{"loc":{"start":{"line":237,"column":4},"end":{"line":237,"column":33}},"range":[6091,6120],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"333":{"loc":{"start":{"line":240,"column":17},"end":{"line":240,"column":34}},"range":[6158,6175],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"334":{"loc":{"start":{"line":240,"column":36},"end":{"line":240,"column":39}},"range":[6177,6180],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"335":{"loc":{"start":{"line":240,"column":41},"end":{"line":240,"column":45}},"range":[6182,6186],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"336":{"loc":{"start":{"line":240,"column":4},"end":{"line":240,"column":46}},"range":[6145,6187],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"337":{"loc":{"start":{"line":240,"column":4},"end":{"line":240,"column":47}},"range":[6145,6188],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement","CatchClause","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"338":{"loc":{"start":{"line":235,"column":3},"end":{"line":241,"column":4}},"range":[6026,6193],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","TryStatement"],"type":"TryStatement","line":true},"339":{"loc":{"start":{"line":244,"column":15},"end":{"line":244,"column":18}},"range":[6214,6217],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"340":{"loc":{"start":{"line":244,"column":20},"end":{"line":244,"column":24}},"range":[6219,6223],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","LogicalExpression","Identifier"],"type":"Identifier","line":false},"341":{"loc":{"start":{"line":244,"column":28},"end":{"line":244,"column":32}},"range":[6227,6231],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","LogicalExpression","Identifier"],"type":"Identifier","line":false},"342":{"loc":{"start":{"line":244,"column":20},"end":{"line":244,"column":32}},"range":[6219,6231],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","LogicalExpression"],"type":"LogicalExpression","line":false},"343":{"loc":{"start":{"line":245,"column":7},"end":{"line":245,"column":10}},"range":[6256,6259],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","Identifier"],"type":"Identifier","line":false},"344":{"loc":{"start":{"line":245,"column":32},"end":{"line":245,"column":39}},"range":[6281,6288],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Literal"],"type":"Literal","line":false},"345":{"loc":{"start":{"line":245,"column":41},"end":{"line":245,"column":44}},"range":[6290,6293],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"346":{"loc":{"start":{"line":245,"column":46},"end":{"line":245,"column":50}},"range":[6295,6299],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"347":{"loc":{"start":{"line":245,"column":19},"end":{"line":245,"column":51}},"range":[6268,6300],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"348":{"loc":{"start":{"line":247,"column":7},"end":{"line":247,"column":23}},"range":[6310,6326],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","Identifier"],"type":"Identifier","line":false},"349":{"loc":{"start":{"line":247,"column":27},"end":{"line":247,"column":47}},"range":[6330,6350],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","MemberExpression"],"type":"MemberExpression","line":false},"350":{"loc":{"start":{"line":247,"column":7},"end":{"line":247,"column":47}},"range":[6310,6350],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"351":{"loc":{"start":{"line":248,"column":17},"end":{"line":248,"column":22}},"range":[6371,6376],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"352":{"loc":{"start":{"line":248,"column":24},"end":{"line":248,"column":44}},"range":[6378,6398],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"353":{"loc":{"start":{"line":248,"column":4},"end":{"line":248,"column":45}},"range":[6358,6399],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"354":{"loc":{"start":{"line":248,"column":4},"end":{"line":248,"column":46}},"range":[6358,6400],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"355":{"loc":{"start":{"line":251,"column":16},"end":{"line":251,"column":27}},"range":[6423,6434],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Literal"],"type":"Literal","line":false},"356":{"loc":{"start":{"line":251,"column":29},"end":{"line":251,"column":33}},"range":[6436,6440],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"357":{"loc":{"start":{"line":251,"column":3},"end":{"line":251,"column":34}},"range":[6410,6441],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"358":{"loc":{"start":{"line":245,"column":3},"end":{"line":245,"column":52}},"range":[6252,6301],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"359":{"loc":{"start":{"line":247,"column":3},"end":{"line":249,"column":4}},"range":[6306,6405],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"360":{"loc":{"start":{"line":251,"column":3},"end":{"line":251,"column":35}},"range":[6410,6442],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"361":{"loc":{"start":{"line":244,"column":34},"end":{"line":252,"column":3}},"range":[6233,6446],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"362":{"loc":{"start":{"line":244,"column":2},"end":{"line":252,"column":4}},"range":[6201,6447],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"363":{"loc":{"start":{"line":223,"column":2},"end":{"line":223,"column":51}},"range":[5784,5833],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"364":{"loc":{"start":{"line":225,"column":2},"end":{"line":225,"column":33}},"range":[5837,5868],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"365":{"loc":{"start":{"line":227,"column":2},"end":{"line":228,"column":8}},"range":[5872,5902],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"366":{"loc":{"start":{"line":229,"column":3},"end":{"line":229,"column":23}},"range":[5906,5926],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"367":{"loc":{"start":{"line":231,"column":2},"end":{"line":242,"column":3}},"range":[5930,6197],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"368":{"loc":{"start":{"line":244,"column":2},"end":{"line":252,"column":5}},"range":[6201,6448],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"369":{"loc":{"start":{"line":222,"column":19},"end":{"line":253,"column":2}},"range":[5760,6451],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"370":{"loc":{"start":{"line":222,"column":1},"end":{"line":253,"column":3}},"range":[5742,6452],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"371":{"loc":{"start":{"line":255,"column":8},"end":{"line":255,"column":15}},"range":[6463,6470],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"372":{"loc":{"start":{"line":210,"column":1},"end":{"line":212,"column":22}},"range":[5436,5525],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"373":{"loc":{"start":{"line":214,"column":1},"end":{"line":220,"column":8}},"range":[5528,5739],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"374":{"loc":{"start":{"line":222,"column":1},"end":{"line":253,"column":4}},"range":[5742,6453],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"375":{"loc":{"start":{"line":255,"column":1},"end":{"line":255,"column":16}},"range":[6456,6471],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"376":{"loc":{"start":{"line":208,"column":29},"end":{"line":256,"column":1}},"range":[5391,6473],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"377":{"loc":{"start":{"line":208,"column":0},"end":{"line":256,"column":1}},"range":[5362,6473],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"378":{"loc":{"start":{"line":1,"column":0},"end":{"line":1,"column":13}},"range":[0,13],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"379":{"loc":{"start":{"line":3,"column":0},"end":{"line":8,"column":47}},"range":[15,210],"results":[],"stack":["Program","VariableDeclaration"],"type":"VariableDeclaration","line":true},"380":{"loc":{"start":{"line":29,"column":0},"end":{"line":113,"column":2}},"range":[702,3060],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"381":{"loc":{"start":{"line":135,"column":0},"end":{"line":184,"column":2}},"range":[3548,4727],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"382":{"loc":{"start":{"line":208,"column":0},"end":{"line":256,"column":2}},"range":[5362,6474],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true}},"comments":[[212,700],[817,856],[858,914],[1473,1529],[2313,2384],[2386,2458],[2460,2532],[2534,2593],[3062,3546],[4729,5360]]};
}
function instrumentor_record(filetag, id, expression) {
	var scope = global || window;
	scope.st_timeCommenced = scope.st_timeCommenced || Date.now();
	scope.st_iterator = (scope.st_iterator || 0);

	var depth =
		(new Error()).stack.split("\n")
			.slice(1)
			.length;

	if (!scope.mem)
		scope.mem = (
			typeof process !== "undefined" ? process.memoryUsage : function(){}
		);

	__instrumentor_map__[filetag].instruments[id].results.push({
		"depth": depth,
		"time": Date.now(),
		"timeOffset": Date.now() - scope.st_timeCommenced,
		"memoryUsage": scope.mem(),
		"invocation": scope.st_iterator,
		"milestone": scope.__steamshovel_milestone,
		"value": scope.__steamshovel_record_expression ? expression : null
	});

	scope.st_iterator ++;

	// Make sure we return the original expresison result
	return expression;
}
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 378);
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 1, 'use strict');
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 379);
var fs = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 4, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 2, require)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 3, 'fs'))), path = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 7, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 5, require)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 6, 'path'))), mkdirp = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 10, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 8, require)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 9, 'mkdirp'))), instrument = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 13, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 11, require)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 12, './instrumentor'))), assert = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 16, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 14, require)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 15, 'assert'))), EventEmitter = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 19, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 17, require)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 18, 'events')).EventEmitter);
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 380);
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 170, module.exports = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 169, function instrumentTree(inFile, outFile, emitter) {
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 160);
    var combinedMap = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 20, {}), isDirectory = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 21, false);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 161);
    var firstCall = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 22, !emitter);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 162);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 32, emitter = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 31, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 27, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 23, emitter) && instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 26, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 24, emitter) instanceof instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 25, EventEmitter))) ? instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 28, emitter) : instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 30, new (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 29, EventEmitter))())));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 163);
    if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 33, !inFile))
        throw instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 36, new (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 34, Error))(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 35, 'You must specify an input file/directory.')));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 164);
    if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 37, !outFile))
        throw instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 40, new (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 38, Error))(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 39, 'You must specify an output file/directory.')));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 165);
    if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 43, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 41, inFile) instanceof instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 42, Array))) {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 67);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 51, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 44, assert)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 47, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 45, outFile) instanceof instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 46, Array)), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 50, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 48, 'If an array if input resources is provided, ') + instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 49, 'the output must also be an array.'))));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 68);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 57, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 52, assert)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 55, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 53, outFile.length) === instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 54, inFile.length)), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 56, 'The lengths of the input and destination arrays must match.')));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 69);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 65, inFile.forEach(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 64, function (file, index) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 63);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 62, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 58, instrumentTree)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 59, file), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 60, outFile[index]), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 61, emitter)));
        })));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 70);
        return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 66, emitter);
    } else {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 77);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 76, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 71, assert)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 74, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 72, typeof outFile) === instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 73, 'string')), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 75, 'If the first parameter is not an array, the second mustn\'t be.')));
    }
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 166);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 125, fs.stat(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 78, inFile), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 124, function (err, stats) {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 120);
        if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 79, err))
            return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 83, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 80, 'error'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 81, err), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 82, inFile)));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 121);
        if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 84, stats.isDirectory())) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 102);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 86, isDirectory = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 85, true));
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 103);
            if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 87, firstCall))
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 97, emitter.on(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 88, 'dircomplete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 96, function (dir) {
                    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 95);
                    if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 91, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 89, dir) === instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 90, inFile)))
                        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 94, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 92, '_int_complete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 93, inFile)));
                })));
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 104);
            return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 101, module.exports.processDir(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 98, inFile), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 99, outFile), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 100, emitter)));
        }
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 122);
        if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 105, firstCall))
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 115, emitter.on(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 106, 'writefile'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 114, function (file) {
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 113);
                if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 109, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 107, file) === instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 108, inFile)))
                    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 112, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 110, 'complete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 111, inFile)));
            })));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 123);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 119, module.exports.processFile(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 116, inFile), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 117, outFile), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 118, emitter)));
    })));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 167);
    if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 126, firstCall)) {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 157);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 132, emitter.on(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 127, 'map'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 131, function (sourceMap) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 130);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 129, combinedMap[sourceMap.filetag] = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 128, sourceMap));
        })));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 158);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 156, emitter.on(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 133, '_int_complete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 155, function (inFile) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 153);
            var writePath = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 136, path.join(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 134, outFile), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 135, './instruments.shovelmap')));
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 154);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 152, fs.writeFile(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 137, writePath), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 141, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 138, '__instrumentor_map__ = ') + instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 140, JSON.stringify(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 139, combinedMap)))), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 151, function (err) {
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 149);
                if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 142, err))
                    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 145, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 143, 'maperror'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 144, err)));
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 150);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 148, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 146, 'complete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 147, inFile)));
            })));
        })));
    }
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 168);
    return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 159, emitter);
}));
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 381);
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 278, module.exports.processDir = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 277, function processDir(dir, out, emitter) {
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 272);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 180, emitter = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 179, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 175, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 171, emitter) && instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 174, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 172, emitter) instanceof instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 173, EventEmitter))) ? instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 176, emitter) : instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 178, new (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 177, EventEmitter))())));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 273);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 184, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 181, 'processdir'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 182, dir), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 183, out)));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 274);
    try {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 187);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 186, fs.statSync(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 185, out)));
    } catch (e) {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 193);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 190, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 188, 'mkdir'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 189, out)));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 194);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 192, mkdirp.mkdirp(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 191, out)));
    }
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 275);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 270, fs.readdir(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 195, dir), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 269, function (err, dirContents) {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 262);
        if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 196, err))
            return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 200, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 197, 'error'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 198, err), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 199, dir)));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 263);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 203, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 201, 'readdir'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 202, dir)));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 264);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 217, dirContents.forEach(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 216, function (file) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 214);
            var filePath = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 206, path.join(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 204, dir), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 205, file))), outPath = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 209, path.join(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 207, out), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 208, file)));
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 215);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 213, module.exports(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 210, filePath), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 211, outPath), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 212, emitter)));
        })));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 265);
        var instrumentedFiles = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 218, 0);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 266);
        function checkComplete(file) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 253);
            if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 221, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 219, file) instanceof instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 220, Error)))
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 223, file = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 222, arguments[1]));
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 254);
            if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 236, dirContents.reduce(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 234, function (acc, cur) {
                    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 233);
                    return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 232, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 224, acc) || instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 231, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 229, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 227, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 225, dir) + instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 226, '/')) + instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 228, cur)) === instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 230, file)));
                }), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 235, false))))
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 237, instrumentedFiles++);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 255);
            if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 240, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 238, instrumentedFiles) === instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 239, dirContents.length))) {
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 250);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 243, emitter.removeListener(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 241, 'writefile'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 242, checkComplete)));
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 251);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 246, emitter.removeListener(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 244, 'dircomplete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 245, checkComplete)));
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 252);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 249, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 247, 'dircomplete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 248, dir)));
            }
        }
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 267);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 258, emitter.on(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 256, 'writefile'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 257, checkComplete)));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 268);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 261, emitter.on(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 259, 'dircomplete'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 260, checkComplete)));
    })));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 276);
    return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 271, emitter);
}));
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 382);
instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 377, module.exports.processFile = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 376, function processFile(file, out, emitter) {
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 372);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 288, emitter = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 287, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 283, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 279, emitter) && instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 282, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 280, emitter) instanceof instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 281, EventEmitter))) ? instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 284, emitter) : instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 286, new (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 285, EventEmitter))())));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 373);
    if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 290, !file.match(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 289, /\.js$/i))))
        return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 293, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 291, 'nojs'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 292, file))), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 303, fs.createReadStream(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 294, file)).pipe(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 296, fs.createWriteStream(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 295, out)))).on(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 297, 'close'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 302, function () {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 301);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 300, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 298, 'writefile'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 299, file)));
        })));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 374);
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 370, fs.readFile(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 304, file), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 369, function (err, data) {
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 363);
        if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 305, err))
            return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 309, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 306, 'error'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 307, err), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 308, file)));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 364);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 312, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 310, 'readfile'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 311, file)));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 365);
        var instrumentResult, code;
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 366);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 316, data = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 315, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 313, String)(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 314, data))));
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 367);
        if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 320, ~data.indexOf(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 319, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 317, 'steamShovel:') + instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 318, 'ignore'))))) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 324);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 323, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 321, 'ignore'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 322, file)));
        } else {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 338);
            try {
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 331);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 328, instrumentResult = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 327, instrument.withmap(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 325, data), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 326, file))));
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 332);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 330, code = instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 329, instrumentResult.code));
            } catch (err) {
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 337);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 336, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 333, 'instrumenterror'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 334, err), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 335, file)));
            }
        }
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 368);
        instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 362, fs.writeFile(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 339, out), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 342, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 340, code) || instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 341, data)), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 361, function (err) {
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 358);
            if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 343, err))
                return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 347, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 344, 'error'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 345, err), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 346, file)));
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 359);
            if (instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 350, instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 348, instrumentResult) && instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 349, instrumentResult.map))) {
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 354);
                instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 353, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 351, 'map'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 352, instrumentResult.map)));
            }
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 360);
            instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 357, emitter.emit(instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 355, 'writefile'), instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 356, file)));
        })));
    })));
    instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 375);
    return instrumentor_record('ibdd0ef4c9ff8ae316997d80042bcc53d', 371, emitter);
}));