if (typeof __instrumentor_map__ === 'undefined') {
__instrumentor_map__ = {};
}
if (typeof __instrumentor_map__.i4283e89b3baf979028afccb2c82cfaf3 === 'undefined') {
__instrumentor_map__.i4283e89b3baf979028afccb2c82cfaf3 = {"filetag":"i4283e89b3baf979028afccb2c82cfaf3","filename":"lib/stats/basic.js","source":"\"use strict\";\n\nvar util = require(\"./utils\");\n\n\n/*\n\tPublic: Given a map of instrument data, this function returns a map of\n\tgenerated statistics regarding coverage, split into four groups:\n\tlines, conditions, expressions, and functions. This is a convenience\n\tfunction which simply calls the other four functions in this file.\n\n\tFor more detail on how each statistic is calculated, refer to the notes\n\tagainst each function.\n\n\tinstrumentData\t-\tThe map of instrument data.\n\tfiletag\t\t\t-\tSpecify the tag associated with a specific file,\n\t\t\t\t\t\tallowing statistics to be scoped a single file.\n\n\tReturns a map of maps containing statistics grouped into the categories\n\tlines, conditions, expressions, and functions.\n\n\tExamples\n\n\t\tstats = stats.basic(instrumentData);\n\n*/\nmodule.exports = function generateStatistics(instrumentData, key) {\n\tvar self = module.exports;\n\n\tif (!instrumentData)\n\t\tthrow new Error(\"Input was undefined.\");\n\n\treturn {\n\t\t\"lines\":\t\tself.lines(instrumentData, key),\n\t\t\"conditions\":\tself.conditions(instrumentData, key),\n\t\t\"expressions\":\tself.expressions(instrumentData, key),\n\t\t\"functions\":\tself.functions(instrumentData, key)\n\t};\n};\n\n/*\n\tPublic: Given a map of instrument data, this function returns a map of\n\tgenerated statistics regarding code coverage on a per-line level.\n\n\tThis is ascertained by filtering the probes which were identified at\n\tthe time of instrumentation as 'line' probes — that is, they don't execute\n\tin expression context, and are tied to statements, rather than expressions.\n\n\tinstrumentData\t-\tThe map of instrument data.\n\tfiletag\t\t\t-\tSpecify the tag associated with a specific file,\n\t\t\t\t\t\tallowing statistics to be scoped a single file.\n\n\tReturns a map of the coverage statistics associated with these probes. The\n\tmap contains a count of the number of probes covered, a total number of\n\tprobes, and a percentage of probes covered.\n\n\tFor example:\n\n\t\t{ covered: 15, total: 25, percentage: 0.6 }\n\n\tExamples\n\n\t\tstats = stats.basic.lines(instrumentData);\n\n*/\nmodule.exports.lines = function(instrumentData, key) {\n\tvar totalInstruments, coveredInstruments, covered, total = 0,\n\t\tgetValue = util.calculateValue(instrumentData);\n\n\ttotalInstruments =\n\t\tutil.getInstruments(instrumentData, key)\n\t\t\t.filter(util.withkey(\"line\"));\n\n\ttotal = totalInstruments.length;\n\n\tcoveredInstruments =\n\t\ttotalInstruments.filter(util.hasResults);\n\n\tcovered =\n\t\tcoveredInstruments\n\t\t\t.reduce(function(count, instrument) {\n\t\t\t\treturn count + getValue(instrument);\n\t\t\t}, 0);\n\n\treturn {\n\t\tcovered: covered,\n\t\ttotal: total,\n\t\tpercentage: util.percentage(covered, total)\n\t};\n};\n\n/*\n\tPublic: Given a map of instrument data, this function returns a map of\n\tgenerated statistics regarding code coverage on a per-expression level.\n\n\tThis is ascertained by filtering the probes which were identified at\n\tthe time of instrumentation as 'expression' probes — that is, they execute\n\tin expression context, and are tied to expressions, rather than statements.\n\n\tinstrumentData\t-\tThe map of instrument data.\n\tfiletag\t\t\t-\tSpecify the tag associated with a specific file,\n\t\t\t\t\t\tallowing statistics to be scoped a single file.\n\n\tReturns a map of the coverage statistics associated with these probes. The\n\tmap contains a count of the number of probes covered, a total number of\n\tprobes, and a percentage of probes covered.\n\n\tFor example:\n\n\t\t{ covered: 15, total: 25, percentage: 0.6 }\n\n\tExamples\n\n\t\tstats = stats.basic.expressions(instrumentData);\n\n*/\nmodule.exports.expressions = function(inputData, key) {\n\tvar covered = 0, total = 0,\n\t\tgetValue = util.calculateValue(inputData),\n\t\tinstruments =\n\t\t\tutil.getInstruments(inputData, key)\n\t\t\t\t.filter(function(item) {\n\t\t\t\t\treturn !item.line;\n\t\t\t\t});\n\n\ttotal = instruments.length;\n\tcovered =\n\t\tinstruments\n\t\t\t.filter(util.hasResults)\n\t\t\t.reduce(function(count, item) {\n\t\t\t\treturn count + getValue(item);\n\t\t\t}, 0);\n\n\treturn {\n\t\tcovered: covered,\n\t\ttotal: total,\n\t\tpercentage: util.percentage(covered, total)\n\t};\n};\n\n/*\n\tPublic: Given a map of instrument data, this function returns a map of\n\tgenerated statistics regarding code coverage within conditional statements\n\tin the instrumented code.\n\n\tThis is ascertained by filtering the probes which were identified at\n\tthe time of instrumentation as being conditional expressions or statements,\n\tor exist within conditional expressions or statements.\n\n\tIn this instance, the 'total' value returned is the number of probes which\n\tcorrespond *explicitly* to conditionals in the document. However, the\n\tcoverage value returned is calculated based on the coverage of the probes\n\tdirectly associated with these conditionals *as well as* the probes for the\n\texpressions within them.\n\n\tIn order for a conditional to be considered completely tested, all the\n\tcodepaths within it must have been evaluated at appropriate depth.\n\n\tinstrumentData\t-\tThe map of instrument data.\n\tfiletag\t\t\t-\tSpecify the tag associated with a specific file,\n\t\t\t\t\t\tallowing statistics to be scoped a single file.\n\n\tReturns a map of the coverage statistics associated with these probes. The\n\tmap contains a count of the number of probes covered, a total number of\n\tprobes, and a percentage of probes covered.\n\n\tFor example:\n\n\t\t{ covered: 15, total: 25, percentage: 0.6 }\n\n\tExamples\n\n\t\tstats = stats.basic.conditions(instrumentData);\n\n*/\nmodule.exports.conditions = function(inputData, key) {\n\tvar covered = 0, total = 0,\n\t\tgetValue = util.calculateValue(inputData),\n\t\tinstruments =\n\t\t\tutil.getInstruments(inputData, key)\n\t\t\t\t.filter(util.isOfType([\n\t\t\t\t\t\t\"IfStatement\",\n\t\t\t\t\t\t\"LogicalExpression\",\n\t\t\t\t\t\t\"ConditionalExpression\"\n\t\t\t\t\t]));\n\n\tvar strictTotal =\n\t\tinstruments.filter(function(instrument) {\n\t\t\treturn \t~[\n\t\t\t\t\t\t\"IfStatement\",\n\t\t\t\t\t\t\"LogicalExpression\",\n\t\t\t\t\t\t\"ConditionalExpression\"\n\t\t\t\t\t]\n\t\t\t\t\t.indexOf(instrument.type);\n\t\t})\n\t\t.length;\n\n\ttotal = instruments.length;\n\tcovered =\n\t\tinstruments\n\t\t\t.filter(util.hasResults)\n\t\t\t.reduce(function(count, instrument) {\n\t\t\t\treturn count + getValue(instrument);\n\t\t\t}, 0);\n\n\treturn {\n\t\tcovered: covered * (strictTotal / total),\n\t\ttotal: strictTotal,\n\t\tpercentage: util.percentage(covered, total)\n\t};\n};\n\n/*\n\tPublic: Given a map of instrument data, this function returns a map of\n\tgenerated statistics regarding code coverage within function declarations\n\tand expressions in the instrumented code.\n\n\tThis is ascertained by filtering the probes which were identified at\n\tthe time of instrumentation as being function expressions or declarations,\n\tor exist within function expressions or declarations.\n\n\tIn this instance, the 'total' value returned is the number of probes which\n\tcorrespond *explicitly* to functions in the document. However, the coverage\n\tvalue returned is calculated based on the coverage of the probes directly\n\tassociated with these declarations and expressions *as well as* the probes\n\tfor the expressions within them.\n\n\tIn order for a function to be considered completely tested, all the\n\tcodepaths within it must have been evaluated at appropriate depth.\n\n\tinstrumentData\t-\tThe map of instrument data.\n\tfiletag\t\t\t-\tSpecify the tag associated with a specific file,\n\t\t\t\t\t\tallowing statistics to be scoped a single file.\n\n\tReturns a map of the coverage statistics associated with these probes. The\n\tmap contains a count of the number of probes covered, a total number of\n\tprobes, and a percentage of probes covered.\n\n\tFor example:\n\n\t\t{ covered: 15, total: 25, percentage: 0.6 }\n\n\tExamples\n\n\t\tstats = stats.basic.conditions(instrumentData);\n\n*/\nmodule.exports.functions = function(inputData, key) {\n\tvar covered = 0, total = 0,\n\t\tgetValue = util.calculateValue(inputData),\n\t\tinstruments =\n\t\t\tutil.getInstruments(inputData, key)\n\t\t\t\t.filter(util.isOfType([\n\t\t\t\t\t\t\"FunctionDeclaration\",\n\t\t\t\t\t\t\"FunctionExpression\"\n\t\t\t\t\t]));\n\n\tvar strictTotal =\n\t\tinstruments.filter(function(instrument) {\n\t\t\treturn \t~[\n\t\t\t\t\t\t\"FunctionDeclaration\",\n\t\t\t\t\t\t\"FunctionExpression\"\n\t\t\t\t\t]\n\t\t\t\t\t.indexOf(instrument.type);\n\t\t})\n\t\t.length;\n\n\ttotal = instruments.length;\n\n\tcovered =\n\t\tinstruments\n\t\t\t.filter(util.hasResults)\n\t\t\t.reduce(function(count, item) {\n\t\t\t\treturn count + getValue(item);\n\t\t\t}, 0);\n\n\treturn {\n\t\tcovered: covered  * (strictTotal / total),\n\t\ttotal: strictTotal,\n\t\tpercentage: util.percentage(covered, total)\n\t};\n};","instruments":{"1":{"loc":{"start":{"line":1,"column":0},"end":{"line":1,"column":12}},"range":[0,12],"results":[],"stack":["Program","ExpressionStatement","Literal"],"type":"Literal","line":false},"2":{"loc":{"start":{"line":3,"column":11},"end":{"line":3,"column":18}},"range":[26,33],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"3":{"loc":{"start":{"line":3,"column":19},"end":{"line":3,"column":28}},"range":[34,43],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression","Literal"],"type":"Literal","line":false},"4":{"loc":{"start":{"line":3,"column":11},"end":{"line":3,"column":29}},"range":[26,44],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"5":{"loc":{"start":{"line":28,"column":12},"end":{"line":28,"column":26}},"range":[845,859],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"6":{"loc":{"start":{"line":30,"column":5},"end":{"line":30,"column":20}},"range":[867,882],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"7":{"loc":{"start":{"line":31,"column":12},"end":{"line":31,"column":17}},"range":[896,901],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression","Identifier"],"type":"Identifier","line":false},"8":{"loc":{"start":{"line":31,"column":18},"end":{"line":31,"column":40}},"range":[902,924],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression","Literal"],"type":"Literal","line":false},"9":{"loc":{"start":{"line":31,"column":8},"end":{"line":31,"column":41}},"range":[892,925],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ThrowStatement","NewExpression"],"type":"NewExpression","line":false},"10":{"loc":{"start":{"line":34,"column":23},"end":{"line":34,"column":37}},"range":[961,975],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"11":{"loc":{"start":{"line":34,"column":39},"end":{"line":34,"column":42}},"range":[977,980],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"12":{"loc":{"start":{"line":34,"column":12},"end":{"line":34,"column":43}},"range":[950,981],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"13":{"loc":{"start":{"line":35,"column":32},"end":{"line":35,"column":46}},"range":[1015,1029],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"14":{"loc":{"start":{"line":35,"column":48},"end":{"line":35,"column":51}},"range":[1031,1034],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"15":{"loc":{"start":{"line":35,"column":16},"end":{"line":35,"column":52}},"range":[999,1035],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"16":{"loc":{"start":{"line":36,"column":34},"end":{"line":36,"column":48}},"range":[1071,1085],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"17":{"loc":{"start":{"line":36,"column":50},"end":{"line":36,"column":53}},"range":[1087,1090],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"18":{"loc":{"start":{"line":36,"column":17},"end":{"line":36,"column":54}},"range":[1054,1091],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"19":{"loc":{"start":{"line":37,"column":30},"end":{"line":37,"column":44}},"range":[1123,1137],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"20":{"loc":{"start":{"line":37,"column":46},"end":{"line":37,"column":49}},"range":[1139,1142],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"21":{"loc":{"start":{"line":37,"column":15},"end":{"line":37,"column":50}},"range":[1108,1143],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"22":{"loc":{"start":{"line":33,"column":8},"end":{"line":38,"column":2}},"range":[936,1146],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"23":{"loc":{"start":{"line":28,"column":1},"end":{"line":28,"column":27}},"range":[834,860],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"24":{"loc":{"start":{"line":30,"column":1},"end":{"line":31,"column":42}},"range":[863,926],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"25":{"loc":{"start":{"line":33,"column":1},"end":{"line":38,"column":3}},"range":[929,1147],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"26":{"loc":{"start":{"line":27,"column":17},"end":{"line":39,"column":1}},"range":[782,1149],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"27":{"loc":{"start":{"line":27,"column":0},"end":{"line":39,"column":1}},"range":[765,1149],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"28":{"loc":{"start":{"line":67,"column":60},"end":{"line":67,"column":61}},"range":[2114,2115],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"29":{"loc":{"start":{"line":68,"column":33},"end":{"line":68,"column":47}},"range":[2150,2164],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"30":{"loc":{"start":{"line":68,"column":13},"end":{"line":68,"column":48}},"range":[2130,2165],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"31":{"loc":{"start":{"line":71,"column":22},"end":{"line":71,"column":36}},"range":[2210,2224],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"32":{"loc":{"start":{"line":71,"column":38},"end":{"line":71,"column":41}},"range":[2226,2229],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"33":{"loc":{"start":{"line":72,"column":24},"end":{"line":72,"column":30}},"range":[2255,2261],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","CallExpression","Literal"],"type":"Literal","line":false},"34":{"loc":{"start":{"line":72,"column":11},"end":{"line":72,"column":31}},"range":[2242,2262],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","CallExpression"],"type":"CallExpression","line":false},"35":{"loc":{"start":{"line":71,"column":2},"end":{"line":72,"column":32}},"range":[2190,2263],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"36":{"loc":{"start":{"line":70,"column":1},"end":{"line":72,"column":32}},"range":[2169,2263],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"37":{"loc":{"start":{"line":74,"column":9},"end":{"line":74,"column":32}},"range":[2275,2298],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","MemberExpression"],"type":"MemberExpression","line":false},"38":{"loc":{"start":{"line":74,"column":1},"end":{"line":74,"column":32}},"range":[2267,2298],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"39":{"loc":{"start":{"line":77,"column":26},"end":{"line":77,"column":41}},"range":[2349,2364],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"40":{"loc":{"start":{"line":77,"column":2},"end":{"line":77,"column":42}},"range":[2325,2365],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"41":{"loc":{"start":{"line":76,"column":1},"end":{"line":77,"column":42}},"range":[2302,2365],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"42":{"loc":{"start":{"line":82,"column":11},"end":{"line":82,"column":16}},"range":[2452,2457],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"43":{"loc":{"start":{"line":82,"column":19},"end":{"line":82,"column":27}},"range":[2460,2468],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"44":{"loc":{"start":{"line":82,"column":28},"end":{"line":82,"column":38}},"range":[2469,2479],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"45":{"loc":{"start":{"line":82,"column":19},"end":{"line":82,"column":39}},"range":[2460,2480],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression"],"type":"CallExpression","line":false},"46":{"loc":{"start":{"line":82,"column":11},"end":{"line":82,"column":39}},"range":[2452,2480],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"47":{"loc":{"start":{"line":82,"column":4},"end":{"line":82,"column":40}},"range":[2445,2481],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"48":{"loc":{"start":{"line":81,"column":11},"end":{"line":83,"column":4}},"range":[2411,2486],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"49":{"loc":{"start":{"line":83,"column":6},"end":{"line":83,"column":7}},"range":[2488,2489],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Literal"],"type":"Literal","line":false},"50":{"loc":{"start":{"line":80,"column":2},"end":{"line":83,"column":8}},"range":[2381,2490],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"51":{"loc":{"start":{"line":79,"column":1},"end":{"line":83,"column":8}},"range":[2369,2490],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"52":{"loc":{"start":{"line":86,"column":11},"end":{"line":86,"column":18}},"range":[2514,2521],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"53":{"loc":{"start":{"line":87,"column":9},"end":{"line":87,"column":14}},"range":[2532,2537],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"54":{"loc":{"start":{"line":88,"column":30},"end":{"line":88,"column":37}},"range":[2569,2576],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"55":{"loc":{"start":{"line":88,"column":39},"end":{"line":88,"column":44}},"range":[2578,2583],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"56":{"loc":{"start":{"line":88,"column":14},"end":{"line":88,"column":45}},"range":[2553,2584],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"57":{"loc":{"start":{"line":85,"column":8},"end":{"line":89,"column":2}},"range":[2501,2587],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"58":{"loc":{"start":{"line":67,"column":1},"end":{"line":68,"column":49}},"range":[2055,2166],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"59":{"loc":{"start":{"line":70,"column":1},"end":{"line":72,"column":33}},"range":[2169,2264],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"60":{"loc":{"start":{"line":74,"column":1},"end":{"line":74,"column":33}},"range":[2267,2299],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"61":{"loc":{"start":{"line":76,"column":1},"end":{"line":77,"column":43}},"range":[2302,2366],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"62":{"loc":{"start":{"line":79,"column":1},"end":{"line":83,"column":9}},"range":[2369,2491],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"63":{"loc":{"start":{"line":85,"column":1},"end":{"line":89,"column":3}},"range":[2494,2588],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"64":{"loc":{"start":{"line":66,"column":23},"end":{"line":90,"column":1}},"range":[2022,2590],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"65":{"loc":{"start":{"line":66,"column":0},"end":{"line":90,"column":1}},"range":[1999,2590],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"66":{"loc":{"start":{"line":118,"column":15},"end":{"line":118,"column":16}},"range":[3523,3524],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"67":{"loc":{"start":{"line":118,"column":26},"end":{"line":118,"column":27}},"range":[3534,3535],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"68":{"loc":{"start":{"line":119,"column":33},"end":{"line":119,"column":42}},"range":[3570,3579],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"69":{"loc":{"start":{"line":119,"column":13},"end":{"line":119,"column":43}},"range":[3550,3580],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"70":{"loc":{"start":{"line":121,"column":23},"end":{"line":121,"column":32}},"range":[3621,3630],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"71":{"loc":{"start":{"line":121,"column":34},"end":{"line":121,"column":37}},"range":[3632,3635],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"72":{"loc":{"start":{"line":123,"column":12},"end":{"line":123,"column":22}},"range":[3678,3688],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"73":{"loc":{"start":{"line":123,"column":5},"end":{"line":123,"column":23}},"range":[3671,3689],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"74":{"loc":{"start":{"line":122,"column":12},"end":{"line":124,"column":5}},"range":[3649,3695],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"75":{"loc":{"start":{"line":121,"column":3},"end":{"line":124,"column":6}},"range":[3601,3696],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"76":{"loc":{"start":{"line":126,"column":9},"end":{"line":126,"column":27}},"range":[3708,3726],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","MemberExpression"],"type":"MemberExpression","line":false},"77":{"loc":{"start":{"line":126,"column":1},"end":{"line":126,"column":27}},"range":[3700,3726],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"78":{"loc":{"start":{"line":129,"column":11},"end":{"line":129,"column":26}},"range":[3764,3779],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"79":{"loc":{"start":{"line":131,"column":11},"end":{"line":131,"column":16}},"range":[3827,3832],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"80":{"loc":{"start":{"line":131,"column":19},"end":{"line":131,"column":27}},"range":[3835,3843],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"81":{"loc":{"start":{"line":131,"column":28},"end":{"line":131,"column":32}},"range":[3844,3848],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"82":{"loc":{"start":{"line":131,"column":19},"end":{"line":131,"column":33}},"range":[3835,3849],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression"],"type":"CallExpression","line":false},"83":{"loc":{"start":{"line":131,"column":11},"end":{"line":131,"column":33}},"range":[3827,3849],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"84":{"loc":{"start":{"line":131,"column":4},"end":{"line":131,"column":34}},"range":[3820,3850],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"85":{"loc":{"start":{"line":130,"column":11},"end":{"line":132,"column":4}},"range":[3792,3855],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"86":{"loc":{"start":{"line":132,"column":6},"end":{"line":132,"column":7}},"range":[3857,3858],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Literal"],"type":"Literal","line":false},"87":{"loc":{"start":{"line":128,"column":2},"end":{"line":132,"column":8}},"range":[3741,3859],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"88":{"loc":{"start":{"line":127,"column":1},"end":{"line":132,"column":8}},"range":[3729,3859],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"89":{"loc":{"start":{"line":135,"column":11},"end":{"line":135,"column":18}},"range":[3883,3890],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"90":{"loc":{"start":{"line":136,"column":9},"end":{"line":136,"column":14}},"range":[3901,3906],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"91":{"loc":{"start":{"line":137,"column":30},"end":{"line":137,"column":37}},"range":[3938,3945],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"92":{"loc":{"start":{"line":137,"column":39},"end":{"line":137,"column":44}},"range":[3947,3952],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"93":{"loc":{"start":{"line":137,"column":14},"end":{"line":137,"column":45}},"range":[3922,3953],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"94":{"loc":{"start":{"line":134,"column":8},"end":{"line":138,"column":2}},"range":[3870,3956],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"95":{"loc":{"start":{"line":118,"column":1},"end":{"line":124,"column":7}},"range":[3509,3697],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"96":{"loc":{"start":{"line":126,"column":1},"end":{"line":126,"column":28}},"range":[3700,3727],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"97":{"loc":{"start":{"line":127,"column":1},"end":{"line":132,"column":9}},"range":[3729,3860],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"98":{"loc":{"start":{"line":134,"column":1},"end":{"line":138,"column":3}},"range":[3863,3957],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"99":{"loc":{"start":{"line":117,"column":29},"end":{"line":139,"column":1}},"range":[3481,3959],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"100":{"loc":{"start":{"line":117,"column":0},"end":{"line":139,"column":1}},"range":[3452,3959],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"101":{"loc":{"start":{"line":177,"column":15},"end":{"line":177,"column":16}},"range":[5367,5368],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"102":{"loc":{"start":{"line":177,"column":26},"end":{"line":177,"column":27}},"range":[5378,5379],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"103":{"loc":{"start":{"line":178,"column":33},"end":{"line":178,"column":42}},"range":[5414,5423],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"104":{"loc":{"start":{"line":178,"column":13},"end":{"line":178,"column":43}},"range":[5394,5424],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"105":{"loc":{"start":{"line":180,"column":23},"end":{"line":180,"column":32}},"range":[5465,5474],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"106":{"loc":{"start":{"line":180,"column":34},"end":{"line":180,"column":37}},"range":[5476,5479],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"107":{"loc":{"start":{"line":182,"column":6},"end":{"line":182,"column":19}},"range":[5515,5528],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"108":{"loc":{"start":{"line":183,"column":6},"end":{"line":183,"column":25}},"range":[5536,5555],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"109":{"loc":{"start":{"line":184,"column":6},"end":{"line":184,"column":29}},"range":[5563,5586],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"110":{"loc":{"start":{"line":181,"column":26},"end":{"line":185,"column":6}},"range":[5507,5593],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"111":{"loc":{"start":{"line":181,"column":12},"end":{"line":185,"column":7}},"range":[5493,5594],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression"],"type":"CallExpression","line":false},"112":{"loc":{"start":{"line":180,"column":3},"end":{"line":185,"column":8}},"range":[5445,5595],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"113":{"loc":{"start":{"line":190,"column":6},"end":{"line":190,"column":19}},"range":[5681,5694],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"114":{"loc":{"start":{"line":191,"column":6},"end":{"line":191,"column":25}},"range":[5702,5721],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"115":{"loc":{"start":{"line":192,"column":6},"end":{"line":192,"column":29}},"range":[5729,5752],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"116":{"loc":{"start":{"line":194,"column":14},"end":{"line":194,"column":29}},"range":[5774,5789],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"117":{"loc":{"start":{"line":189,"column":11},"end":{"line":194,"column":30}},"range":[5672,5790],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"118":{"loc":{"start":{"line":189,"column":3},"end":{"line":194,"column":31}},"range":[5664,5791],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"119":{"loc":{"start":{"line":188,"column":21},"end":{"line":195,"column":3}},"range":[5638,5795],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"120":{"loc":{"start":{"line":188,"column":2},"end":{"line":196,"column":9}},"range":[5619,5806],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"121":{"loc":{"start":{"line":198,"column":9},"end":{"line":198,"column":27}},"range":[5818,5836],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","MemberExpression"],"type":"MemberExpression","line":false},"122":{"loc":{"start":{"line":198,"column":1},"end":{"line":198,"column":27}},"range":[5810,5836],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"123":{"loc":{"start":{"line":201,"column":11},"end":{"line":201,"column":26}},"range":[5874,5889],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"124":{"loc":{"start":{"line":203,"column":11},"end":{"line":203,"column":16}},"range":[5943,5948],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"125":{"loc":{"start":{"line":203,"column":19},"end":{"line":203,"column":27}},"range":[5951,5959],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"126":{"loc":{"start":{"line":203,"column":28},"end":{"line":203,"column":38}},"range":[5960,5970],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"127":{"loc":{"start":{"line":203,"column":19},"end":{"line":203,"column":39}},"range":[5951,5971],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression"],"type":"CallExpression","line":false},"128":{"loc":{"start":{"line":203,"column":11},"end":{"line":203,"column":39}},"range":[5943,5971],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"129":{"loc":{"start":{"line":203,"column":4},"end":{"line":203,"column":40}},"range":[5936,5972],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"130":{"loc":{"start":{"line":202,"column":11},"end":{"line":204,"column":4}},"range":[5902,5977],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"131":{"loc":{"start":{"line":204,"column":6},"end":{"line":204,"column":7}},"range":[5979,5980],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Literal"],"type":"Literal","line":false},"132":{"loc":{"start":{"line":200,"column":2},"end":{"line":204,"column":8}},"range":[5851,5981],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"133":{"loc":{"start":{"line":199,"column":1},"end":{"line":204,"column":8}},"range":[5839,5981],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"134":{"loc":{"start":{"line":207,"column":11},"end":{"line":207,"column":18}},"range":[6005,6012],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","Identifier"],"type":"Identifier","line":false},"135":{"loc":{"start":{"line":207,"column":22},"end":{"line":207,"column":33}},"range":[6016,6027],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"136":{"loc":{"start":{"line":207,"column":36},"end":{"line":207,"column":41}},"range":[6030,6035],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"137":{"loc":{"start":{"line":207,"column":22},"end":{"line":207,"column":41}},"range":[6016,6035],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"138":{"loc":{"start":{"line":207,"column":11},"end":{"line":207,"column":42}},"range":[6005,6036],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression"],"type":"BinaryExpression","line":false},"139":{"loc":{"start":{"line":208,"column":9},"end":{"line":208,"column":20}},"range":[6047,6058],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"140":{"loc":{"start":{"line":209,"column":30},"end":{"line":209,"column":37}},"range":[6090,6097],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"141":{"loc":{"start":{"line":209,"column":39},"end":{"line":209,"column":44}},"range":[6099,6104],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"142":{"loc":{"start":{"line":209,"column":14},"end":{"line":209,"column":45}},"range":[6074,6105],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"143":{"loc":{"start":{"line":206,"column":8},"end":{"line":210,"column":2}},"range":[5992,6108],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"144":{"loc":{"start":{"line":177,"column":1},"end":{"line":185,"column":9}},"range":[5353,5596],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"145":{"loc":{"start":{"line":187,"column":1},"end":{"line":196,"column":10}},"range":[5599,5807],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"146":{"loc":{"start":{"line":198,"column":1},"end":{"line":198,"column":28}},"range":[5810,5837],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"147":{"loc":{"start":{"line":199,"column":1},"end":{"line":204,"column":9}},"range":[5839,5982],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"148":{"loc":{"start":{"line":206,"column":1},"end":{"line":210,"column":3}},"range":[5985,6109],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"149":{"loc":{"start":{"line":176,"column":28},"end":{"line":211,"column":1}},"range":[5325,6111],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"150":{"loc":{"start":{"line":176,"column":0},"end":{"line":211,"column":1}},"range":[5297,6111],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"151":{"loc":{"start":{"line":249,"column":15},"end":{"line":249,"column":16}},"range":[7541,7542],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"152":{"loc":{"start":{"line":249,"column":26},"end":{"line":249,"column":27}},"range":[7552,7553],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"153":{"loc":{"start":{"line":250,"column":33},"end":{"line":250,"column":42}},"range":[7588,7597],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"154":{"loc":{"start":{"line":250,"column":13},"end":{"line":250,"column":43}},"range":[7568,7598],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"155":{"loc":{"start":{"line":252,"column":23},"end":{"line":252,"column":32}},"range":[7639,7648],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"156":{"loc":{"start":{"line":252,"column":34},"end":{"line":252,"column":37}},"range":[7650,7653],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"157":{"loc":{"start":{"line":254,"column":6},"end":{"line":254,"column":27}},"range":[7689,7710],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"158":{"loc":{"start":{"line":255,"column":6},"end":{"line":255,"column":26}},"range":[7718,7738],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"159":{"loc":{"start":{"line":253,"column":26},"end":{"line":256,"column":6}},"range":[7681,7745],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"160":{"loc":{"start":{"line":253,"column":12},"end":{"line":256,"column":7}},"range":[7667,7746],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","CallExpression"],"type":"CallExpression","line":false},"161":{"loc":{"start":{"line":252,"column":3},"end":{"line":256,"column":8}},"range":[7619,7747],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"162":{"loc":{"start":{"line":261,"column":6},"end":{"line":261,"column":27}},"range":[7833,7854],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"163":{"loc":{"start":{"line":262,"column":6},"end":{"line":262,"column":26}},"range":[7862,7882],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression","CallExpression","MemberExpression","ArrayExpression","Literal"],"type":"Literal","line":false},"164":{"loc":{"start":{"line":264,"column":14},"end":{"line":264,"column":29}},"range":[7904,7919],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"165":{"loc":{"start":{"line":260,"column":11},"end":{"line":264,"column":30}},"range":[7824,7920],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","UnaryExpression"],"type":"UnaryExpression","line":false},"166":{"loc":{"start":{"line":260,"column":3},"end":{"line":264,"column":31}},"range":[7816,7921],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"167":{"loc":{"start":{"line":259,"column":21},"end":{"line":265,"column":3}},"range":[7790,7925],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"168":{"loc":{"start":{"line":259,"column":2},"end":{"line":266,"column":9}},"range":[7771,7936],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"169":{"loc":{"start":{"line":268,"column":9},"end":{"line":268,"column":27}},"range":[7948,7966],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","MemberExpression"],"type":"MemberExpression","line":false},"170":{"loc":{"start":{"line":268,"column":1},"end":{"line":268,"column":27}},"range":[7940,7966],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"171":{"loc":{"start":{"line":272,"column":11},"end":{"line":272,"column":26}},"range":[8005,8020],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"172":{"loc":{"start":{"line":274,"column":11},"end":{"line":274,"column":16}},"range":[8068,8073],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"173":{"loc":{"start":{"line":274,"column":19},"end":{"line":274,"column":27}},"range":[8076,8084],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"174":{"loc":{"start":{"line":274,"column":28},"end":{"line":274,"column":32}},"range":[8085,8089],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"175":{"loc":{"start":{"line":274,"column":19},"end":{"line":274,"column":33}},"range":[8076,8090],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","CallExpression"],"type":"CallExpression","line":false},"176":{"loc":{"start":{"line":274,"column":11},"end":{"line":274,"column":33}},"range":[8068,8090],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"177":{"loc":{"start":{"line":274,"column":4},"end":{"line":274,"column":34}},"range":[8061,8091],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"178":{"loc":{"start":{"line":273,"column":11},"end":{"line":275,"column":4}},"range":[8033,8096],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"179":{"loc":{"start":{"line":275,"column":6},"end":{"line":275,"column":7}},"range":[8098,8099],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression","Literal"],"type":"Literal","line":false},"180":{"loc":{"start":{"line":271,"column":2},"end":{"line":275,"column":8}},"range":[7982,8100],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"181":{"loc":{"start":{"line":270,"column":1},"end":{"line":275,"column":8}},"range":[7970,8100],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"182":{"loc":{"start":{"line":278,"column":11},"end":{"line":278,"column":18}},"range":[8124,8131],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","Identifier"],"type":"Identifier","line":false},"183":{"loc":{"start":{"line":278,"column":23},"end":{"line":278,"column":34}},"range":[8136,8147],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"184":{"loc":{"start":{"line":278,"column":37},"end":{"line":278,"column":42}},"range":[8150,8155],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"185":{"loc":{"start":{"line":278,"column":23},"end":{"line":278,"column":42}},"range":[8136,8155],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"186":{"loc":{"start":{"line":278,"column":11},"end":{"line":278,"column":43}},"range":[8124,8156],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression"],"type":"BinaryExpression","line":false},"187":{"loc":{"start":{"line":279,"column":9},"end":{"line":279,"column":20}},"range":[8167,8178],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","Identifier"],"type":"Identifier","line":false},"188":{"loc":{"start":{"line":280,"column":30},"end":{"line":280,"column":37}},"range":[8210,8217],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"189":{"loc":{"start":{"line":280,"column":39},"end":{"line":280,"column":44}},"range":[8219,8224],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"190":{"loc":{"start":{"line":280,"column":14},"end":{"line":280,"column":45}},"range":[8194,8225],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"191":{"loc":{"start":{"line":277,"column":8},"end":{"line":281,"column":2}},"range":[8111,8228],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"192":{"loc":{"start":{"line":249,"column":1},"end":{"line":256,"column":9}},"range":[7527,7748],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"193":{"loc":{"start":{"line":258,"column":1},"end":{"line":266,"column":10}},"range":[7751,7937],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"194":{"loc":{"start":{"line":268,"column":1},"end":{"line":268,"column":28}},"range":[7940,7967],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"195":{"loc":{"start":{"line":270,"column":1},"end":{"line":275,"column":9}},"range":[7970,8101],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"196":{"loc":{"start":{"line":277,"column":1},"end":{"line":281,"column":3}},"range":[8104,8229],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"197":{"loc":{"start":{"line":248,"column":27},"end":{"line":282,"column":1}},"range":[7499,8231],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"198":{"loc":{"start":{"line":248,"column":0},"end":{"line":282,"column":1}},"range":[7472,8231],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"199":{"loc":{"start":{"line":1,"column":0},"end":{"line":1,"column":13}},"range":[0,13],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"200":{"loc":{"start":{"line":3,"column":0},"end":{"line":3,"column":30}},"range":[15,45],"results":[],"stack":["Program","VariableDeclaration"],"type":"VariableDeclaration","line":true},"201":{"loc":{"start":{"line":27,"column":0},"end":{"line":39,"column":2}},"range":[765,1150],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"202":{"loc":{"start":{"line":66,"column":0},"end":{"line":90,"column":2}},"range":[1999,2591],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"203":{"loc":{"start":{"line":117,"column":0},"end":{"line":139,"column":2}},"range":[3452,3960],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"204":{"loc":{"start":{"line":176,"column":0},"end":{"line":211,"column":2}},"range":[5297,6112],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"205":{"loc":{"start":{"line":248,"column":0},"end":{"line":282,"column":2}},"range":[7472,8232],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true}},"comments":[[48,764],[1152,1998],[2593,3451],[3962,5296],[6114,7471]]};
}
function instrumentor_record(filetag, id, expression) {
	var scope = global || window;
	scope.st_timeCommenced = scope.st_timeCommenced || Date.now();
	scope.st_iterator = (scope.st_iterator || 0);

	var depth =
		(new Error()).stack.split("\n")
			.slice(1)
			.length;

	if (!scope.mem)
		scope.mem = (
			typeof process !== "undefined" ? process.memoryUsage : function(){}
		);

	__instrumentor_map__[filetag].instruments[id].results.push({
		"depth": depth,
		"time": Date.now(),
		"timeOffset": Date.now() - scope.st_timeCommenced,
		"memoryUsage": scope.mem(),
		"invocation": scope.st_iterator,
		"milestone": scope.__steamshovel_milestone,
		"value": scope.__steamshovel_record_expression ? expression : null
	});

	scope.st_iterator ++;

	// Make sure we return the original expresison result
	return expression;
}
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 199);
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 1, 'use strict');
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 200);
var util = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 4, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 2, require)(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 3, './utils')));
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 201);
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 27, module.exports = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 26, function generateStatistics(instrumentData, key) {
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 23);
    var self = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 5, module.exports);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 24);
    if (instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 6, !instrumentData))
        throw instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 9, new (instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 7, Error))(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 8, 'Input was undefined.')));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 25);
    return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 22, {
        'lines': instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 12, self.lines(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 10, instrumentData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 11, key))),
        'conditions': instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 15, self.conditions(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 13, instrumentData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 14, key))),
        'expressions': instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 18, self.expressions(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 16, instrumentData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 17, key))),
        'functions': instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 21, self.functions(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 19, instrumentData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 20, key)))
    });
}));
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 202);
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 65, module.exports.lines = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 64, function (instrumentData, key) {
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 58);
    var totalInstruments, coveredInstruments, covered, total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 28, 0), getValue = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 30, util.calculateValue(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 29, instrumentData)));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 59);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 36, totalInstruments = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 35, util.getInstruments(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 31, instrumentData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 32, key)).filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 34, util.withkey(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 33, 'line'))))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 60);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 38, total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 37, totalInstruments.length));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 61);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 41, coveredInstruments = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 40, totalInstruments.filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 39, util.hasResults))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 62);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 51, covered = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 50, coveredInstruments.reduce(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 48, function (count, instrument) {
        instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 47);
        return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 46, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 42, count) + instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 45, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 43, getValue)(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 44, instrument))));
    }), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 49, 0))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 63);
    return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 57, {
        covered: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 52, covered),
        total: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 53, total),
        percentage: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 56, util.percentage(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 54, covered), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 55, total)))
    });
}));
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 203);
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 100, module.exports.expressions = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 99, function (inputData, key) {
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 95);
    var covered = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 66, 0), total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 67, 0), getValue = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 69, util.calculateValue(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 68, inputData))), instruments = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 75, util.getInstruments(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 70, inputData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 71, key)).filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 74, function (item) {
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 73);
            return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 72, !item.line);
        })));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 96);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 77, total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 76, instruments.length));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 97);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 88, covered = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 87, instruments.filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 78, util.hasResults)).reduce(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 85, function (count, item) {
        instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 84);
        return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 83, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 79, count) + instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 82, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 80, getValue)(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 81, item))));
    }), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 86, 0))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 98);
    return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 94, {
        covered: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 89, covered),
        total: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 90, total),
        percentage: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 93, util.percentage(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 91, covered), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 92, total)))
    });
}));
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 204);
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 150, module.exports.conditions = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 149, function (inputData, key) {
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 144);
    var covered = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 101, 0), total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 102, 0), getValue = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 104, util.calculateValue(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 103, inputData))), instruments = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 112, util.getInstruments(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 105, inputData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 106, key)).filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 111, util.isOfType(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 110, [
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 107, 'IfStatement'),
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 108, 'LogicalExpression'),
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 109, 'ConditionalExpression')
        ])))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 145);
    var strictTotal = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 120, instruments.filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 119, function (instrument) {
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 118);
            return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 117, ~[
                instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 113, 'IfStatement'),
                instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 114, 'LogicalExpression'),
                instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 115, 'ConditionalExpression')
            ].indexOf(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 116, instrument.type)));
        })).length);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 146);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 122, total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 121, instruments.length));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 147);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 133, covered = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 132, instruments.filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 123, util.hasResults)).reduce(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 130, function (count, instrument) {
        instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 129);
        return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 128, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 124, count) + instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 127, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 125, getValue)(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 126, instrument))));
    }), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 131, 0))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 148);
    return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 143, {
        covered: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 138, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 134, covered) * instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 137, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 135, strictTotal) / instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 136, total))),
        total: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 139, strictTotal),
        percentage: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 142, util.percentage(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 140, covered), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 141, total)))
    });
}));
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 205);
instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 198, module.exports.functions = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 197, function (inputData, key) {
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 192);
    var covered = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 151, 0), total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 152, 0), getValue = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 154, util.calculateValue(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 153, inputData))), instruments = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 161, util.getInstruments(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 155, inputData), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 156, key)).filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 160, util.isOfType(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 159, [
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 157, 'FunctionDeclaration'),
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 158, 'FunctionExpression')
        ])))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 193);
    var strictTotal = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 168, instruments.filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 167, function (instrument) {
            instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 166);
            return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 165, ~[
                instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 162, 'FunctionDeclaration'),
                instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 163, 'FunctionExpression')
            ].indexOf(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 164, instrument.type)));
        })).length);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 194);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 170, total = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 169, instruments.length));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 195);
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 181, covered = instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 180, instruments.filter(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 171, util.hasResults)).reduce(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 178, function (count, item) {
        instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 177);
        return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 176, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 172, count) + instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 175, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 173, getValue)(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 174, item))));
    }), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 179, 0))));
    instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 196);
    return instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 191, {
        covered: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 186, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 182, covered) * instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 185, instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 183, strictTotal) / instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 184, total))),
        total: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 187, strictTotal),
        percentage: instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 190, util.percentage(instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 188, covered), instrumentor_record('i4283e89b3baf979028afccb2c82cfaf3', 189, total)))
    });
}));