if (typeof __instrumentor_map__ === 'undefined') {
__instrumentor_map__ = {};
}
if (typeof __instrumentor_map__.ieec4706ee2dbc9bb8da0c5853f3cf7ad === 'undefined') {
__instrumentor_map__.ieec4706ee2dbc9bb8da0c5853f3cf7ad = {"filetag":"ieec4706ee2dbc9bb8da0c5853f3cf7ad","filename":"lib/stats/utils.js","source":"// Utilities for generating statistics\n\nvar util = module.exports = {};\n\n/*\n\tPublic: Very lame memoiser function. Only looks at the first argument.\n\n\tfn\t\t\t\t-\tThe function to memoise.\n\n\tReturns a wrapped function\n\n\tExamples\n\n\t\tmyFunction = util.cache(function myFunction() { ... });\n\n*/\nutil.functionCache = {};\nutil.cache = function cache(fn) {\n\tutil.functionCache[fn.name] = {};\n\n\treturn function(key) {\n\t\tif (typeof key === \"object\") {\n\t\t\tif (key[\"__cache_\" + fn.name])\n\t\t\t\treturn key[\"__cache_\" + fn.name];\n\n\t\t\tObject.defineProperty(key, \"__cache_\" + fn.name, {\n\t\t\t\tvalue: fn.apply(util, arguments),\n\t\t\t\tenumerable: false\n\t\t\t});\n\n\t\t\treturn key[\"__cache_\" + fn.name];\n\t\t}\n\n\t\tif (typeof key !== \"object\" &&\n\t\t\tutil.functionCache[fn.name][key])\n\t\t\t\treturn util.functionCache[fn.name][key];\n\n\t\treturn (\n\t\t\tutil.functionCache[fn.name][key] =\n\t\t\t\tfn.apply(util, arguments)\n\t\t);\n\t};\n};\n\n/*\n\tPublic: For use in mapping. Returns a function that returns the specified\n\tkey from its input data.\n\n\tkey\t\t\t\t-\tThe key to return.\n\n\tReturns the function that performs the property access.\n\n\tExamples\n\n\t\t// Returns all the data properties from the objects in the array\n\t\tvar arr = arr.map(util.withkey(\"data\"));\n\n*/\nutil.withkey = util.cache(function withkey(key) {\n\treturn function(data) {\n\t\treturn data[key];\n\t};\n});\n\n/*\n\tPublic: For use in filtering. Returns a function that returns true for\n\tinstruments of the type (AST Node type) specified in the wrapper function.\n\n\ttype\t\t\t-\tThe AST Node type which will trigger a truthy return\n\n\tReturns true or false\n\n\tExamples\n\n\t\tvar varDecs = instruments.map(util.isOfType(\"VariableDeclaration\"));\n\n*/\nutil.isOfType = util.cache(function isOfType(types) {\n\ttypes = types instanceof Array ? types : [types];\n\n\treturn function(item) {\n\t\treturn types.reduce(function(acc, cur) {\n\t\t\treturn acc || ~item.stack.indexOf(cur);\n\t\t}, false);\n\t};\n});\n\n/*\n\tPublic: For use in filtering. Returns true for a given instrument which has\n\tresults.\n\n\tinstrument\t\t-\tAn instrument to test\n\n\tReturns true if the instrument has results, false if not.\n\n\tExamples\n\n\t\tvar instrumentsWithResults = instruments.map(util.hasResults);\n\n*/\nutil.hasResults = function hasResults(instrument) {\n\treturn (instrument && instrument.results && instrument.results.length);\n};\n\n/*\n\tPublic: Returns rounded percentage for the percent a of b.\n\n\tReturns a number representing a percentage.\n\n*/\nutil.percentage = function percentage(a, b) {\n\treturn (+((a/b)*10000)|0) / 100;\n};\n\n/*\n\tPublic: Gets the instrument map from the raw instrumentation data.\n\n\tinstrumentData\t-\tThe complete, raw instrument Data\n\tfiletag\t\t\t-\tOnly return instruments matching this filetag\n\n\tReturns an array of instruments.\n\n\tExamples\n\n\t\tvar instruments = util.getInstruments(instrumentData);\n\n*/\nutil.getInstruments = function getInstruments(instrumentData, filetag) {\n\tif (instrumentData instanceof Array)\n\t\treturn instrumentData;\n\n\treturn Object.keys(instrumentData).reduce(function(acc, cur) {\n\t\tvar instruments = instrumentData[cur].instruments;\n\n\t\tif (filetag && cur !== filetag) return acc;\n\n\t\treturn acc.concat(Object.keys(instruments).map(function(key) {\n\t\t\tinstruments[key].id = key;\n\t\t\tinstruments[key].filetag = cur;\n\t\t\treturn instruments[key];\n\t\t}));\n\t}, []);\n};\n\n/*\n\tPublic: Returns the stack depth of the shallowest call across all\n\tinstruments in the instrument map\n\n\tinstrumentMap\t-\tThe complete instrument map\n\tfiletag\t\t\t-\tOnly measure the depth of instruments matching this\n\t\t\t\t\t\tfiletag\n\n\tReturns a function which calculates the value of a given instrument.\n\n\tExamples\n\n\t\tvar shallowestCallDepth = util.shallowestCall(instrumentMap);\n\n*/\nutil.shallowestCall = function shallowestCall(instrumentData, key) {\n\treturn Math.min.apply(Math,\n\t\tutil.getInstruments(instrumentData, key)\n\t\t\t.filter(util.hasResults)\n\t\t\t.map(function(item) {\n\t\t\t\treturn Math.min.apply(Math,\n\t\t\t\t\titem.results.map(util.withkey(\"depth\")));\n\t\t\t})\n\t);\n};\n\n/*\n\tPublic: Given the entire instrument map, returns a function which calculates\n\tthe 'coverage' value of a given instrument.\n\n\tinstrumentMap\t-\tThe complete instrument map\n\n\tReturns a function which calculates the value of a given instrument.\n\n\tExamples\n\n\t\tvar getValue = util.calculateValue(instrumentMap);\n\t\tvar value = getValue(instrument);\n\n*/\nutil.calculateValue = function calculateValue(instrumentMap) {\n\tvar shallowMark = util.shallowestCall(instrumentMap),\n\t\tgraceThreshold = 5,\n\t\tinverseDampingFactor = 1.25;\n\n\treturn function(item) {\n\t\treturn (\n\t\t\tMath.max.apply(Math,\n\t\t\t\titem.results\n\t\t\t\t\t.map(util.withkey(\"depth\"))\n\t\t\t\t\t.map(util.calculate.bind(\tnull,\n\t\t\t\t\t\t\t\t\t\t\t\tshallowMark,\n\t\t\t\t\t\t\t\t\t\t\t\tgraceThreshold,\n\t\t\t\t\t\t\t\t\t\t\t\tinverseDampingFactor ))));\n\t};\n};\n\n/*\n\tPublic: Given a shallowMark, graceThreshold, and depth, this function runs\n\tthe per-node calculation responsible for delivering the coverage value.\n\n\tshallowMark\t-\tThe shallowest measured call in the test suite. This is an\n\t\t\t\t\tapproximation of the actual call stack depth of the test,\n\t\t\t\t\tand is really a balancing term. The true depth of a call as\n\t\t\t\t\tfar as steamshovel is concerned is 'depth - shallowMark' to\n\t\t\t\t\tdetermine the relative depth.\n\tgrace\t\t-\tThe relative depth threshold below which the score of a\n\t\t\t\t\tgiven result will not be decreased.\n\tdamping\t\t-\tA fitting parameter by which the calculation is made.\n\t\t\t\t\tSteamShovel uses a default of 1.25. This value is purely\n\t\t\t\t\tone that seemed to produce 'fair' results in testing.\n\tdepth\t\t-\tThe depth of the invocation.\n\n\tReturns a number representing the score of the given instrument. This score\n\tis derived from the straightforward equation:\n\n\t\tscore = 1 / ((damping ^ relativeDepth) / damping)\n\n\trelativeDepth is calculated by subtracting the shallowMark and then the\n\tgraceThreshold values from the input depth, and taking the result, or 1 —\n\twhichever is larger.\n\n\tExamples\n\n\t\tnodeValue = calculate(5, 5, 1.25, 15);\n\n*/\nutil.calculate = function calculate(\tshallowMark,\n\t\t\t\t\t\t\t\t\t\tgraceThreshold,\n\t\t\t\t\t\t\t\t\t\tinverseDampingFactor,\n\t\t\t\t\t\t\t\t\t\tdepth\n\t\t\t\t\t\t\t\t\t) {\n\n\t// Calculate inverse logarithm\n\tvar relativeDepth = (depth - shallowMark) + 1;\n\t\trelativeDepth = relativeDepth - graceThreshold;\n\t\trelativeDepth = relativeDepth <= 1 ? 1 : relativeDepth;\n\n\treturn 1 / (\n\t\tMath.pow(inverseDampingFactor, relativeDepth) /\n\t\t\tinverseDampingFactor);\n};\n\n/*\n\tPublic: Given the entire instrument map, burns memory usage information\n\tcalculated from the probe data back into the instrument map.\n\n\tThis function is poorly realised and is likely to change.\n\n\tinstrumentMap\t-\tThe complete instrument map\n\n\tReturns the mutated instrument map.\n\n\tExamples\n\n\t\tvar instruments = util.generateMemoryStats(instrumentMap);\n\n*/\nutil.generateMemoryStats = function generateMemoryStats(inputData) {\n\tvar defaults\t\t= { rss: 0, heapTotal: 0, heapUsed: 0 },\n\t\tinstruments\t\t= util.getInstruments(inputData),\n\t\titerationMap\t= util.generateIterationMap(inputData);\n\n\tfunction mapper(result) {\n\t\tvar prev = result.invocation > 0 ? result.invocation - 1 : 0;\n\t\t\tprev = iterationMap[prev].memoryUsage;\n\n\t\treturn {\n\t\t\trss: result.memoryUsage.rss - prev.rss,\n\t\t\theapTotal: result.memoryUsage.heapTotal - prev.heapTotal,\n\t\t\theapUsed: result.memoryUsage.heapUsed - prev.heapUsed,\n\t\t};\n\t}\n\n\tfunction reducer(acc, cur, idx, array) {\n\t\tvar factor = 1 / array.length;\n\t\tacc.rss\t\t\t+= cur.rss * factor;\n\t\tacc.heapTotal\t+= cur.heapTotal * factor;\n\t\tacc.heapUsed\t+= cur.heapUsed * factor;\n\t\treturn acc;\n\t}\n\n\tfor (var i = 0; i < instruments.length; i++)\n\t\tinstruments[i].avgMemChanges =\n\t\t\tinstruments[i]\n\t\t\t\t.results.map(mapper)\n\t\t\t\t.reduce(reducer, defaults);\n\n\treturn instruments;\n};\n\n/*\n\tPublic: Given the entire instrument map, return a list of probe calls with\n\ttheir associated data.\n\n\tinstrumentMap\t-\tThe complete instrument map\n\tfiletag\t\t\t-\tScope to instruments matching this filetag\n\n\tReturns a list of invocations (probe calls) with associated data.\n\n\tExamples\n\n\t\tvar iterations = util.generateIterationMap(instrumentMap);\n\n*/\nutil.generateIterationMap = function generateIterationMap(instrumentMap, key) {\n\treturn (\n\t\tutil.getInstruments(instrumentMap, key)\n\t\t\t.reduce(function(acc, cur) {\n\t\t\t\treturn acc.concat(cur.results.map(function(result) {\n\t\t\t\t\treturn {\n\t\t\t\t\t\tinvocation:\t\tresult.invocation,\n\t\t\t\t\t\tkey:\t\t\tcur.key,\n\t\t\t\t\t\tloc:\t\t\tcur.loc.start,\n\t\t\t\t\t\tkind:\t\t\tcur.stack[cur.stack.length-1],\n\t\t\t\t\t\tdepth:\t\t\tresult.depth,\n\t\t\t\t\t\ttime:\t\t\tresult.time,\n\t\t\t\t\t\ttimeOffset:\t\tresult.timeOffset,\n\t\t\t\t\t\tmemoryUsage:\tresult.memoryUsage,\n\t\t\t\t\t\tmilestone:\t\tresult.milestone,\n\t\t\t\t\t\tisLine:\t\t\t!!cur.line\n\t\t\t\t\t};\n\t\t\t\t}));\n\t\t\t}, [])\n\t\t\t.sort(function(a, b) {\n\t\t\t\treturn a.invocation - b.invocation;\n\t\t\t})\n\t);\n};\n\n\n\n// var util = module.exports = {};\n// util.getMinimumDepth = function getMinimumDepth(item) {\n//\t return item.minDepth || Math.min.apply(Math, item.results.map(util.depth));\n// };\n//\n// util.depth = function depth(result) {\n//\t return result.depth;\n// };\n//\n//\n","instruments":{"1":{"loc":{"start":{"line":3,"column":28},"end":{"line":3,"column":30}},"range":[68,70],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","AssignmentExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"2":{"loc":{"start":{"line":3,"column":11},"end":{"line":3,"column":30}},"range":[51,70],"results":[],"stack":["Program","VariableDeclaration","VariableDeclarator","AssignmentExpression"],"type":"AssignmentExpression","line":false},"3":{"loc":{"start":{"line":17,"column":21},"end":{"line":17,"column":23}},"range":[307,309],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"4":{"loc":{"start":{"line":17,"column":0},"end":{"line":17,"column":23}},"range":[286,309],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"5":{"loc":{"start":{"line":19,"column":31},"end":{"line":19,"column":33}},"range":[376,378],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"6":{"loc":{"start":{"line":19,"column":1},"end":{"line":19,"column":33}},"range":[346,378],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"7":{"loc":{"start":{"line":22,"column":6},"end":{"line":22,"column":16}},"range":[411,421],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"8":{"loc":{"start":{"line":22,"column":21},"end":{"line":22,"column":29}},"range":[426,434],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Literal"],"type":"Literal","line":false},"9":{"loc":{"start":{"line":22,"column":6},"end":{"line":22,"column":29}},"range":[411,434],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"10":{"loc":{"start":{"line":23,"column":11},"end":{"line":23,"column":21}},"range":[449,459],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","MemberExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"11":{"loc":{"start":{"line":23,"column":24},"end":{"line":23,"column":31}},"range":[462,469],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","MemberExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"12":{"loc":{"start":{"line":23,"column":7},"end":{"line":23,"column":32}},"range":[445,470],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","MemberExpression"],"type":"MemberExpression","line":false},"13":{"loc":{"start":{"line":24,"column":15},"end":{"line":24,"column":25}},"range":[487,497],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ReturnStatement","MemberExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"14":{"loc":{"start":{"line":24,"column":28},"end":{"line":24,"column":35}},"range":[500,507],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ReturnStatement","MemberExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"15":{"loc":{"start":{"line":24,"column":11},"end":{"line":24,"column":36}},"range":[483,508],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement","ReturnStatement","MemberExpression"],"type":"MemberExpression","line":false},"16":{"loc":{"start":{"line":26,"column":25},"end":{"line":26,"column":28}},"range":[536,539],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"17":{"loc":{"start":{"line":26,"column":30},"end":{"line":26,"column":40}},"range":[541,551],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"18":{"loc":{"start":{"line":26,"column":43},"end":{"line":26,"column":50}},"range":[554,561],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"19":{"loc":{"start":{"line":26,"column":30},"end":{"line":26,"column":50}},"range":[541,561],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"20":{"loc":{"start":{"line":27,"column":20},"end":{"line":27,"column":24}},"range":[585,589],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"21":{"loc":{"start":{"line":27,"column":26},"end":{"line":27,"column":35}},"range":[591,600],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","CallExpression","Identifier"],"type":"Identifier","line":false},"22":{"loc":{"start":{"line":27,"column":11},"end":{"line":27,"column":36}},"range":[576,601],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","CallExpression"],"type":"CallExpression","line":false},"23":{"loc":{"start":{"line":28,"column":16},"end":{"line":28,"column":21}},"range":[619,624],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"24":{"loc":{"start":{"line":26,"column":52},"end":{"line":29,"column":4}},"range":[563,629],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression","ObjectExpression"],"type":"ObjectExpression","line":false},"25":{"loc":{"start":{"line":26,"column":3},"end":{"line":29,"column":5}},"range":[514,630],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement","CallExpression"],"type":"CallExpression","line":false},"26":{"loc":{"start":{"line":31,"column":14},"end":{"line":31,"column":24}},"range":[647,657],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","MemberExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"27":{"loc":{"start":{"line":31,"column":27},"end":{"line":31,"column":34}},"range":[660,667],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","MemberExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"28":{"loc":{"start":{"line":31,"column":10},"end":{"line":31,"column":35}},"range":[643,668],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement","MemberExpression"],"type":"MemberExpression","line":false},"29":{"loc":{"start":{"line":23,"column":3},"end":{"line":24,"column":37}},"range":[441,509],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"30":{"loc":{"start":{"line":26,"column":3},"end":{"line":29,"column":6}},"range":[514,631],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"31":{"loc":{"start":{"line":31,"column":3},"end":{"line":31,"column":36}},"range":[636,669],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"32":{"loc":{"start":{"line":34,"column":6},"end":{"line":34,"column":16}},"range":[681,691],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","BinaryExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"33":{"loc":{"start":{"line":34,"column":21},"end":{"line":34,"column":29}},"range":[696,704],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"34":{"loc":{"start":{"line":34,"column":6},"end":{"line":34,"column":29}},"range":[681,704],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"35":{"loc":{"start":{"line":35,"column":3},"end":{"line":35,"column":35}},"range":[711,743],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","MemberExpression"],"type":"MemberExpression","line":false},"36":{"loc":{"start":{"line":34,"column":6},"end":{"line":35,"column":35}},"range":[681,743],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"37":{"loc":{"start":{"line":36,"column":11},"end":{"line":36,"column":43}},"range":[756,788],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","MemberExpression"],"type":"MemberExpression","line":false},"38":{"loc":{"start":{"line":40,"column":13},"end":{"line":40,"column":17}},"range":[853,857],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"39":{"loc":{"start":{"line":40,"column":19},"end":{"line":40,"column":28}},"range":[859,868],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"40":{"loc":{"start":{"line":40,"column":4},"end":{"line":40,"column":29}},"range":[844,869],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"41":{"loc":{"start":{"line":39,"column":3},"end":{"line":40,"column":29}},"range":[805,869],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"42":{"loc":{"start":{"line":22,"column":2},"end":{"line":32,"column":3}},"range":[407,673],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"43":{"loc":{"start":{"line":34,"column":2},"end":{"line":36,"column":44}},"range":[677,789],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"44":{"loc":{"start":{"line":38,"column":2},"end":{"line":41,"column":4}},"range":[793,874],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"45":{"loc":{"start":{"line":21,"column":8},"end":{"line":42,"column":2}},"range":[389,877],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression"],"type":"FunctionExpression","line":false},"46":{"loc":{"start":{"line":19,"column":1},"end":{"line":19,"column":34}},"range":[346,379],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"47":{"loc":{"start":{"line":21,"column":1},"end":{"line":42,"column":3}},"range":[382,878],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"48":{"loc":{"start":{"line":18,"column":13},"end":{"line":43,"column":1}},"range":[324,880],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"49":{"loc":{"start":{"line":18,"column":0},"end":{"line":43,"column":1}},"range":[311,880],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"50":{"loc":{"start":{"line":61,"column":9},"end":{"line":61,"column":18}},"range":[1285,1294],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","MemberExpression"],"type":"MemberExpression","line":false},"51":{"loc":{"start":{"line":61,"column":2},"end":{"line":61,"column":19}},"range":[1278,1295],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"52":{"loc":{"start":{"line":60,"column":8},"end":{"line":62,"column":2}},"range":[1259,1298],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression"],"type":"FunctionExpression","line":false},"53":{"loc":{"start":{"line":60,"column":1},"end":{"line":62,"column":3}},"range":[1252,1299],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"54":{"loc":{"start":{"line":59,"column":26},"end":{"line":63,"column":1}},"range":[1227,1301],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"55":{"loc":{"start":{"line":59,"column":15},"end":{"line":63,"column":2}},"range":[1216,1302],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"56":{"loc":{"start":{"line":59,"column":0},"end":{"line":63,"column":2}},"range":[1201,1302],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"57":{"loc":{"start":{"line":79,"column":9},"end":{"line":79,"column":14}},"range":[1694,1699],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"58":{"loc":{"start":{"line":79,"column":26},"end":{"line":79,"column":31}},"range":[1711,1716],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"59":{"loc":{"start":{"line":79,"column":9},"end":{"line":79,"column":31}},"range":[1694,1716],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"60":{"loc":{"start":{"line":79,"column":34},"end":{"line":79,"column":39}},"range":[1719,1724],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Identifier"],"type":"Identifier","line":false},"61":{"loc":{"start":{"line":79,"column":43},"end":{"line":79,"column":48}},"range":[1728,1733],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","ArrayExpression","Identifier"],"type":"Identifier","line":false},"62":{"loc":{"start":{"line":79,"column":42},"end":{"line":79,"column":49}},"range":[1727,1734],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"63":{"loc":{"start":{"line":79,"column":9},"end":{"line":79,"column":49}},"range":[1694,1734],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression"],"type":"ConditionalExpression","line":false},"64":{"loc":{"start":{"line":79,"column":1},"end":{"line":79,"column":49}},"range":[1686,1734],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"65":{"loc":{"start":{"line":83,"column":10},"end":{"line":83,"column":13}},"range":[1815,1818],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","Identifier"],"type":"Identifier","line":false},"66":{"loc":{"start":{"line":83,"column":37},"end":{"line":83,"column":40}},"range":[1842,1845],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","UnaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"67":{"loc":{"start":{"line":83,"column":17},"end":{"line":83,"column":41}},"range":[1822,1846],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"68":{"loc":{"start":{"line":83,"column":10},"end":{"line":83,"column":41}},"range":[1815,1846],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"69":{"loc":{"start":{"line":83,"column":3},"end":{"line":83,"column":42}},"range":[1808,1847],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"70":{"loc":{"start":{"line":82,"column":22},"end":{"line":84,"column":3}},"range":[1784,1851],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"71":{"loc":{"start":{"line":84,"column":5},"end":{"line":84,"column":10}},"range":[1853,1858],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Literal"],"type":"Literal","line":false},"72":{"loc":{"start":{"line":82,"column":9},"end":{"line":84,"column":11}},"range":[1771,1859],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"73":{"loc":{"start":{"line":82,"column":2},"end":{"line":84,"column":12}},"range":[1764,1860],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"74":{"loc":{"start":{"line":81,"column":8},"end":{"line":85,"column":2}},"range":[1745,1863],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression"],"type":"FunctionExpression","line":false},"75":{"loc":{"start":{"line":79,"column":1},"end":{"line":79,"column":50}},"range":[1686,1735],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"76":{"loc":{"start":{"line":81,"column":1},"end":{"line":85,"column":3}},"range":[1738,1864],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"77":{"loc":{"start":{"line":78,"column":27},"end":{"line":86,"column":1}},"range":[1658,1866],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"78":{"loc":{"start":{"line":78,"column":16},"end":{"line":86,"column":2}},"range":[1647,1867],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"79":{"loc":{"start":{"line":78,"column":0},"end":{"line":86,"column":2}},"range":[1631,1867],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"80":{"loc":{"start":{"line":102,"column":9},"end":{"line":102,"column":19}},"range":[2200,2210],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","LogicalExpression","Identifier"],"type":"Identifier","line":false},"81":{"loc":{"start":{"line":102,"column":23},"end":{"line":102,"column":41}},"range":[2214,2232],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","LogicalExpression","MemberExpression"],"type":"MemberExpression","line":false},"82":{"loc":{"start":{"line":102,"column":9},"end":{"line":102,"column":41}},"range":[2200,2232],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","LogicalExpression"],"type":"LogicalExpression","line":false},"83":{"loc":{"start":{"line":102,"column":45},"end":{"line":102,"column":70}},"range":[2236,2261],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression","MemberExpression"],"type":"MemberExpression","line":false},"84":{"loc":{"start":{"line":102,"column":9},"end":{"line":102,"column":70}},"range":[2200,2261],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"85":{"loc":{"start":{"line":102,"column":1},"end":{"line":102,"column":72}},"range":[2192,2263],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"86":{"loc":{"start":{"line":101,"column":18},"end":{"line":103,"column":1}},"range":[2157,2265],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"87":{"loc":{"start":{"line":101,"column":0},"end":{"line":103,"column":1}},"range":[2139,2265],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"88":{"loc":{"start":{"line":112,"column":12},"end":{"line":112,"column":13}},"range":[2439,2440],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","UnaryExpression","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"89":{"loc":{"start":{"line":112,"column":14},"end":{"line":112,"column":15}},"range":[2441,2442],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","UnaryExpression","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"90":{"loc":{"start":{"line":112,"column":12},"end":{"line":112,"column":15}},"range":[2439,2442],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","UnaryExpression","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"91":{"loc":{"start":{"line":112,"column":17},"end":{"line":112,"column":22}},"range":[2444,2449],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","UnaryExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"92":{"loc":{"start":{"line":112,"column":9},"end":{"line":112,"column":23}},"range":[2436,2450],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","UnaryExpression"],"type":"UnaryExpression","line":false},"93":{"loc":{"start":{"line":112,"column":24},"end":{"line":112,"column":25}},"range":[2451,2452],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"94":{"loc":{"start":{"line":112,"column":9},"end":{"line":112,"column":25}},"range":[2436,2452],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"95":{"loc":{"start":{"line":112,"column":29},"end":{"line":112,"column":32}},"range":[2456,2459],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","Literal"],"type":"Literal","line":false},"96":{"loc":{"start":{"line":112,"column":8},"end":{"line":112,"column":32}},"range":[2435,2459],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"97":{"loc":{"start":{"line":112,"column":1},"end":{"line":112,"column":33}},"range":[2428,2460],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"98":{"loc":{"start":{"line":111,"column":18},"end":{"line":113,"column":1}},"range":[2399,2462],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"99":{"loc":{"start":{"line":111,"column":0},"end":{"line":113,"column":1}},"range":[2381,2462],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"100":{"loc":{"start":{"line":129,"column":5},"end":{"line":129,"column":19}},"range":[2834,2848],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"101":{"loc":{"start":{"line":129,"column":31},"end":{"line":129,"column":36}},"range":[2860,2865],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"102":{"loc":{"start":{"line":129,"column":5},"end":{"line":129,"column":36}},"range":[2834,2865],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"103":{"loc":{"start":{"line":130,"column":9},"end":{"line":130,"column":23}},"range":[2876,2890],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"104":{"loc":{"start":{"line":132,"column":20},"end":{"line":132,"column":34}},"range":[2913,2927],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"105":{"loc":{"start":{"line":133,"column":20},"end":{"line":133,"column":51}},"range":[2977,3008],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","MemberExpression"],"type":"MemberExpression","line":false},"106":{"loc":{"start":{"line":135,"column":6},"end":{"line":135,"column":13}},"range":[3017,3024],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","Identifier"],"type":"Identifier","line":false},"107":{"loc":{"start":{"line":135,"column":17},"end":{"line":135,"column":20}},"range":[3028,3031],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"108":{"loc":{"start":{"line":135,"column":25},"end":{"line":135,"column":32}},"range":[3036,3043],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"109":{"loc":{"start":{"line":135,"column":17},"end":{"line":135,"column":32}},"range":[3028,3043],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"110":{"loc":{"start":{"line":135,"column":6},"end":{"line":135,"column":32}},"range":[3017,3043],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","LogicalExpression"],"type":"LogicalExpression","line":false},"111":{"loc":{"start":{"line":135,"column":41},"end":{"line":135,"column":44}},"range":[3052,3055],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"112":{"loc":{"start":{"line":137,"column":32},"end":{"line":137,"column":43}},"range":[3090,3101],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"113":{"loc":{"start":{"line":138,"column":25},"end":{"line":138,"column":28}},"range":[3148,3151],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","Identifier"],"type":"Identifier","line":false},"114":{"loc":{"start":{"line":138,"column":3},"end":{"line":138,"column":28}},"range":[3126,3151],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"115":{"loc":{"start":{"line":139,"column":30},"end":{"line":139,"column":33}},"range":[3183,3186],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","Identifier"],"type":"Identifier","line":false},"116":{"loc":{"start":{"line":139,"column":3},"end":{"line":139,"column":33}},"range":[3156,3186],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"117":{"loc":{"start":{"line":140,"column":10},"end":{"line":140,"column":26}},"range":[3198,3214],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","MemberExpression"],"type":"MemberExpression","line":false},"118":{"loc":{"start":{"line":138,"column":3},"end":{"line":138,"column":29}},"range":[3126,3152],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"119":{"loc":{"start":{"line":139,"column":3},"end":{"line":139,"column":34}},"range":[3156,3187],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"120":{"loc":{"start":{"line":140,"column":3},"end":{"line":140,"column":27}},"range":[3191,3215],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"121":{"loc":{"start":{"line":137,"column":49},"end":{"line":141,"column":3}},"range":[3107,3219],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"122":{"loc":{"start":{"line":137,"column":20},"end":{"line":141,"column":4}},"range":[3078,3220],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression"],"type":"CallExpression","line":false},"123":{"loc":{"start":{"line":137,"column":9},"end":{"line":141,"column":5}},"range":[3067,3221],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"124":{"loc":{"start":{"line":133,"column":2},"end":{"line":133,"column":52}},"range":[2959,3009],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"125":{"loc":{"start":{"line":135,"column":2},"end":{"line":135,"column":45}},"range":[3013,3056],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"126":{"loc":{"start":{"line":137,"column":2},"end":{"line":141,"column":6}},"range":[3060,3222],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"127":{"loc":{"start":{"line":132,"column":43},"end":{"line":142,"column":2}},"range":[2936,3225],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"128":{"loc":{"start":{"line":142,"column":4},"end":{"line":142,"column":6}},"range":[3227,3229],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"129":{"loc":{"start":{"line":132,"column":8},"end":{"line":142,"column":7}},"range":[2901,3230],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"130":{"loc":{"start":{"line":129,"column":1},"end":{"line":130,"column":24}},"range":[2830,2891],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","IfStatement"],"type":"IfStatement","line":true},"131":{"loc":{"start":{"line":132,"column":1},"end":{"line":142,"column":8}},"range":[2894,3231],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"132":{"loc":{"start":{"line":128,"column":22},"end":{"line":143,"column":1}},"range":[2778,3233],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"133":{"loc":{"start":{"line":128,"column":0},"end":{"line":143,"column":1}},"range":[2756,3233],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"134":{"loc":{"start":{"line":161,"column":23},"end":{"line":161,"column":27}},"range":[3709,3713],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"135":{"loc":{"start":{"line":162,"column":22},"end":{"line":162,"column":36}},"range":[3737,3751],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","MemberExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"136":{"loc":{"start":{"line":162,"column":38},"end":{"line":162,"column":41}},"range":[3753,3756],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","MemberExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"137":{"loc":{"start":{"line":163,"column":11},"end":{"line":163,"column":26}},"range":[3769,3784],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","MemberExpression","CallExpression","MemberExpression"],"type":"MemberExpression","line":false},"138":{"loc":{"start":{"line":165,"column":26},"end":{"line":165,"column":30}},"range":[3837,3841],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"139":{"loc":{"start":{"line":166,"column":35},"end":{"line":166,"column":42}},"range":[3878,3885],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Literal"],"type":"Literal","line":false},"140":{"loc":{"start":{"line":166,"column":22},"end":{"line":166,"column":43}},"range":[3865,3886],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression"],"type":"CallExpression","line":false},"141":{"loc":{"start":{"line":166,"column":5},"end":{"line":166,"column":44}},"range":[3848,3887],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression"],"type":"CallExpression","line":false},"142":{"loc":{"start":{"line":165,"column":11},"end":{"line":166,"column":45}},"range":[3822,3888],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"143":{"loc":{"start":{"line":165,"column":4},"end":{"line":166,"column":46}},"range":[3815,3889],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"144":{"loc":{"start":{"line":164,"column":8},"end":{"line":167,"column":4}},"range":[3794,3894],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"145":{"loc":{"start":{"line":162,"column":2},"end":{"line":167,"column":5}},"range":[3717,3895],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression"],"type":"CallExpression","line":false},"146":{"loc":{"start":{"line":161,"column":8},"end":{"line":168,"column":2}},"range":[3694,3898],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"147":{"loc":{"start":{"line":161,"column":1},"end":{"line":168,"column":3}},"range":[3687,3899],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"148":{"loc":{"start":{"line":160,"column":22},"end":{"line":169,"column":1}},"range":[3639,3901],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"149":{"loc":{"start":{"line":160,"column":0},"end":{"line":169,"column":1}},"range":[3617,3901],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"150":{"loc":{"start":{"line":186,"column":39},"end":{"line":186,"column":52}},"range":[4354,4367],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"151":{"loc":{"start":{"line":186,"column":19},"end":{"line":186,"column":53}},"range":[4334,4368],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"152":{"loc":{"start":{"line":187,"column":19},"end":{"line":187,"column":20}},"range":[4389,4390],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"153":{"loc":{"start":{"line":188,"column":25},"end":{"line":188,"column":29}},"range":[4417,4421],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"154":{"loc":{"start":{"line":192,"column":18},"end":{"line":192,"column":22}},"range":[4478,4482],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","Identifier"],"type":"Identifier","line":false},"155":{"loc":{"start":{"line":194,"column":23},"end":{"line":194,"column":30}},"range":[4524,4531],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","MemberExpression","CallExpression","CallExpression","Literal"],"type":"Literal","line":false},"156":{"loc":{"start":{"line":194,"column":10},"end":{"line":194,"column":31}},"range":[4511,4532],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","MemberExpression","CallExpression","CallExpression"],"type":"CallExpression","line":false},"157":{"loc":{"start":{"line":195,"column":31},"end":{"line":195,"column":35}},"range":[4565,4569],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Literal"],"type":"Literal","line":false},"158":{"loc":{"start":{"line":196,"column":12},"end":{"line":196,"column":23}},"range":[4583,4594],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"159":{"loc":{"start":{"line":197,"column":12},"end":{"line":197,"column":26}},"range":[4608,4622],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"160":{"loc":{"start":{"line":198,"column":12},"end":{"line":198,"column":32}},"range":[4636,4656],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"161":{"loc":{"start":{"line":195,"column":10},"end":{"line":198,"column":34}},"range":[4544,4658],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","CallExpression"],"type":"CallExpression","line":false},"162":{"loc":{"start":{"line":193,"column":4},"end":{"line":198,"column":35}},"range":[4488,4659],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression"],"type":"CallExpression","line":false},"163":{"loc":{"start":{"line":192,"column":3},"end":{"line":198,"column":36}},"range":[4463,4660],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"164":{"loc":{"start":{"line":191,"column":2},"end":{"line":198,"column":38}},"range":[4451,4662],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"165":{"loc":{"start":{"line":190,"column":8},"end":{"line":199,"column":2}},"range":[4432,4665],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","FunctionExpression"],"type":"FunctionExpression","line":false},"166":{"loc":{"start":{"line":186,"column":1},"end":{"line":188,"column":30}},"range":[4316,4422],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"167":{"loc":{"start":{"line":190,"column":1},"end":{"line":199,"column":3}},"range":[4425,4666],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"168":{"loc":{"start":{"line":185,"column":22},"end":{"line":200,"column":1}},"range":[4274,4668],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"169":{"loc":{"start":{"line":185,"column":0},"end":{"line":200,"column":1}},"range":[4252,4668],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"170":{"loc":{"start":{"line":239,"column":22},"end":{"line":239,"column":27}},"range":[6057,6062],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"171":{"loc":{"start":{"line":239,"column":30},"end":{"line":239,"column":41}},"range":[6065,6076],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"172":{"loc":{"start":{"line":239,"column":22},"end":{"line":239,"column":41}},"range":[6057,6076],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"173":{"loc":{"start":{"line":239,"column":45},"end":{"line":239,"column":46}},"range":[6080,6081],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression","Literal"],"type":"Literal","line":false},"174":{"loc":{"start":{"line":239,"column":21},"end":{"line":239,"column":46}},"range":[6056,6081],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression"],"type":"BinaryExpression","line":false},"175":{"loc":{"start":{"line":240,"column":18},"end":{"line":240,"column":31}},"range":[6101,6114],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"176":{"loc":{"start":{"line":240,"column":34},"end":{"line":240,"column":48}},"range":[6117,6131],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"177":{"loc":{"start":{"line":240,"column":18},"end":{"line":240,"column":48}},"range":[6101,6131],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"178":{"loc":{"start":{"line":240,"column":2},"end":{"line":240,"column":48}},"range":[6085,6131],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"179":{"loc":{"start":{"line":241,"column":18},"end":{"line":241,"column":31}},"range":[6151,6164],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"180":{"loc":{"start":{"line":241,"column":35},"end":{"line":241,"column":36}},"range":[6168,6169],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"181":{"loc":{"start":{"line":241,"column":18},"end":{"line":241,"column":36}},"range":[6151,6169],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"182":{"loc":{"start":{"line":241,"column":39},"end":{"line":241,"column":40}},"range":[6172,6173],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Literal"],"type":"Literal","line":false},"183":{"loc":{"start":{"line":241,"column":43},"end":{"line":241,"column":56}},"range":[6176,6189],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression","Identifier"],"type":"Identifier","line":false},"184":{"loc":{"start":{"line":241,"column":18},"end":{"line":241,"column":56}},"range":[6151,6189],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression","ConditionalExpression"],"type":"ConditionalExpression","line":false},"185":{"loc":{"start":{"line":241,"column":2},"end":{"line":241,"column":56}},"range":[6135,6189],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"186":{"loc":{"start":{"line":243,"column":8},"end":{"line":243,"column":9}},"range":[6200,6201],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","Literal"],"type":"Literal","line":false},"187":{"loc":{"start":{"line":244,"column":11},"end":{"line":244,"column":31}},"range":[6217,6237],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"188":{"loc":{"start":{"line":244,"column":33},"end":{"line":244,"column":46}},"range":[6239,6252],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"189":{"loc":{"start":{"line":244,"column":2},"end":{"line":244,"column":47}},"range":[6208,6253],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","CallExpression"],"type":"CallExpression","line":false},"190":{"loc":{"start":{"line":245,"column":3},"end":{"line":245,"column":23}},"range":[6259,6279],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"191":{"loc":{"start":{"line":244,"column":2},"end":{"line":245,"column":23}},"range":[6208,6279],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"192":{"loc":{"start":{"line":243,"column":8},"end":{"line":245,"column":24}},"range":[6200,6280],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"193":{"loc":{"start":{"line":239,"column":1},"end":{"line":239,"column":47}},"range":[6036,6082],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"194":{"loc":{"start":{"line":240,"column":2},"end":{"line":240,"column":49}},"range":[6085,6132],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"195":{"loc":{"start":{"line":241,"column":2},"end":{"line":241,"column":57}},"range":[6135,6190],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"196":{"loc":{"start":{"line":243,"column":1},"end":{"line":245,"column":25}},"range":[6193,6281],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"197":{"loc":{"start":{"line":232,"column":17},"end":{"line":246,"column":1}},"range":[5882,6283],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"198":{"loc":{"start":{"line":232,"column":0},"end":{"line":246,"column":1}},"range":[5865,6283],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"199":{"loc":{"start":{"line":264,"column":24},"end":{"line":264,"column":25}},"range":[6738,6739],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"200":{"loc":{"start":{"line":264,"column":38},"end":{"line":264,"column":39}},"range":[6752,6753],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"201":{"loc":{"start":{"line":264,"column":51},"end":{"line":264,"column":52}},"range":[6765,6766],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression","Property","Literal"],"type":"Literal","line":false},"202":{"loc":{"start":{"line":264,"column":17},"end":{"line":264,"column":54}},"range":[6731,6768],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","ObjectExpression"],"type":"ObjectExpression","line":false},"203":{"loc":{"start":{"line":265,"column":37},"end":{"line":265,"column":46}},"range":[6807,6816],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"204":{"loc":{"start":{"line":265,"column":17},"end":{"line":265,"column":47}},"range":[6787,6817],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"205":{"loc":{"start":{"line":266,"column":43},"end":{"line":266,"column":52}},"range":[6862,6871],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression","Identifier"],"type":"Identifier","line":false},"206":{"loc":{"start":{"line":266,"column":17},"end":{"line":266,"column":53}},"range":[6836,6872],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration","VariableDeclarator","CallExpression"],"type":"CallExpression","line":false},"207":{"loc":{"start":{"line":269,"column":13},"end":{"line":269,"column":30}},"range":[6915,6932],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"208":{"loc":{"start":{"line":269,"column":33},"end":{"line":269,"column":34}},"range":[6935,6936],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"209":{"loc":{"start":{"line":269,"column":13},"end":{"line":269,"column":34}},"range":[6915,6936],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"210":{"loc":{"start":{"line":269,"column":37},"end":{"line":269,"column":54}},"range":[6939,6956],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"211":{"loc":{"start":{"line":269,"column":57},"end":{"line":269,"column":58}},"range":[6959,6960],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"212":{"loc":{"start":{"line":269,"column":37},"end":{"line":269,"column":58}},"range":[6939,6960],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"213":{"loc":{"start":{"line":269,"column":61},"end":{"line":269,"column":62}},"range":[6963,6964],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression","Literal"],"type":"Literal","line":false},"214":{"loc":{"start":{"line":269,"column":13},"end":{"line":269,"column":62}},"range":[6915,6964],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","ConditionalExpression"],"type":"ConditionalExpression","line":false},"215":{"loc":{"start":{"line":270,"column":10},"end":{"line":270,"column":40}},"range":[6976,7006],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","MemberExpression"],"type":"MemberExpression","line":false},"216":{"loc":{"start":{"line":270,"column":3},"end":{"line":270,"column":40}},"range":[6969,7006],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"217":{"loc":{"start":{"line":273,"column":8},"end":{"line":273,"column":30}},"range":[7028,7050],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"218":{"loc":{"start":{"line":273,"column":33},"end":{"line":273,"column":41}},"range":[7053,7061],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"219":{"loc":{"start":{"line":273,"column":8},"end":{"line":273,"column":41}},"range":[7028,7061],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression"],"type":"BinaryExpression","line":false},"220":{"loc":{"start":{"line":274,"column":14},"end":{"line":274,"column":42}},"range":[7077,7105],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"221":{"loc":{"start":{"line":274,"column":45},"end":{"line":274,"column":59}},"range":[7108,7122],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"222":{"loc":{"start":{"line":274,"column":14},"end":{"line":274,"column":59}},"range":[7077,7122],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression"],"type":"BinaryExpression","line":false},"223":{"loc":{"start":{"line":275,"column":13},"end":{"line":275,"column":40}},"range":[7137,7164],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"224":{"loc":{"start":{"line":275,"column":43},"end":{"line":275,"column":56}},"range":[7167,7180],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"225":{"loc":{"start":{"line":275,"column":13},"end":{"line":275,"column":56}},"range":[7137,7180],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression","Property","BinaryExpression"],"type":"BinaryExpression","line":false},"226":{"loc":{"start":{"line":272,"column":9},"end":{"line":276,"column":3}},"range":[7018,7185],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"227":{"loc":{"start":{"line":269,"column":2},"end":{"line":269,"column":63}},"range":[6904,6965],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"228":{"loc":{"start":{"line":270,"column":3},"end":{"line":270,"column":41}},"range":[6969,7007],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"229":{"loc":{"start":{"line":272,"column":2},"end":{"line":276,"column":4}},"range":[7011,7186],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"230":{"loc":{"start":{"line":280,"column":15},"end":{"line":280,"column":16}},"range":[7248,7249],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression","Literal"],"type":"Literal","line":false},"231":{"loc":{"start":{"line":280,"column":19},"end":{"line":280,"column":31}},"range":[7252,7264],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"232":{"loc":{"start":{"line":280,"column":15},"end":{"line":280,"column":31}},"range":[7248,7264],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration","VariableDeclarator","BinaryExpression"],"type":"BinaryExpression","line":false},"233":{"loc":{"start":{"line":281,"column":15},"end":{"line":281,"column":22}},"range":[7281,7288],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"234":{"loc":{"start":{"line":281,"column":25},"end":{"line":281,"column":31}},"range":[7291,7297],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"235":{"loc":{"start":{"line":281,"column":15},"end":{"line":281,"column":31}},"range":[7281,7297],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"236":{"loc":{"start":{"line":281,"column":2},"end":{"line":281,"column":31}},"range":[7268,7297],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"237":{"loc":{"start":{"line":282,"column":19},"end":{"line":282,"column":32}},"range":[7318,7331],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"238":{"loc":{"start":{"line":282,"column":35},"end":{"line":282,"column":41}},"range":[7334,7340],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"239":{"loc":{"start":{"line":282,"column":19},"end":{"line":282,"column":41}},"range":[7318,7340],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"240":{"loc":{"start":{"line":282,"column":2},"end":{"line":282,"column":41}},"range":[7301,7340],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"241":{"loc":{"start":{"line":283,"column":18},"end":{"line":283,"column":30}},"range":[7360,7372],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"242":{"loc":{"start":{"line":283,"column":33},"end":{"line":283,"column":39}},"range":[7375,7381],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression","Identifier"],"type":"Identifier","line":false},"243":{"loc":{"start":{"line":283,"column":18},"end":{"line":283,"column":39}},"range":[7360,7381],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression","BinaryExpression"],"type":"BinaryExpression","line":false},"244":{"loc":{"start":{"line":283,"column":2},"end":{"line":283,"column":39}},"range":[7344,7381],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"245":{"loc":{"start":{"line":284,"column":9},"end":{"line":284,"column":12}},"range":[7392,7395],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"246":{"loc":{"start":{"line":280,"column":2},"end":{"line":280,"column":32}},"range":[7235,7265],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"247":{"loc":{"start":{"line":281,"column":2},"end":{"line":281,"column":32}},"range":[7268,7298],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"248":{"loc":{"start":{"line":282,"column":2},"end":{"line":282,"column":42}},"range":[7301,7341],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"249":{"loc":{"start":{"line":283,"column":2},"end":{"line":283,"column":40}},"range":[7344,7382],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ExpressionStatement"],"type":"ExpressionStatement","line":true},"250":{"loc":{"start":{"line":284,"column":2},"end":{"line":284,"column":13}},"range":[7385,7396],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"251":{"loc":{"start":{"line":287,"column":14},"end":{"line":287,"column":15}},"range":[7415,7416],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","VariableDeclaration","VariableDeclarator","Literal"],"type":"Literal","line":false},"252":{"loc":{"start":{"line":287,"column":17},"end":{"line":287,"column":18}},"range":[7418,7419],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","BinaryExpression","Identifier"],"type":"Identifier","line":false},"253":{"loc":{"start":{"line":287,"column":21},"end":{"line":287,"column":39}},"range":[7422,7440],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"254":{"loc":{"start":{"line":287,"column":17},"end":{"line":287,"column":39}},"range":[7418,7440],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"255":{"loc":{"start":{"line":287,"column":41},"end":{"line":287,"column":44}},"range":[7442,7445],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","UpdateExpression"],"type":"UpdateExpression","line":false},"256":{"loc":{"start":{"line":290,"column":17},"end":{"line":290,"column":23}},"range":[7515,7521],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","ExpressionStatement","AssignmentExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"257":{"loc":{"start":{"line":291,"column":12},"end":{"line":291,"column":19}},"range":[7535,7542],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"258":{"loc":{"start":{"line":291,"column":21},"end":{"line":291,"column":29}},"range":[7544,7552],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","ExpressionStatement","AssignmentExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"259":{"loc":{"start":{"line":289,"column":3},"end":{"line":291,"column":30}},"range":[7483,7553],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","ExpressionStatement","AssignmentExpression","CallExpression"],"type":"CallExpression","line":false},"260":{"loc":{"start":{"line":288,"column":2},"end":{"line":291,"column":30}},"range":[7449,7553],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"261":{"loc":{"start":{"line":293,"column":8},"end":{"line":293,"column":19}},"range":[7564,7575],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","Identifier"],"type":"Identifier","line":false},"262":{"loc":{"start":{"line":264,"column":1},"end":{"line":266,"column":54}},"range":[6715,6873],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","VariableDeclaration"],"type":"VariableDeclaration","line":true},"263":{"loc":{"start":{"line":268,"column":1},"end":{"line":277,"column":2}},"range":[6876,7189],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration"],"type":"FunctionDeclaration","line":true},"264":{"loc":{"start":{"line":279,"column":1},"end":{"line":285,"column":2}},"range":[7192,7399],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","FunctionDeclaration"],"type":"FunctionDeclaration","line":true},"265":{"loc":{"start":{"line":287,"column":1},"end":{"line":291,"column":31}},"range":[7402,7554],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ForStatement"],"type":"ForStatement","line":true},"266":{"loc":{"start":{"line":293,"column":1},"end":{"line":293,"column":20}},"range":[7557,7576],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"267":{"loc":{"start":{"line":263,"column":27},"end":{"line":294,"column":1}},"range":[6672,7578],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"268":{"loc":{"start":{"line":263,"column":0},"end":{"line":294,"column":1}},"range":[6645,7578],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"269":{"loc":{"start":{"line":312,"column":22},"end":{"line":312,"column":35}},"range":[8043,8056],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"270":{"loc":{"start":{"line":312,"column":37},"end":{"line":312,"column":40}},"range":[8058,8061],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","MemberExpression","CallExpression","Identifier"],"type":"Identifier","line":false},"271":{"loc":{"start":{"line":316,"column":19},"end":{"line":316,"column":36}},"range":[8185,8202],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"272":{"loc":{"start":{"line":317,"column":13},"end":{"line":317,"column":20}},"range":[8217,8224],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"273":{"loc":{"start":{"line":318,"column":13},"end":{"line":318,"column":26}},"range":[8239,8252],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"274":{"loc":{"start":{"line":319,"column":24},"end":{"line":319,"column":40}},"range":[8278,8294],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"275":{"loc":{"start":{"line":319,"column":41},"end":{"line":319,"column":42}},"range":[8295,8296],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression","BinaryExpression","Literal"],"type":"Literal","line":false},"276":{"loc":{"start":{"line":319,"column":14},"end":{"line":319,"column":43}},"range":[8268,8297],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"277":{"loc":{"start":{"line":320,"column":15},"end":{"line":320,"column":27}},"range":[8314,8326],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"278":{"loc":{"start":{"line":321,"column":14},"end":{"line":321,"column":25}},"range":[8342,8353],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"279":{"loc":{"start":{"line":322,"column":19},"end":{"line":322,"column":36}},"range":[8374,8391],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"280":{"loc":{"start":{"line":323,"column":19},"end":{"line":323,"column":37}},"range":[8412,8430],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"281":{"loc":{"start":{"line":324,"column":18},"end":{"line":324,"column":34}},"range":[8450,8466],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","MemberExpression"],"type":"MemberExpression","line":false},"282":{"loc":{"start":{"line":325,"column":16},"end":{"line":325,"column":26}},"range":[8484,8494],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression","Property","UnaryExpression"],"type":"UnaryExpression","line":false},"283":{"loc":{"start":{"line":315,"column":12},"end":{"line":326,"column":6}},"range":[8164,8501],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","ObjectExpression"],"type":"ObjectExpression","line":false},"284":{"loc":{"start":{"line":315,"column":5},"end":{"line":326,"column":7}},"range":[8157,8502],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"285":{"loc":{"start":{"line":314,"column":38},"end":{"line":327,"column":5}},"range":[8133,8508],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"286":{"loc":{"start":{"line":314,"column":22},"end":{"line":327,"column":6}},"range":[8117,8509],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","CallExpression"],"type":"CallExpression","line":false},"287":{"loc":{"start":{"line":314,"column":11},"end":{"line":327,"column":7}},"range":[8106,8510],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"288":{"loc":{"start":{"line":314,"column":4},"end":{"line":327,"column":8}},"range":[8099,8511],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"289":{"loc":{"start":{"line":313,"column":11},"end":{"line":328,"column":4}},"range":[8074,8516],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"290":{"loc":{"start":{"line":328,"column":6},"end":{"line":328,"column":8}},"range":[8518,8520],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","MemberExpression","CallExpression","ArrayExpression"],"type":"ArrayExpression","line":false},"291":{"loc":{"start":{"line":330,"column":11},"end":{"line":330,"column":23}},"range":[8559,8571],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"292":{"loc":{"start":{"line":330,"column":26},"end":{"line":330,"column":38}},"range":[8574,8586],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression","MemberExpression"],"type":"MemberExpression","line":false},"293":{"loc":{"start":{"line":330,"column":11},"end":{"line":330,"column":38}},"range":[8559,8586],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement","BinaryExpression"],"type":"BinaryExpression","line":false},"294":{"loc":{"start":{"line":330,"column":4},"end":{"line":330,"column":39}},"range":[8552,8587],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"295":{"loc":{"start":{"line":329,"column":9},"end":{"line":331,"column":4}},"range":[8531,8592],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"296":{"loc":{"start":{"line":312,"column":2},"end":{"line":331,"column":5}},"range":[8023,8593],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement","CallExpression"],"type":"CallExpression","line":false},"297":{"loc":{"start":{"line":311,"column":1},"end":{"line":332,"column":3}},"range":[8012,8597],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression","BlockStatement","ReturnStatement"],"type":"ReturnStatement","line":true},"298":{"loc":{"start":{"line":310,"column":28},"end":{"line":333,"column":1}},"range":[7959,8599],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression","FunctionExpression"],"type":"FunctionExpression","line":false},"299":{"loc":{"start":{"line":310,"column":0},"end":{"line":333,"column":1}},"range":[7931,8599],"results":[],"stack":["Program","ExpressionStatement","AssignmentExpression"],"type":"AssignmentExpression","line":false},"300":{"loc":{"start":{"line":3,"column":0},"end":{"line":3,"column":31}},"range":[40,71],"results":[],"stack":["Program","VariableDeclaration"],"type":"VariableDeclaration","line":true},"301":{"loc":{"start":{"line":17,"column":0},"end":{"line":17,"column":24}},"range":[286,310],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"302":{"loc":{"start":{"line":18,"column":0},"end":{"line":43,"column":2}},"range":[311,881],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"303":{"loc":{"start":{"line":59,"column":0},"end":{"line":63,"column":3}},"range":[1201,1303],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"304":{"loc":{"start":{"line":78,"column":0},"end":{"line":86,"column":3}},"range":[1631,1868],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"305":{"loc":{"start":{"line":101,"column":0},"end":{"line":103,"column":2}},"range":[2139,2266],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"306":{"loc":{"start":{"line":111,"column":0},"end":{"line":113,"column":2}},"range":[2381,2463],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"307":{"loc":{"start":{"line":128,"column":0},"end":{"line":143,"column":2}},"range":[2756,3234],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"308":{"loc":{"start":{"line":160,"column":0},"end":{"line":169,"column":2}},"range":[3617,3902],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"309":{"loc":{"start":{"line":185,"column":0},"end":{"line":200,"column":2}},"range":[4252,4669],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"310":{"loc":{"start":{"line":232,"column":0},"end":{"line":246,"column":2}},"range":[5865,6284],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"311":{"loc":{"start":{"line":263,"column":0},"end":{"line":294,"column":2}},"range":[6645,7579],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true},"312":{"loc":{"start":{"line":310,"column":0},"end":{"line":333,"column":2}},"range":[7931,8600],"results":[],"stack":["Program","ExpressionStatement"],"type":"ExpressionStatement","line":true}},"comments":[[0,38],[73,285],[883,1200],[1305,1630],[1870,2138],[2268,2380],[2465,2755],[3236,3616],[3904,4251],[4671,5864],[6004,6034],[6286,6644],[7581,7930],[8604,8638],[8639,8697],[8698,8777],[8778,8783],[8784,8786],[8787,8827],[8828,8852],[8853,8858],[8859,8861],[8862,8864]]};
}
function instrumentor_record(filetag, id, expression) {
	var scope = global || window;
	scope.st_timeCommenced = scope.st_timeCommenced || Date.now();
	scope.st_iterator = (scope.st_iterator || 0);

	var depth =
		(new Error()).stack.split("\n")
			.slice(1)
			.length;

	if (!scope.mem)
		scope.mem = (
			typeof process !== "undefined" ? process.memoryUsage : function(){}
		);

	__instrumentor_map__[filetag].instruments[id].results.push({
		"depth": depth,
		"time": Date.now(),
		"timeOffset": Date.now() - scope.st_timeCommenced,
		"memoryUsage": scope.mem(),
		"invocation": scope.st_iterator,
		"milestone": scope.__steamshovel_milestone,
		"value": scope.__steamshovel_record_expression ? expression : null
	});

	scope.st_iterator ++;

	// Make sure we return the original expresison result
	return expression;
}
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 300);
var util = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 2, module.exports = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 1, {}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 301);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 4, util.functionCache = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 3, {}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 302);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 49, util.cache = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 48, function cache(fn) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 46);
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 6, util.functionCache[fn.name] = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 5, {}));
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 47);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 45, function (key) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 42);
        if (instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 9, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 7, typeof key) === instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 8, 'object'))) {
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 29);
            if (instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 12, key[instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 10, '__cache_') + instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 11, fn.name)]))
                return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 15, key[instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 13, '__cache_') + instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 14, fn.name)]);
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 30);
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 25, Object.defineProperty(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 16, key), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 19, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 17, '__cache_') + instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 18, fn.name)), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 24, {
                value: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 22, fn.apply(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 20, util), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 21, arguments))),
                enumerable: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 23, false)
            })));
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 31);
            return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 28, key[instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 26, '__cache_') + instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 27, fn.name)]);
        }
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 43);
        if (instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 36, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 34, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 32, typeof key) !== instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 33, 'object')) && instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 35, util.functionCache[fn.name][key])))
            return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 37, util.functionCache[fn.name][key]);
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 44);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 41, util.functionCache[fn.name][key] = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 40, fn.apply(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 38, util), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 39, arguments))));
    });
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 303);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 56, util.withkey = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 55, util.cache(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 54, function withkey(key) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 53);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 52, function (data) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 51);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 50, data[key]);
    });
}))));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 304);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 79, util.isOfType = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 78, util.cache(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 77, function isOfType(types) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 75);
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 64, types = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 63, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 59, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 57, types) instanceof instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 58, Array)) ? instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 60, types) : instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 62, [instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 61, types)])));
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 76);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 74, function (item) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 73);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 72, types.reduce(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 70, function (acc, cur) {
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 69);
            return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 68, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 65, acc) || instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 67, ~item.stack.indexOf(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 66, cur))));
        }), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 71, false)));
    });
}))));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 305);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 87, util.hasResults = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 86, function hasResults(instrument) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 85);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 84, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 82, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 80, instrument) && instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 81, instrument.results)) && instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 83, instrument.results.length));
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 306);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 99, util.percentage = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 98, function percentage(a, b) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 97);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 96, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 94, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 92, +(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 90, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 88, a) / instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 89, b)) * instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 91, 10000))) | instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 93, 0)) / instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 95, 100));
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 307);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 133, util.getInstruments = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 132, function getInstruments(instrumentData, filetag) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 130);
    if (instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 102, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 100, instrumentData) instanceof instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 101, Array)))
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 103, instrumentData);
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 131);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 129, Object.keys(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 104, instrumentData)).reduce(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 127, function (acc, cur) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 124);
        var instruments = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 105, instrumentData[cur].instruments);
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 125);
        if (instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 110, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 106, filetag) && instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 109, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 107, cur) !== instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 108, filetag))))
            return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 111, acc);
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 126);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 123, acc.concat(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 122, Object.keys(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 112, instruments)).map(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 121, function (key) {
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 118);
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 114, instruments[key].id = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 113, key));
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 119);
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 116, instruments[key].filetag = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 115, cur));
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 120);
            return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 117, instruments[key]);
        })))));
    }), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 128, [])));
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 308);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 149, util.shallowestCall = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 148, function shallowestCall(instrumentData, key) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 147);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 146, Math.min.apply(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 134, Math), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 145, util.getInstruments(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 135, instrumentData), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 136, key)).filter(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 137, util.hasResults)).map(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 144, function (item) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 143);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 142, Math.min.apply(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 138, Math), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 141, item.results.map(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 140, util.withkey(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 139, 'depth')))))));
    })))));
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 309);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 169, util.calculateValue = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 168, function calculateValue(instrumentMap) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 166);
    var shallowMark = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 151, util.shallowestCall(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 150, instrumentMap))), graceThreshold = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 152, 5), inverseDampingFactor = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 153, 1.25);
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 167);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 165, function (item) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 164);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 163, Math.max.apply(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 154, Math), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 162, item.results.map(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 156, util.withkey(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 155, 'depth')))).map(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 161, util.calculate.bind(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 157, null), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 158, shallowMark), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 159, graceThreshold), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 160, inverseDampingFactor)))))));
    });
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 310);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 198, util.calculate = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 197, function calculate(shallowMark, graceThreshold, inverseDampingFactor, depth) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 193);
    var relativeDepth = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 174, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 172, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 170, depth) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 171, shallowMark)) + instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 173, 1));
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 194);
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 178, relativeDepth = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 177, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 175, relativeDepth) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 176, graceThreshold)));
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 195);
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 185, relativeDepth = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 184, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 181, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 179, relativeDepth) <= instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 180, 1)) ? instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 182, 1) : instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 183, relativeDepth)));
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 196);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 192, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 186, 1) / instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 191, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 189, Math.pow(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 187, inverseDampingFactor), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 188, relativeDepth))) / instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 190, inverseDampingFactor)));
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 311);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 268, util.generateMemoryStats = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 267, function generateMemoryStats(inputData) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 262);
    var defaults = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 202, {
            rss: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 199, 0),
            heapTotal: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 200, 0),
            heapUsed: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 201, 0)
        }), instruments = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 204, util.getInstruments(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 203, inputData))), iterationMap = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 206, util.generateIterationMap(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 205, inputData)));
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 263);
    function mapper(result) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 227);
        var prev = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 214, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 209, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 207, result.invocation) > instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 208, 0)) ? instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 212, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 210, result.invocation) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 211, 1)) : instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 213, 0));
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 228);
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 216, prev = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 215, iterationMap[prev].memoryUsage));
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 229);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 226, {
            rss: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 219, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 217, result.memoryUsage.rss) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 218, prev.rss)),
            heapTotal: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 222, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 220, result.memoryUsage.heapTotal) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 221, prev.heapTotal)),
            heapUsed: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 225, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 223, result.memoryUsage.heapUsed) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 224, prev.heapUsed))
        });
    }
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 264);
    function reducer(acc, cur, idx, array) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 246);
        var factor = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 232, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 230, 1) / instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 231, array.length));
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 247);
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 236, acc.rss += instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 235, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 233, cur.rss) * instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 234, factor)));
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 248);
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 240, acc.heapTotal += instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 239, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 237, cur.heapTotal) * instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 238, factor)));
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 249);
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 244, acc.heapUsed += instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 243, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 241, cur.heapUsed) * instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 242, factor)));
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 250);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 245, acc);
    }
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 265);
    for (var i = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 251, 0); instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 254, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 252, i) < instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 253, instruments.length)); instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 255, i++))
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 260, instruments[i].avgMemChanges = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 259, instruments[i].results.map(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 256, mapper)).reduce(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 257, reducer), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 258, defaults))));
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 266);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 261, instruments);
}));
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 312);
instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 299, util.generateIterationMap = instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 298, function generateIterationMap(instrumentMap, key) {
    instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 297);
    return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 296, util.getInstruments(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 269, instrumentMap), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 270, key)).reduce(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 289, function (acc, cur) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 288);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 287, acc.concat(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 286, cur.results.map(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 285, function (result) {
            instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 284);
            return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 283, {
                invocation: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 271, result.invocation),
                key: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 272, cur.key),
                loc: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 273, cur.loc.start),
                kind: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 276, cur.stack[instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 274, cur.stack.length) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 275, 1)]),
                depth: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 277, result.depth),
                time: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 278, result.time),
                timeOffset: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 279, result.timeOffset),
                memoryUsage: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 280, result.memoryUsage),
                milestone: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 281, result.milestone),
                isLine: instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 282, !!cur.line)
            });
        })))));
    }), instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 290, [])).sort(instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 295, function (a, b) {
        instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 294);
        return instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 293, instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 291, a.invocation) - instrumentor_record('ieec4706ee2dbc9bb8da0c5853f3cf7ad', 292, b.invocation));
    })));
}));